/***************************************************************************
                          ksqlanalyzer.cpp  -  description
                             -------------------
    begin                : Sun Jan 26 13:05:56 CET 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define DEBUG 1


// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qvbox.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qstring.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>

// application specific includes
#include "ksqlanalyzer.h"
#include "ksqlanalyzerview.h"
#include "ksqlanalyzerdoc.h"

// Buttons
#include "pix/checkBut.xpm"
#include "pix/runBut.xpm"
#include "pix/stopBut.xpm"
#include "pix/textout_pix.xpm"
#include "pix/gridout_pix.xpm"
#include "pix/textout_act_pix.xpm"
#include "pix/gridout_act_pix.xpm"

#ifdef DEBUG
#include <iostream.h>
#endif

KSqlAnalyzerApp::KSqlAnalyzerApp():KMainWindow(0,"KSqlAnalyzer")
{
	config=kapp->config();
	printer = new QPrinter;
	untitledCount=0;
	pDocList = new QList<KSqlAnalyzerDoc>();
	pDocList->setAutoDelete(true);
	tools_enabled = FALSE;

	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initStatusBar();
	initView();
	initActions();

	readOptions();

	///////////////////////////////////////////////////////////////////
	// disable actions at startup
	fileDisconnect->setEnabled(false);
	fileDisconnectAll->setEnabled(false);
	fileNew->setEnabled(false);
	fileOpen->setEnabled(false);
	fileSave->setEnabled(false);
	fileSaveAs->setEnabled(false);
	fileSaveAll->setEnabled(false);
	fileOpenRecent->setEnabled(false);
	fileClose->setEnabled(false);
	filePrint->setEnabled(false);
	editCut->setEnabled(false);
	editCopy->setEnabled(false);
	editSearch->setEnabled(false);
	editReplace->setEnabled(false);
	editPaste->setEnabled(false);
	queryRun->setEnabled(false);
	queryStop->setEnabled(false);
	queryCheck->setEnabled(false);

	startTimer(120);
}

KSqlAnalyzerApp::~KSqlAnalyzerApp()
{
  delete printer;
}

void KSqlAnalyzerApp::enableStdActions(bool ena)
{
	fileDisconnect->setEnabled(ena);
	fileDisconnectAll->setEnabled(ena);
	fileNew->setEnabled(ena);
	fileOpen->setEnabled(ena);
	fileOpenRecent->setEnabled(ena);
	fileClose->setEnabled(ena);
	fileSave->setEnabled(ena);
	fileSaveAs->setEnabled(ena);
	fileSaveAll->setEnabled(ena);
	filePrint->setEnabled(ena);
	queryRun->setEnabled(ena);
	queryStop->setEnabled(ena);
	queryCheck->setEnabled(ena);
	if ( !ena )
		toolBar()->clearCombo(3);
}

void KSqlAnalyzerApp::enableModifiers(bool ena)
{
	editCut->setEnabled(ena);
	editCopy->setEnabled(ena);
	editPaste->setEnabled(ena);
	editSearch->setEnabled(ena);
	editReplace->setEnabled(ena);
}

void KSqlAnalyzerApp::initActions()
{
	fileConnect = new KAction(i18n("Connec&t..."), 0, this, SLOT(slotNewConn()), actionCollection(),"file_connect");
	fileDisconnect = new KAction(i18n("&Disconnect..."), 0, this, SLOT(slotDisConn()), actionCollection(),"file_disconnect");
	fileDisconnectAll = new KAction(i18n("D&isconnect all..."), 0, this, SLOT(slotDisConnAll()), actionCollection(),"file_disconnect_all");

	fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
	fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
	fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
	fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
	fileSaveAll = new KAction(i18n("Sa&ve all Queries"), 0, this, SLOT(slotFileSaveAll()), actionCollection(),"file_save_all");
	filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
	fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
	fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
	fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

	QIconSet cutIcon (SmallIcon("editcut"));

	editCut = new KAction(i18n("Cut"), cutIcon, 0, this, SLOT(slotEditCut()), actionCollection(), "edit_cut");
	editCopy = new KAction(i18n("Copy"), "editcopy", 0, this, SLOT(slotEditCopy()), actionCollection(), "edit_copy");
	editPaste = new KAction(i18n("Paste"), "editpaste", 0, this, SLOT(slotEditPaste()), actionCollection(), "edit_paste");
	editSearch = new KAction(i18n("Search"), 0, this, SLOT(slotEditSearch()), actionCollection(),"edit_search");
	editReplace = new KAction(i18n("Replace"), 0, this, SLOT(slotEditReplace()), actionCollection(),"edit_replace");

	queryRun = new KAction(i18n("Run Query"), 0, this, SLOT(runQuery()), actionCollection(),"query_run");
	queryStop = new KAction(i18n("Stop Query"), 0, this, SLOT(stopQuery()), actionCollection(),"query_stop");
	queryCheck = new KAction(i18n("Parse Query"), 0, this, SLOT(parseQuery()), actionCollection(),"query_check");

	QIconSet textIcon (QPixmap((const char**)textout_pix_xpm));
	QIconSet gridIcon (QPixmap((const char**)gridout_act_pix_xpm));

	textResults = new KAction(i18n("Results in Text"), textIcon, 0, this, SLOT(enableTextResults()), actionCollection(),"text_results");

	gridResults = new KAction(i18n("Results in Grid"), gridIcon, 0, this, SLOT(enableGridResults()), actionCollection(),"grid_results");

	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
	viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

	windowNewWindow = new KAction(i18n("New &Window"), 0, this, SLOT(slotWindowNewWindow()), actionCollection(),"window_new_window");
	windowTile = new KAction(i18n("&Tile"), 0, this, SLOT(slotWindowTile()), actionCollection(),"window_tile");
	windowCascade = new KAction(i18n("&Cascade"), 0, this, SLOT(slotWindowCascade()), actionCollection(),"window_cascade");

	fileConnect->setStatusText(i18n("Open a new connection"));
	fileDisconnect->setStatusText(i18n("Close the current connection"));
	fileDisconnectAll->setStatusText(i18n("Close all connections"));

	fileNew->setStatusText(i18n("Creates a new document"));
	fileOpen->setStatusText(i18n("Opens an existing document"));
	fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
	fileSave->setStatusText(i18n("Saves the actual document"));
	fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
	fileSaveAll->setStatusText(i18n("Saves all documents as..."));
	fileClose->setStatusText(i18n("Closes the actual document"));
	filePrint ->setStatusText(i18n("Prints out the actual document"));
	fileQuit->setStatusText(i18n("Quits the application"));

	editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
	editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
	editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
	editSearch->setStatusText(i18n("Search."));
	editReplace->setStatusText(i18n("Replace."));

	viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
	viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

	windowMenu = new KActionMenu(i18n("&Window"), actionCollection(), "window_menu");
	connect(windowMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(windowMenuAboutToShow()));

	createGUI();
}


void KSqlAnalyzerApp::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	// STATUSBAR
	// TODO: add your own items you need for displaying current application status.
	statusBar()->insertItem(i18n("Ready."),1);
	statusBar()->setItemFixed(1,180);
}

void KSqlAnalyzerApp::initView()
{
	QValueList<int> splitSizes;
	splitSizes.append(90);
	splitSizes.append(this->width()-150);

	////////////////////////////////////////////////////////////////////
	// here the main view of the KMainWindow is created by a background box and
	// the QWorkspace instance for MDI view.
	QSplitter *split = new QSplitter(Qt::Horizontal, this);
	split->setBackgroundColor(darkGray);

	appBar = new MyAppBar(split);
	appBar->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );

	QVBox* view_back = new QVBox( split );
	view_back->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );

	split->setSizes(splitSizes);

	pWorkspace = new QWorkspace( view_back );
	setCentralWidget(split);
}

void KSqlAnalyzerApp::createClient(KSqlAnalyzerDoc* doc, bool new_conn)
{
	bool is_max = FALSE;

	QStringList login_info;
	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
		is_max = m->isMaximized();
	}
	if ( !new_conn ) {
		if ( m ) {
			login_info = m->getLoginInfo();
		}
	}
	KSqlAnalyzerView* w = new KSqlAnalyzerView(doc, new_conn, pWorkspace,0,WDestructiveClose);

	w->installEventFilter(this);
	doc->addView(w);
	w->setIcon(kapp->miniIcon());

	if ( !new_conn ) {
		w->setLoginInfo(login_info);
	} else {
		connect(w, SIGNAL(serverInfo(const QString &, const QString &, const QString &, int)),
					 appBar, SLOT(setServerInfo(const QString &, const QString &, const QString &, int)));
	}
	w->show();
	connect(w, SIGNAL(emitDb()) , this, SLOT(loginSucceed()));
	connect(w, SIGNAL(queryRunning(bool)) , this, SLOT(enableQueryTools(bool)));
	connect(w, SIGNAL(toggleAppBar()) , this, SLOT(toggleAppBar()));
	connect(w, SIGNAL(man_dbchange(const QString &)) , this, SLOT(man_dbchange(const QString &)));
	connect(w, SIGNAL(closeIt()) , this, SLOT(slotFileClose()));
	connect(w, SIGNAL(textModified(bool)) , this, SLOT(enableModifiers(bool)));

	enableStdActions(TRUE);

	w->resize(((this->width()/4)*3), ((this->height()/4)*3));
	w->showMaximized();

	slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::openDocumentFile(const KURL& url, bool new_conn)
{
//	slotStatusMsg(i18n("Opening file..."));
	QString itext;
	KSqlAnalyzerDoc* doc;

	// check, if document already open. If yes, set the focus to the first view
	for(doc=pDocList->first(); doc > 0; doc=pDocList->next())
	{
		if(doc->URL()==url)
		{
			KSqlAnalyzerView* view=doc->firstView();
			view->setFocus();
			return;
		}
	}

	// Creates an untitled window if file is 0
	if(url.isEmpty()) {
		doc = new KSqlAnalyzerDoc();
		pDocList->append(doc);
		doc->newDocument();
		untitledCount+=1;
		QString fileName=QString(i18n("Untitled%1")).arg(untitledCount);
		KURL url;
		url.setFileName(fileName);
		doc->setURL(url);

		// create the window
		createClient(doc, new_conn);
		connect(pWorkspace, SIGNAL(windowActivated(QWidget *)), this, SLOT(windowChanged(QWidget *)));
	} else {
		// Open the file
		KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
		if ( m ) {
			KSqlAnalyzerDoc* pDoc=m->getDocument();
			if(!(itext = pDoc->openDocument(url)) ) {
				KMessageBox::error (this,i18n("Could not open document !"), i18n("Error !"));
				return;
			}
			m->setText(itext);
			pDoc->setURL(url);
		}
		fileOpenRecent->addURL(url);
	}

	slotStatusMsg(i18n("Ready."));
}


void KSqlAnalyzerApp::saveOptions()
{
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", toolBar()->isVisible());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config,"Recent Files");
}


void KSqlAnalyzerApp::readOptions()
{
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void KSqlAnalyzerApp::saveProperties(KConfig *_cfg)
{

}


void KSqlAnalyzerApp::readProperties(KConfig* _cfg)
{
}

bool KSqlAnalyzerApp::queryClose()
{
  QStringList saveFiles;
  KSqlAnalyzerDoc* doc;
  if(pDocList->isEmpty())
    return true;

  for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
  {
    if(doc->isModified())
      saveFiles.append(doc->URL().fileName());
  }
  if(saveFiles.isEmpty())
    return true;

  switch (KMessageBox::questionYesNoList(this,
          i18n("One or more documents have been modified.\nSave changes before exiting?"),saveFiles))
  {
  case KMessageBox::Yes:
    for(doc=pDocList->first(); doc!=0;doc=pDocList->next())
    {
      if(doc->URL().fileName().contains(i18n("Untitled")))
        slotFileSaveAs();
      else
      {
        if(!doc->saveDocument(doc->URL()))
        {
          KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
          return false;
         }
       }
     }
     return true;
  case KMessageBox::No:
  default:
  return true;
  }
}

bool KSqlAnalyzerApp::queryExit()
{
  saveOptions();
  return true;
}

bool KSqlAnalyzerApp::eventFilter(QObject* object, QEvent* event)
{
  if((event->type() == QEvent::Close)&&((KSqlAnalyzerApp*)object!=this))
  {
    QCloseEvent* e=(QCloseEvent*)event;

    KSqlAnalyzerView* pView=(KSqlAnalyzerView*)object;
    KSqlAnalyzerDoc* pDoc=pView->getDocument();
    if(pDoc->canCloseFrame(pView))
    {
      pDoc->removeView(pView);
      if(!pDoc->firstView())
        pDocList->remove(pDoc);

	  if( !pDocList->count() )
			enableStdActions(FALSE);

      e->accept();
    }
    else
      e->ignore();
  }
  return QWidget::eventFilter( object, event );    // standard event processing
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KSqlAnalyzerApp::slotNewConn()
{
//  slotStatusMsg(i18n("Open a new connection..."));

  openDocumentFile(0L, TRUE);

//  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotDisConn()
{
	slotStatusMsg(i18n("Close actual connection..."));

	slotFileClose();

	slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotDisConnAll()
{
	MyMessageBox *msgBox = new MyMessageBox(i18n("Save changes ?"), this);

	bool no_to_all = FALSE;
  slotStatusMsg(i18n("Close all connections..."));
	QString tmp;

	KSqlAnalyzerDoc* doc;
	while ( pDocList->count() ) {
		doc = pDocList->last();

	  KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	  if( m )
	  {
			m->setFocus();
			if ( m->isModified() && !no_to_all ) {
					tmp.sprintf(i18n("The text in %s has changed.\n\nDo you want to save the changes ?"), (const char*)doc->URL().fileName());
					msgBox->setMessage(tmp);
					switch ( msgBox->exec() ) {
					case 0:
						slotFileSave();
						break;
					case 1:
						break;
					case 2:
						no_to_all = TRUE;
						break;
					case 3:
						return;
						break;
				}
			}
		}
		doc->closeDocument();
	}
	if( !pDocList->count() )
		enableStdActions(FALSE);

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  openDocumentFile(0L, FALSE);

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));

	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
			m->setFocus();
			if ( m->isModified() ) {
				switch ( QMessageBox::information(this, i18n("Save changes ?"),
					i18n("Do you want to save the changes ?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Abort) ) {
					case QMessageBox::Yes:
						slotFileSave();
						break;
					case QMessageBox::No:
						break;
					case QMessageBox::Abort:
						return;
						break;
				}
			}
	}
  KURL url=KFileDialog::getOpenURL(QString::null,
      i18n("*|All files"), this, i18n("Open File..."));
  if(!url.isEmpty())
  {
    openDocumentFile(url, FALSE);
    fileOpenRecent->addURL( url );
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
			m->setFocus();
			if ( m->isModified() ) {
				switch ( QMessageBox::information(this, i18n("Save changes ?"),
					i18n("Do you want to save the changes ?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Abort) ) {
					case QMessageBox::Yes:
						slotFileSave();
						break;
					case QMessageBox::No:
						break;
					case QMessageBox::Abort:
						return;
						break;
				}
			}
	}
  openDocumentFile(url, FALSE);

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileSave()
{
	QString txt;
	slotStatusMsg(i18n("Saving file..."));

	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
		txt = m->getSaveText();
		if ( !m->isTextOutput() ) {
			KSqlAnalyzerDoc* doc = m->getDocument();
			if(doc->URL().fileName().contains(i18n("Untitled")))
				slotFileSaveAs();
			else
				if(!doc->saveDocument(doc->URL(),0,txt)) {
					KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
					m->setModified(TRUE);
				}
				m->setModified(FALSE);
			} else {
				if ( last_save_url.isEmpty() )
					last_save_url = QDir::currentDirPath();

				KURL url=KFileDialog::getSaveURL(last_save_url,
					i18n("*.rpt|Report Files (*.rpt)\n*|All files (*)"), this, i18n("Save as..."));
				if(!url.isEmpty()) {
					QString filename;
					filename = url.encodedPathAndQuery();

					QFile f( filename );
					if ( f.exists() ) {
						switch ( QMessageBox::information(this, i18n("File exists ?"),
							i18n("Do you want to overwrite the file ?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Abort) ) {
							case QMessageBox::Yes:
								break;
							case QMessageBox::No:
							case QMessageBox::Abort:
								return;
								break;
						}
					}
					if ( !f.open( IO_WriteOnly ) ) {
						KMessageBox::error (this,i18n("Could not save the file !"), i18n("I/O Error !"));
						return;
					}
					QTextStream stream( &f );
					stream << txt;
					f.close();
				}
			}
	}
	slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileSaveAs()
{
	QString txt;
	slotStatusMsg(i18n("Saving file with a new filename..."));

	if ( last_save_url.isEmpty() )
		last_save_url = QDir::currentDirPath();

	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if( m ) {
		txt = m->getSaveText();
		if ( !m->isTextOutput() ) {
			KURL url=KFileDialog::getSaveURL(last_save_url,
			i18n("*.sql|Query Files (*.sql)\n*.tql|Template Sql Files (*.tql)\n*|All files (*)"), this, i18n("Save as..."));
			if(!url.isEmpty()) {
				last_save_url = url.url();

				KSqlAnalyzerDoc* doc =	m->getDocument();
				if(!doc->saveDocument(url,0,txt)) {
					KMessageBox::error (this,i18n("Could not save the current document !"), i18n("I/O Error !"));
					m->setModified(TRUE);
					return;
				}
				m->setModified(FALSE);
				doc->changedViewList();
				fileOpenRecent->addURL(url);
			}
		} else {
			KURL url=KFileDialog::getSaveURL(last_save_url,
				i18n("*.rpt|Report Files (*.rpt)\n*|All files (*)"), this, i18n("Save as..."));
			if(!url.isEmpty()) {
				last_save_url = url.url();

				QString filename;
				filename = url.encodedPathAndQuery();

				QFile f( filename );
				if ( f.exists() ) {
					switch ( QMessageBox::information(this, i18n("File exists ?"),
						i18n("Do you want to overwrite the file ?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Abort) ) {
						case QMessageBox::Yes:
							break;
						case QMessageBox::No:
						case QMessageBox::Abort:
							return;
							break;
					}
				}
				if ( !f.open( IO_WriteOnly ) ) {
					KMessageBox::error (this,i18n("Could not save the file !"), i18n("I/O Error !"));
					return;
				}
				QTextStream stream( &f );
				stream << txt;
				f.close();
			}
		}
	}
	slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileSaveAll()
{
	KSqlAnalyzerDoc* doc;
	for(doc=pDocList->first(); doc > 0; doc=pDocList->next())
	{
		KSqlAnalyzerView* view=doc->firstView();
		view->setFocus();
		slotFileSaveAs();
	}
}

void KSqlAnalyzerApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
  KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
  if( m )
  {
		m->setFocus();
		if ( m->isModified() ) {
			switch ( QMessageBox::information(this, i18n("Save changes ?"),
				i18n("Do you want to save the changes ?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Abort) ) {
				case QMessageBox::Yes:
					slotFileSave();
					break;
				case QMessageBox::No:
					break;
				case QMessageBox::Abort:
					return;
					break;
			}
		}
    KSqlAnalyzerDoc* doc=m->getDocument();
    doc->closeDocument();
  }

  if( !pDocList->count() )
		enableStdActions(FALSE);

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m )
    m->print( printer );

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
        break;
    }
  }
  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::closeEvent(QCloseEvent *ce)
{
	slotDisConnAll();
	if( !pDocList->count() ) {
		QWidget::closeEvent(ce);
		exit(0);
	}
}

void KSqlAnalyzerApp::slotEditUndo()
{
  slotStatusMsg(i18n("Reverting last action..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m )
//  m->undo();

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotEditSearch()
{
  slotStatusMsg(i18n("Search..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m )
		m->doSearch();

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotEditReplace()
{
  slotStatusMsg(i18n("Replace..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m )
		m->doReplace();

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m )
    m->inputField->cut();

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m ) {
		if ( !m->focusWidg() )
	    m->inputField->copy();
		else
			m->copySelection();
	}
  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  KSqlAnalyzerView* m = (KSqlAnalyzerView*) pWorkspace->activeWindow();
  if ( m )
    m->inputField->paste();

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotWindowNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  openDocumentFile(0L, FALSE);

  slotStatusMsg(i18n("Ready."));
}

void KSqlAnalyzerApp::slotWindowTile(){
  pWorkspace->tile();
}

void KSqlAnalyzerApp::slotWindowCascade(){
  pWorkspace->cascade();
}

void KSqlAnalyzerApp::slotStatusMsg(const QString &text)
{
	///////////////////////////////////////////////////////////////////
	// change status message permanently
	statusBar()->clear();
	statusBar()->changeItem(text,1);
}


void KSqlAnalyzerApp::windowMenuAboutToShow()
{
  windowMenu->popupMenu()->clear();
  windowMenu->insert(windowNewWindow);
  windowMenu->insert(windowCascade);
  windowMenu->insert(windowTile);

  if ( pWorkspace->windowList().isEmpty() ){
    windowNewWindow->setEnabled(false);
    windowCascade->setEnabled(false);
    windowTile->setEnabled(false);
  }
  else{
    windowNewWindow->setEnabled(true);
    windowCascade->setEnabled(true);
    windowTile->setEnabled(true);
  }
  windowMenu->popupMenu()->insertSeparator();

  QWidgetList windows = pWorkspace->windowList();
  for ( int i = 0; i < int(windows.count()); ++i )
  {
    int id = windowMenu->popupMenu()->insertItem(QString("&%1 ").arg(i+1)+windows.at(i)->caption(), this, SLOT( windowMenuActivated( int ) ) );
    windowMenu->popupMenu()->setItemParameter( id, i );
    windowMenu->popupMenu()->setItemChecked( id, pWorkspace->activeWindow() == windows.at(i) );
  }
}

void KSqlAnalyzerApp::windowMenuActivated( int id )
{
  QWidget* w = pWorkspace->windowList().at( id );
  if ( w )
    w->setFocus();
}

void KSqlAnalyzerApp::changeDB(const QString &newdb)
{
	QString tmp;
	db = newdb;
	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if( m ) {
		m->changeDb(db);
	}
}


void KSqlAnalyzerApp::toggleAppBar()
{
	if ( appBar->isVisible() )
		appBar->hide();
	else
		appBar->show();
}

/*
 * Insert every single DB-Name into the Toolbar-Combo
*/
void KSqlAnalyzerApp::dbReceived(QValueList<QStringList> &dbname_list, const QString & act_dbname)
{
	QStringList tmp;
	QString dbs[2048];
	QString default_db;
	int db_cnt;
	db_cnt = 0;

	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if( m ) {
		default_db = m->getDefaultDB();
	}
	if ( default_db.isEmpty() || default_db.isNull() )
		default_db = "master";

	toolBar()->clearCombo(3);
	for ( QValueList<QStringList>::Iterator it=dbname_list.begin() ; it != dbname_list.end() ; ++it ) {
		tmp = *it;
		if ( tmp.first() != "" ) {
			dbs[db_cnt] = tmp.first();
			db_cnt++;
		}
	}
	for ( int i=db_cnt-1 ; i>=0 ; i-- )
		toolBar()->insertComboItem(3, dbs[i], -1);

	KComboBox *combo = toolBar()->getCombo(3);

	for ( int i=0 ; i < combo->count() ; i++ ) {
		if ( toolBar()->getComboItem(3, i) == default_db ) //act_dbname )
			toolBar()->setCurrentComboItem(3,i);
	}
}

/*
  The user has changed the db
*/
void KSqlAnalyzerApp::man_dbchange(const QString &newdb)
{
	QString all_db, cleared_db;
	all_db = newdb;
	KComboBox *combo = toolBar()->getCombo(3);
	cleared_db = all_db.replace('[', "");
	cleared_db = cleared_db.replace(']', "");
	for ( int i=0 ; i < combo->count() ; i++ ) {
		if ( toolBar()->getComboItem(3, i) == cleared_db )
			toolBar()->setCurrentComboItem(3,i);
	}
}

void KSqlAnalyzerApp::runQuery()
{
  KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
  if( m ) {
		m->runQuery();
	}
}

void KSqlAnalyzerApp::parseQuery()
{
  KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
  if( m ) {
		m->runQuery(TRUE);
	}
}

void KSqlAnalyzerApp::stopQuery()
{
  KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
  if( m ) {
		m->stopQuery();
	}
}

void KSqlAnalyzerApp::windowChanged(QWidget *win)
{
	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
		dbReceived(m->getDBList(), m->getDbName());
	}
}

void KSqlAnalyzerApp::loginSucceed()
{
	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
		dbReceived(m->getDBList(), m->getDbName());
		appBar->setDbNames(m->getDBNames());
	}
}

void KSqlAnalyzerApp::timerEvent(QTimerEvent *te)
{
	if ( !tools_enabled ) {
		toolBar()->insertButton(QPixmap((const char**)checkBut_xpm), 0, SIGNAL(clicked()), this, SLOT(parseQuery()), true, "Test", -1 );
		toolBar()->insertButton(QPixmap((const char**)runBut_xpm), 1, SIGNAL(clicked()), this, SLOT(runQuery()), true, 0L, -1 );
		toolBar()->insertButton(QPixmap((const char**)stopBut_xpm), 2, SIGNAL(clicked()), this, SLOT(stopQuery()), true, 0L, -1 );
		toolBar()->setItemEnabled ( 2, FALSE );

		// Combobox fr die Datenbanknamen
		toolBar()->insertCombo(0L, 3, false, SIGNAL(activated(const QString &)),
				this, SLOT(changeDB(const QString &)), true, "Change DB", 170, -1, QComboBox::AtBottom);
		toolBar()->clearCombo(3);

		toolBar()->showItem(0);
		toolBar()->showItem(1);
		toolBar()->showItem(2);
		toolBar()->showItem(3);
		tools_enabled = TRUE;
		killTimers();
	}
}

void KSqlAnalyzerApp::enableQueryTools(bool ena)
{
	toolBar()->setItemEnabled ( 0, ena );
	toolBar()->setItemEnabled ( 1, ena );
	toolBar()->setItemEnabled ( 2, !ena );
	toolBar()->setItemEnabled ( 3, ena );
}

void KSqlAnalyzerApp::enableTextResults()
{
	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
		m->setTextOutput(TRUE);
		QIconSet textIcon (QPixmap((const char**)textout_act_pix_xpm));
		QIconSet gridIcon (QPixmap((const char**)gridout_pix_xpm));
		textResults->setIconSet(textIcon);
		gridResults->setIconSet(gridIcon);
	}
}

void KSqlAnalyzerApp::enableGridResults()
{
	KSqlAnalyzerView* m = (KSqlAnalyzerView*)pWorkspace->activeWindow();
	if ( m ) {
		m->setTextOutput(FALSE);
		QIconSet textIcon (QPixmap((const char**)textout_pix_xpm));
		QIconSet gridIcon (QPixmap((const char**)gridout_act_pix_xpm));
		textResults->setIconSet(textIcon);
		gridResults->setIconSet(gridIcon);
	}
}
