/***************************************************************************
                          ksqlanalyzerdoc.cpp  -  description
                             -------------------
    begin                : Sun Jan 26 13:05:56 CET 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/job.h>
#include <kio/netaccess.h>

// application specific includes
#include "ksqlanalyzer.h"
#include "ksqlanalyzerview.h"
#include "ksqlanalyzerdoc.h"

KSqlAnalyzerDoc::KSqlAnalyzerDoc()
{
  pViewList = new QList<KSqlAnalyzerView>;
  pViewList->setAutoDelete(false);
}

KSqlAnalyzerDoc::~KSqlAnalyzerDoc()
{
  delete pViewList;
}

void KSqlAnalyzerDoc::addView(KSqlAnalyzerView *view)
{
  pViewList->append(view);
	changedViewList();
}

void KSqlAnalyzerDoc::removeView(KSqlAnalyzerView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void KSqlAnalyzerDoc::changedViewList()
{
	KSqlAnalyzerView *w;
	if((int)pViewList->count() == 1){
	  	w=pViewList->first();
	  	w->setCaption(URL().fileName());
	} else {
		int i;
	    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
	  		w->setCaption(QString(URL().fileName()+":%1").arg(i));
	}
}

bool KSqlAnalyzerDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void KSqlAnalyzerDoc::updateAllViews(KSqlAnalyzerView *sender)
{
  KSqlAnalyzerView *w;
  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void KSqlAnalyzerDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KSqlAnalyzerDoc::URL() const
{
  return doc_url;
}

void KSqlAnalyzerDoc::closeDocument()
{
  KSqlAnalyzerView *w;
  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
	 	w=pViewList->first();
  	w->close();
  }
}

bool KSqlAnalyzerDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  return true;
}

QString KSqlAnalyzerDoc::openDocument(const KURL &url, const char *format /*=0*/)
{
  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile );

  /////////////////////////////////////////////////
	QFile f( tmpfile );
	if ( !f.open( IO_ReadOnly ) )
		return 0L;

	QTextStream stream( &f );
	QString line, lines;
	int i = 1;
	while ( !stream.eof() ) {
		line = stream.readLine(); // line of text excluding '\n'
		line += "\n";
		lines += line;
	}

  /////////////////////////////////////////////////
  // TODO: Add your document opening code here
  /////////////////////////////////////////////////
	f.close();

  /////////////////////////////////////////////////
  KIO::NetAccess::removeTempFile( tmpfile );
  doc_url=url;

  modified=false;
  return lines;
}

bool KSqlAnalyzerDoc::saveDocument(const KURL &url, const char *format, const QString &savetext)
{
	QString filename;
	filename = url.encodedPathAndQuery();

	QFile f( filename );
	if ( !f.open( IO_WriteOnly ) )
		return false;

	QTextStream stream( &f );
	stream << savetext;
	f.close();

	setURL(url);

	modified=false;
	return true;
}

void KSqlAnalyzerDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool KSqlAnalyzerDoc::canCloseFrame(KSqlAnalyzerView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		KURL saveURL;
  	switch(KMessageBox::warningYesNoCancel(pFrame, i18n("The current file has been modified.\n"
                          "Do you want to save it?"),URL().fileName()))
    {
			case KMessageBox::Yes:
				if(URL().fileName().contains(i18n("Untitled")))
				{
					saveURL=KFileDialog::getSaveURL(QDir::currentDirPath(),
                      i18n("*|All files"), pFrame, i18n("Save as..."));
          if(saveURL.isEmpty())
          	return false;
				}
				else
						saveURL=URL();
					
				if(!saveDocument(saveURL))
				{
 					switch(KMessageBox::warningYesNo(pFrame,i18n("Could not save the current document !\n"
																												"Close anyway ?"), i18n("I/O Error !")))
 					{
 						case KMessageBox::Yes:
 							ret=true;
 						case KMessageBox::No:
 							ret=false;
 					}	        			
				}
				else
					ret=true;
				break;
			case KMessageBox::No:
				ret=true;
				break;
			case KMessageBox::Cancel:
			default:
				ret=false; 				
				break;
		}
	}
	else
		ret=true;
		
	return ret;
}
