/***************************************************************************
                          ksqlanalyzerview.cpp  -  description
                             -------------------
    begin                : Sun Jan 26 13:05:56 CET 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <qdir.h>
#include <qlabel.h>

// include files for KDE

// application specific includes
#include "ksqlanalyzer.h"
#include "ksqlanalyzerview.h"
#include "ksqlanalyzerdoc.h"

#define TXT 1
#define PIC 2
#define BIN 3

KSqlAnalyzerView::KSqlAnalyzerView(KSqlAnalyzerDoc* pDoc, bool new_conn, QWidget *parent, const char* name, int wflags)
 : QWidget(parent, name, wflags)
{
	all_canceled = FALSE;
	doc=pDoc;
	resultset = -1;
	num_rows = 0;
	s2height = 0;
	cnt = 0;
	query_ready = TRUE;
	dbList.append(0L);
	new_connection = new_conn;
	text_output = FALSE;
	modified = FALSE;
	is_init = TRUE;
	is_error = FALSE;

	// Init some Variables
	server = "localhost";
	login = "sa";
	port = 1433;
	pass = "";
	db = "master";
	dbuser = "dbo";
	conn_error = false;
	conn_ready = false;
	tsql = new TSQLCont();
	connect(tsql, SIGNAL(tsql_fatal(const QString &)), this, SLOT(fatal_error(const QString &)));
	connect(tsql, SIGNAL(tsql_message(const QString &)), this, SLOT(sql_message(const QString &)));

	s1 = new QSplitter(QSplitter::Vertical, this);
	s1->setGeometry(0,0,this->width(), this->height()-25);
	s1->setFrameStyle(QFrame::Panel | QFrame::Sunken);

	inputField = new KWrite(s1);
	inputField->setCursorPosition(0,0);
	connect(inputField, SIGNAL(newCurPos()), this, SLOT(showCurPos()));
	connect(inputField, SIGNAL(overwriteMode(int)), this, SLOT(showOverwriteMode(int)));

//inputField->insert("select top 20 * from syscolumns\n");

	outputField = new MyScrollView(s1);
	outputField->setBackgroundColor(white);
	outputField->hide();
	connect(outputField, SIGNAL(resized(bool)), this, SLOT(resizeOutput(bool)));

	messageText = new KTextEdit(s1);
	messageText->setWordWrap(QTextEdit::NoWrap);
	messageText->hide();

	s2 = new QSplitter(QSplitter::Vertical, 0L);
	s2->setOpaqueResize(TRUE);

	statBar = new KStatusBar(this);
	statBar->insertItem(i18n("Ready."),1);
	statBar->insertItem("                ",2);
	statBar->insertItem("                ",3);
	statBar->insertItem(i18n("Ln 0, Col 0"),4);
	statBar->insertItem(i18n("INS"),5);
	statBar->setItemFixed(1,180);
	statBar->setItemAlignment(1, Qt::AlignLeft);

	statBar->show();
	statBar->setGeometry(0,this->height()-25, this->width(), 25);
	statBar->setSizeGripEnabled(TRUE);

	outputText = new KTextEdit(s2);
	outputText->setWordWrap(QTextEdit::NoWrap);
	outputText->setFont(QFont("Courier", 9));
	outputText->hide();

	outputField->addChild(s2);

	winTab = new QTabBar(this);
	QTab *gridTab = new QTab(i18n("Grids"));
	QTab *msgTab = new QTab(i18n("Messages"));
	grid_id = winTab->addTab(gridTab);
	msg_id = winTab->addTab(msgTab);
	winTab->setShape(QTabBar::TriangularBelow);
	winTab->show();
	connect(winTab, SIGNAL(selected(int)), this, SLOT(switchOutput(int)));

	if ( new_conn )
		openLoginDialog();

	QTimer::singleShot(400, this, SLOT(focusIt()));
}

KSqlAnalyzerView::~KSqlAnalyzerView()
{
}

void KSqlAnalyzerView::focusIt()
{
	inputField->setFocus();
}

QString KSqlAnalyzerView::getSaveText()
{
	if ( inputField->hasFocus() || !text_output || !outputField->isVisible() )
		return inputField->text();
	else if ( outputText->hasFocus() )
		return outputText->text();
	else
		return inputField->text();
}

bool KSqlAnalyzerView::isTextOutput()
{
	if ( text_output && outputField->isVisible() && outputText->hasFocus() )
		return TRUE;
	return FALSE;
}

void KSqlAnalyzerView::showCurPos()
{
	QString tmp;
	int col, row;
	col = inputField->currentColumn();
	row = inputField->currentLine();
	tmp.sprintf(i18n("Ln %d, Col %d"), row, col);
	statBar->changeItem(tmp,4);

	modified = inputField->isModified();
	emit textModified(inputField->isModified());
}

void KSqlAnalyzerView::showOverwriteMode(int o)
{
	if ( o )
		statBar->changeItem(i18n("OVR"),5);
	else
		statBar->changeItem(i18n("INS"),5);
}

void KSqlAnalyzerView::setText(const QString &text)
{
	if ( !text.isEmpty() && !text.isNull() )
		inputField->setText(text);
}

void KSqlAnalyzerView::openLoginDialog() {
	loginDialog = new LoginDialog(this, 0, true);
	loginDialog->setFixedSize(400,200);
	connect(loginDialog, SIGNAL(canceled()), this, SLOT(loginCanceled()));
	connect(loginDialog, SIGNAL(login_data(QString, QString, QString)), this,
					SLOT(setServerData(QString, QString, QString)));
 	loginDialog->exec();
}

void KSqlAnalyzerView::setServerData(QString srv, QString log, QString pwd, bool new_conn)
{
	server = srv;
	login = log;
	pass = pwd;
	if ( QString(login) == QString("sa") )
		dbuser = "dbo";
	else
		dbuser = login;

	if ( new_conn ) {
		loginDialog->close();
		delete(loginDialog);
	}
	startSession();
}

void KSqlAnalyzerView::startSession()
{
	// Erster connect zum DB-Server
	// Liste aller Datenbanken holen
	tsql->tsql_login(server, port, login, pass);
	if ( tsql->tsql_connect() ) {
		// Die Key-Events aktivieren
		accel = new QAccel( this );
		accel->connectItem( accel->insertItem( Key_F5 ),
		this, SLOT(runQuery()) );
		accel->connectItem( accel->insertItem( Key_F6 ),
		this, SLOT(parseQuery()) );
		accel->connectItem( accel->insertItem( Key_F8 ),
				this, SIGNAL(toggleAppBar()) );


		tsql->tsql_doQuery("set quoted_identifier off");
		tsql->tsql_doQuery("SET TEXTSIZE 64512");
		tsql->tsql_doQuery("set showplan_text off");
		tsql->tsql_doQuery("SET NOEXEC OFF SET PARSEONLY OFF");
		tsql->tsql_doQuery("set showplan_all off");
		tsql->tsql_doQuery("set nocount off;set arithabort off;set concat_null_yields_null off;"
												"set ansi_nulls off;set cursor_close_on_commit off;set ansi_null_dflt_on off;"
												"set implicit_transactions off;set ansi_padding off;set ansi_warnings off;"
												"set quoted_identifier off");
		tsql->tsql_doQuery("set lock_timeout -1");
		tsql->tsql_doQuery("set nocount off;set arithabort on;set concat_null_yields_null on;"
												"set ansi_nulls on;set cursor_close_on_commit off;set ansi_null_dflt_on on;"
												"set implicit_transactions off;set ansi_padding on;set ansi_warnings on;"
												"set quoted_identifier on");

		connect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
					 this, SIGNAL(dbReceived(QValueList<QStringList> &)));
		connect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
					 this, SLOT(storeDBList(QValueList<QStringList> &)));
		connect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(firstSessionReady(int, int)));
		tsql->tsql_doQuery("Create Table #MyHelpDBTable( name varchar(255) NULL,"
					"f_db_size float  NULL, db_size varchar(255)  NULL, owner varchar(255) NULL, dbid int NULL,"
					" created DateTime NULL, status varchar(4000) NULL, comp INT NULL); "
					"insert #MyHelpDBTable(name, db_Size, owner, dbid, created, status, comp) exec sp_Helpdb;"
					"select name from #MyHelpDBTable order by name desc; "
					"drop table #MyHelpDBTable;");
	} else {
		openLoginDialog();
	}
}

/*
 * All DB-Names received.
 * Time to get more info's about the databases
*/
void KSqlAnalyzerView::firstSessionReady(int rows, int time_elapsed)
{
	disconnect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
				 this, SIGNAL(dbReceived(QValueList<QStringList> &)));
	disconnect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
				 this, SLOT(storeDBList(QValueList<QStringList> &)));
	disconnect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(firstSessionReady(int, int)));

	connect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
			 this, SLOT(setDBName(QValueList<QStringList> &)));
	connect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(secondSessionReady(int, int)));
	QString tmp;
	tmp = "select dbname from master..syslogins where name='";
	tmp += dbuser;
	tmp += "'";
	tsql->tsql_doQuery(tmp);
}

void KSqlAnalyzerView::secondSessionReady(int rows, int time_elapsed)
{
	disconnect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
			 this, SLOT(setDBName(QValueList<QStringList> &)));
	disconnect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(secondSessionReady(int, int)));

	connect(tsql, SIGNAL(tsql_tableinfo_received(QStringList &)),
				 this, SLOT(tableInfoReceived(QStringList &)));
	connect(tsql, SIGNAL(tsql_querycolumns_received(int, QStringList &)),
				 this, SLOT(columnsReceived(int, QStringList &)));
	connect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
				 this, SLOT(rowReceived(QValueList<QStringList> &)));
	connect(tsql, SIGNAL(tsql_dbchange(const QString &)),
				 this, SIGNAL(man_dbchange(const QString &)));
	connect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(queryReady(int, int)));
	connect(tsql, SIGNAL(tsql_error(const QString &)), this , SLOT(queryError(const QString &)));
}

KSqlAnalyzerDoc *KSqlAnalyzerView::getDocument() const
{
	return doc;
}

void KSqlAnalyzerView::createOutputField()
{
}

void KSqlAnalyzerView::update(KSqlAnalyzerView* pSender){
	if(pSender != this)
		repaint();
}

void KSqlAnalyzerView::print(QPrinter *pPrinter)
{
	const int Margin = 10;
	int pageNo = 1;

	if (pPrinter->setup(this)) {
		QPainter p;
		p.begin(pPrinter);

		p.setFont( inputField->font() );
		int yPos        = 0;                    // y-position for each line
		QFontMetrics fm = p.fontMetrics();
		QPaintDeviceMetrics metrics( pPrinter ); // need width/height
		// of printer surface

		for( int i = 0 ; i < inputField->lines() ; i++ ) {
			if ( Margin + yPos > metrics.height() - Margin ) {
				QString msg( "Printing (page " );
				msg += QString::number( ++pageNo );
				msg += ")...";
				pPrinter->newPage();             // no more room on this page
				yPos = 0;                       // back to top of page
			}
			p.drawText( Margin, Margin + yPos, metrics.width(), fm.lineSpacing(), ExpandTabs | DontClip, inputField->textLine( i ) );
			yPos = yPos + fm.lineSpacing();
		}
		p.end();
  }
}

void KSqlAnalyzerView::closeEvent(QCloseEvent* e)
{
	all_canceled = TRUE;
// DO NOT CALL QWidget::closeEvent(e) here !!
// This will accept the closing by QCloseEvent::accept() by default.
// The installed eventFilter() in KSqlAnalyzerApp takes care for closing the widget
// or ignoring the close event
}

void KSqlAnalyzerView::resizeEvent(QResizeEvent *re)
{
	try {
		this->resize(this->width(), this->height());
	   s1->setGeometry(0,0,this->width(), this->height()-50);

		s2->resize(outputField->width()-18, outputField->height());
		statBar->setGeometry(0,this->height()-25, this->width(), 25);
		winTab->setGeometry(0, (this->height()-25)-statBar->height(),this->width(), 25);

		QValueList<int> s1sizes;
		s1sizes.append(this->height()/2);
		s1sizes.append(this->height()/2);

		s1->setSizes(s1sizes);
	} catch (...) {
		debug("Exception...");
	}
}

QString KSqlAnalyzerView::returnQuery()
{
	return inputField->text();
}

void KSqlAnalyzerView::showOutputWindow(bool show)
{
	if ( show && !outputField->isVisible() ) {
		// Resize the 1st Splitter to 50%/50%
		QValueList<int> sizes;
		sizes.append((this->height()/2)-50);
		sizes.append(this->height()/2);
		s1->setSizes(sizes);
	} else if (!show) {
		outputField->hide();
	}
}

void KSqlAnalyzerView::setColumns(QStringList &columns)
{
	if ( all_canceled )
		return;

	QString tmp;

	if ( !text_output) {
		resultset++;

		int col = 0;
		QHeader *header = outputList[resultset]->horizontalHeader();
		for ( QStringList::Iterator it=columns.begin() ; it != columns.end() ; ++it ) {
			header->setLabel(col, (*it).latin1(), 80);
			col++;
		}
		header->setMovingEnabled(FALSE);
		header->adjustHeaderSize ();
		outputList[resultset]->show();
		num_rows = 0;

		startTimer(10);
	} else {
		resultset++;
		num_rows = 0;
		for ( QStringList::Iterator it=columns.begin() ; it != columns.end() ; ++it ) {
			tmp = (*it); //.latin1();
			tmp = tmp.replace('\t', "");
			outputCols[resultset] += tmp;
		}
	}
}

void KSqlAnalyzerView::prepareTable(QStringList &tableInfo)
{
	s2sizes.clear();

	if ( all_canceled )
		return;

	QStringList split;
	int rows, cols;
	QString tmp;
	QStringList::Iterator it2;
	cnt = 0;

	for ( QStringList::Iterator it=tableInfo.begin() ; it != tableInfo.end() ; ++it ) {
		if ( cnt > 2480 )
			continue;

		if ( !text_output) {
			tmp = (*it).latin1();
			split = QStringList::split("_", tmp);
			it2=split.begin();
			tmp = (*it2).latin1();
			rows = tmp.toInt();
			s2height += (rows*20);
			s2sizes.append((rows*20));
//s2sizes.append(10);
			++it2;
			tmp = (*it2).latin1();
			cols = tmp.toInt();
			outputList[cnt] = new MyTable(s2);
			outputList[cnt]->setReadOnly(TRUE);
			outputList[cnt]->setNumCols(cols);
			outputList[cnt]->setNumRows(rows);
			outputList[cnt]->setSelectionMode(QTable::Single);

			connect(outputList[cnt], SIGNAL(contextMenuRequested(int,int,const QPoint &)), this,
							SLOT(showTableContext(int,int, const QPoint &)));
			connect(outputList[cnt], SIGNAL(selectionChanged()), this, SLOT(initSelection()));
			connect(outputList[cnt], SIGNAL(copyRequest()), this, SLOT(copySelection()));
			connect(outputList[cnt], SIGNAL(selectAll()), this, SLOT(selectAll()));
			connect(outputList[cnt], SIGNAL(lostFocus(QTable *)), this, SLOT(removeSelection(QTable *)));
		} else {
			outputText->show();
		}
		cnt++;
	}

	if ( cnt > 1 ) {
		if ( s2height > 800 )
			s2height = 800;
	} else {
		s2height = outputField->height();
	}
}

void KSqlAnalyzerView::insertRow(QValueList<QStringList> &all_rows)
{
	if ( all_canceled )
		return;

	QStringList row;
	QString tmp2, msg_str;
	QCString tmp;
	int row_all_cnt = all_rows.count();
	int col_all_cnt = outputCols[resultset].count();
	int col_len[col_all_cnt];
	QString outputAll[row_all_cnt][col_all_cnt];
	int cnt;

	cnt = all_rows.first().count();
	int clen[all_rows.count()][cnt];
	int tmp_clen[cnt];
	int r, c;
	r = -1;

	for ( QValueList<QStringList>::Iterator it=all_rows.begin() ; it != all_rows.end() ; ++it ) {
		if ( all_canceled )
			return;
		row = *it;
		num_rows++;
		r++;
		c = -1;

		for ( QStringList::Iterator it2=row.begin() ; it2 != row.end() ; ++it2 ) {
			if ( all_canceled )
				return;

			c++;
			tmp = (*it2);
			clen[r][c] = (tmp.length()*15);

			if ( !(num_rows%10) ) {
				qApp->processEvents();
			}

			if ( !text_output) {
				if ( all_canceled )
					return;
				outputList[resultset]->setText(num_rows-1, c, tmp);
			} else {
				outputAll[num_rows-1][c] = tmp.left(255);
			}
		}
	}
	msg_str.sprintf(i18n("\n(%d Row(s) affected)\n"), row_all_cnt);
	messageText->insert(msg_str);

	// Set the column-width to optimal values
	if ( !text_output ) {
		for ( int c=0 ; c < cnt ; c++ ) {
			tmp_clen[c] = 0;
		}
		for ( int r=0 ; r < num_rows ; r++ ) {
			for ( int c=0 ; c < cnt ; c++ ) {
				if ( clen[r][c] > tmp_clen[c] )
					tmp_clen[c] = clen[r][c];
			}
		}
		for ( int c=0 ; c < cnt ; c++ ) {
			if ( tmp_clen[c] > 250 )
				tmp_clen[c] = 250;
			else if ( tmp_clen[c] < 70 )
				tmp_clen[c] = 70;
			outputList[resultset]->setColumnWidth(c, tmp_clen[c]);
		}
	}

	// If text-Ouput is recuested, show the result immediatly
	if ( text_output) {
		int act_col = 0;
		QString tmp;

		for ( QStringList::Iterator it=outputCols[resultset].begin() ; it != outputCols[resultset].end() ; ++it ) {
			tmp = (*it).latin1();
			col_len[act_col] = tmp.length();
			act_col++;
		}

		for ( int r=0 ; r < row_all_cnt ; r++ ) {
			for ( int c=0 ; c < col_all_cnt ; c++ ) {
				if ( outputAll[r][c].length() > col_len[c] )
					col_len[c] = outputAll[r][c].length();
			}
		}

		QString outputTxt = "";
		QString outputSep;
		act_col = 0;

		for ( QStringList::Iterator it=outputCols[resultset].begin() ; it != outputCols[resultset].end() ; ++it ) {
			tmp = (*it).latin1();
			if ( tmp.length() < col_len[act_col] ) {
				for ( int i=tmp.length() ; i < col_len[act_col] ; i++ ) {
					tmp += " ";
				}
			}
			tmp += " ";
			for ( int i=0 ; i < tmp.length()-1 ; i++ )
				outputSep += "-";
			outputSep += " ";

			outputTxt += tmp;
			act_col++;
		}
		outputTxt += "\n";
		outputTxt += outputSep;
		outputTxt += "\n\n";

		for ( int r=0 ; r < row_all_cnt ; r++ ) {
			for ( int c=0 ; c < col_all_cnt ; c++ ) {
				tmp = outputAll[r][c];
				if ( tmp.length() < col_len[c] ) {
					for ( int i=tmp.length() ; i < col_len[c] ; i++ )
						tmp += " ";
				}
				tmp += " ";
				outputTxt += tmp;
			}
			outputTxt += "\n";
		}
		tmp.sprintf(i18n("\n(%d row(s) affected)\n\n"), row_all_cnt);
		outputTxt += tmp;

		outputText->insert(outputTxt);
	}

	if ( !is_error && num_rows )
		outputField->show();
}

void KSqlAnalyzerView::clearResults()
{
	if ( all_canceled )
		return;

	is_error = FALSE;
	num_rows = 0;
	if ( (!text_output || was_grid_output) && !was_text_output ) {
		for ( int i=resultset ; i>= 0 ; i-- ) {
			delete(outputList[i]);
		}
	} else if ( (text_output || was_text_output) && !was_grid_output ) {
		for ( int i=resultset ; i>= 0 ; i-- ) {
			outputCols[i].clear();
		}
	}

	was_grid_output = FALSE;
	was_text_output = FALSE;

	resultset = -1;
	outputField->hide();
	outputText->clear();
	outputText->hide();
	winTab->setCurrentTab(grid_id);
	messageText->clear();
	messageText->hide();
	parser = FALSE;
}

void KSqlAnalyzerView::resizeOutput(bool all)
{
	if ( !all ) {
		if ( cnt > 1 ) {
			if ( s2height < outputField->height()  )
				s2height = outputField->height();
			else
				s2height = 500;
		} else {
			s2height = outputField->height();
		}
		s2->resize(outputField->width()-18, s2height);
		s2->setSizes(s2sizes);
	}
	startTimer(10);
}

//----------------------------------------------------------------------------------------

void KSqlAnalyzerView::parseQuery()
{
	runQuery(TRUE);
}

void KSqlAnalyzerView::runQuery() {
	runQuery(FALSE);
}

void KSqlAnalyzerView::runQuery(bool parse_only)
{
	if ( all_canceled )
		return;

	if ( !query_ready )
		return;

	if ( returnQuery().stripWhiteSpace().isEmpty() || returnQuery().stripWhiteSpace() == "" )
		return;

	query_ready = FALSE;

	// TSQ freigeben
	tsql->break_run = FALSE;

	// Ausfhren der Query des aktuellen Fensters
	statBar->changeItem((i18n("Executing query batch...")),1);
	clearResults();
	accel->setEnabled(FALSE);

	emit queryRunning(FALSE);
	this->setCursor(QCursor::WaitCursor);

	QString query;
	query = "";
	if ( parse_only ) {
		query = "SET NOEXEC ON\n";
		query += "SET PARSEONLY ON\n";
		tsql->tsql_doQuery(query);
		parser = TRUE;
	}
	tsql->tsql_doQuery(returnQuery());
	if ( parse_only ) {
		query = "\nSET NOEXEC OFF\n";
		query += "SET PARSEONLY OFF\n";
		tsql->tsql_doQuery(query);
	}

	// Wait at least 0.15 seconds to allow the next query...
	QTimer::singleShot( 150, this, SLOT(freeQuery()) );
}

void KSqlAnalyzerView::stopQuery()
{
	tsql->break_run = TRUE;
}

void KSqlAnalyzerView::columnsReceived(int num, QStringList &columns)
{
	if ( all_canceled )
		return;

	showOutputWindow(true);
	setColumns(columns);
}

void KSqlAnalyzerView::rowReceived(QValueList<QStringList> &row)
{
	if ( all_canceled )
		return;

	insertRow(row);
}

void KSqlAnalyzerView::queryReady(int rows, int time_elapsed)
{
	is_init = FALSE;

	if ( all_canceled )
		return;

	statBar->clear();
	statBar->changeItem(i18n("Ready."),1);

	QString row_str, time_str;
	row_str.sprintf(i18n("Rows: %d"), rows);
	time_str.sprintf(i18n("Time elapsed: %dms"), time_elapsed);

	statBar->changeItem(row_str,2);
	statBar->changeItem(time_str,3);

	resizeOutput(TRUE);

	emit queryRunning(TRUE);

	accel->setEnabled(TRUE);

	if ( !rows )
		winTab->setCurrentTab(msg_id);

	QTimer::singleShot(200, this, SLOT(setReadyText()));
	this->setCursor(QCursor::ArrowCursor);
	inputField->setCursor(QCursor::ArrowCursor);
	outputText->setCursorPosition(0,0);

	if ( resultset < 0 ) {
		outputField->hide();
	}
}

// TSQLCont hat einen Fehler zurckgeliefert
void KSqlAnalyzerView::queryError(const QString &error)
{
	if ( all_canceled )
		return;

	if ( parser )
		messageText->clear();

	QString msg_str;
	msg_str = QString("\n%1\n").arg(error);
debug("\n%s\n", (const char*)msg_str);

	messageText->insert(msg_str);

	statBar->clear();
	statBar->changeItem(i18n("Query batch completed with errors."),1);
	switchOutput(msg_id);
	winTab->setCurrentTab(msg_id);
}

void KSqlAnalyzerView::tableInfoReceived(QStringList &tableInfo)
{
	prepareTable(tableInfo);
}

// Communication with the DB-Server
void KSqlAnalyzerView::fatal_error(const QString &output)
{
	QMessageBox::information(this, I18N_NOOP("An error occoured !"), output);
	query_ready = TRUE;
}

// Communication with the DB-Server
void KSqlAnalyzerView::normal_error(const QString &output)
{
	query_ready = TRUE;
}

void KSqlAnalyzerView::timerEvent(QTimerEvent *te)
{
//	s2->resize(outputField->width()-18, s2height);
//	s2->setSizes(s2sizes);
	killTimers();
}

void KSqlAnalyzerView::storeDBList(QValueList<QStringList> &all_rows)
{
	if ( all_canceled )
		return;

	dbList.clear();
	dbList = all_rows;

	QTimer::singleShot(50, this, SLOT(emitServerInfo()));
	QTimer::singleShot(200, this, SIGNAL(emitDb()));
}

void KSqlAnalyzerView::setDBName(QValueList<QStringList> &list)
{
	QStringList dblist;
	for ( QValueList<QStringList>::Iterator it=list.begin() ; it!=list.end() ; ++it) {
		dblist = *it;
		default_db = dblist.first();
	}
}

QValueList<QStringList> & KSqlAnalyzerView::getDBList()
{
	return dbList;
}

QStringList KSqlAnalyzerView::getDBNames()
{
	if ( all_canceled )
		return NULL;

	QStringList tmp, tmp2;
	for ( QValueList<QStringList>::Iterator it=dbList.begin() ; it != dbList.end() ; ++it) {
		tmp = *it;
		for ( QStringList::Iterator it2=tmp.begin() ; it2 != tmp.end() ; ++it2) {
			tmp2.append((*it2).latin1());
		}
	}
	return tmp2;
}

void KSqlAnalyzerView::freeQuery()
{
	query_ready = TRUE;
}

void KSqlAnalyzerView::changeDb(const QString & newdb)
{
	if ( all_canceled )
		return;

	db = newdb;
	QString tmp;
	disconnect(tsql, SIGNAL(tsql_tableinfo_received(QStringList &)),
				 this, SLOT(tableInfoReceived(QStringList &)));
	disconnect(tsql, SIGNAL(tsql_querycolumns_received(int, QStringList &)),
				 this, SLOT(columnsReceived(int, QStringList &)));
	disconnect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
				 this, SLOT(rowReceived(QValueList<QStringList> &)));
	disconnect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(queryReady(int, int)));
	disconnect(tsql, SIGNAL(tsql_error(const QString &)), this , SLOT(queryError(const QString &)));
	disconnect(tsql, SIGNAL(tsql_dbchange(const QString &)),
				 this, SIGNAL(man_dbchange(const QString &)));

	tmp = QString("use [%1]").arg(newdb);
	default_db = newdb;
	tsql->tsql_doQuery(tmp);

	connect(tsql, SIGNAL(tsql_tableinfo_received(QStringList &)),
				 this, SLOT(tableInfoReceived(QStringList &)));
	connect(tsql, SIGNAL(tsql_querycolumns_received(int, QStringList &)),
				 this, SLOT(columnsReceived(int, QStringList &)));
	connect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
				 this, SLOT(rowReceived(QValueList<QStringList> &)));
	connect(tsql, SIGNAL(tsql_queryReady(int, int)), this , SLOT(queryReady(int, int)));
	connect(tsql, SIGNAL(tsql_error(const QString &)), this , SLOT(queryError(const QString &)));
	connect(tsql, SIGNAL(tsql_dbchange(const QString &)),
				 this, SIGNAL(man_dbchange(const QString &)));
}

void KSqlAnalyzerView::setLoginInfo(QStringList &linfo)
{
	if ( all_canceled )
		return;

	int lcnt = 0;
	for ( QStringList::Iterator it=linfo.begin() ; it != linfo.end() ; ++it ) {
		switch ( lcnt ) {
			case 0:
				server = (*it).latin1();
				break;
			case 1:
				login = (*it).latin1();
				break;
			case 2:
				pass = (*it).latin1();
				break;
			case 3:
				db = (*it).latin1();
				break;
			case 4:
				dbuser = (*it).latin1();
				break;
		}
		lcnt++;
	}
	setServerData(server, login, pass, FALSE);
}

QStringList KSqlAnalyzerView::getLoginInfo()
{
	QStringList linfo;
	linfo.append(server);
	linfo.append(login);
	linfo.append(pass);
	linfo.append(db);
	linfo.append(dbuser);
	return(linfo);
}

const QString & KSqlAnalyzerView::getDbName()
{
	return db;
}

void KSqlAnalyzerView::emitServerInfo()
{
	emit serverInfo(server, login, pass, port);
}

void KSqlAnalyzerView::loginCanceled()
{
	QTimer::singleShot(0, this, SIGNAL(closeIt()));
}

void KSqlAnalyzerView::showTableContext(int row, int col, const QPoint &pos)
{
	QTable *tab = (QTable*)sender();
	clickTxt = tab->text(row,col);

	popup_focus = TRUE;
	KPopupMenu *tabPop = new KPopupMenu(this);
	tabPop->insertItem (i18n("Copy"), this, SLOT(copySelection()), 0L);
	tabPop->insertItem (i18n("Select All"), this, SLOT(selectAll()), 0L);
	tabPop->insertSeparator();
	tabPop->insertItem (i18n("Show Data"), this, SLOT(showData()), 0L);
	tabPop->insertSeparator();
	tabPop->insertItem (i18n("Save as..."), this, SLOT(saveOutput()), 0L);
	tabPop->popup(pos);
	QTimer::singleShot(200, this, SLOT(resetFocus()));
}

void KSqlAnalyzerView::resetFocus()
{
	popup_focus = FALSE;
}

void KSqlAnalyzerView::initSelection()
{
	QTable *contTab;
	bool table_found = FALSE;
	QString tmp;
	int last_row = -1;
	curSelection = "";

	for ( int i=0 ; i <= resultset ; i++ ) {
		if ( this->sender() == outputList[i] ) {
			contTab = outputList[i];
			table_found = TRUE;
		}
	}

	if ( !table_found )
		return;

	int num_rows = contTab->numRows();
	int num_cols = contTab->numCols();

	for ( int r=0 ; r <= num_rows ; r++ ) {
		for ( int c=0 ; c <= num_cols ; c++ ) {
			if ( contTab->isSelected(r,c) ) {
				tmp = QString("%1").arg(contTab->text(r,c));
				if ( last_row > -1 && last_row == r )
					curSelection += "\t";
				else if ( last_row > -1 && last_row != r )
					curSelection += "\n";
				curSelection += tmp;
				last_row = r;
			}
		}
	}
}

void KSqlAnalyzerView::copySelection()
{
	qApp->clipboard()->setText(curSelection, QClipboard::Clipboard);
}

void KSqlAnalyzerView::selectAll()
{
	for ( int i=0 ; i <= resultset ; i++ ) {
		if ( outputList[i]->hasFocus() ) {
			outputList[i]->selectCells (0,0,outputList[i]->numRows(), outputList[i]->numCols());
		}
	}
}

void KSqlAnalyzerView::removeSelection(QTable *tab)
{
	if ( popup_focus )
		return;

	tab->clearSelection();
}

void KSqlAnalyzerView::setTextOutput(bool to)
{
	if ( text_output == to )
		return;

	if ( to )
		was_grid_output = TRUE;
	else
		was_text_output = TRUE;

	text_output = to;
}

bool KSqlAnalyzerView::isModified()
{
	return modified;
}

void KSqlAnalyzerView::setModified(bool m)
{
	modified = m;
}

void KSqlAnalyzerView::sql_message(const QString &msg)
{
	QString msg_str;
	msg_str = QString("\n%1\n").arg(msg);
	messageText->insert(msg_str);
}

void KSqlAnalyzerView::switchOutput(int tab_id)
{
	if ( is_init )
		return;

	is_error = TRUE;
	if ( tab_id == grid_id ) {
		messageText->hide();
		outputField->show();
	} else {
		messageText->show();
	  messageText->resize(this->width(), this->height()/2);
		outputField->hide();
	}
}

void KSqlAnalyzerView::showData()
{
	MyDataViewer *dataView = new MyDataViewer(this);
	dataView->resize(450,500);

	QPixmap px;
	int type = tsql->getData(clickTxt, &px);

	switch ( type ) {
		case TXT:	// Text
			dataView->setText(clickTxt);
			break;
		case PIC:	// Picture
			dataView->setPixmap(px);
			break;
		case BIN: // Binary data
			dataView->setText(clickTxt);
debug("is Binary!!!");
			break;
	}
	dataView->exec();
}

void KSqlAnalyzerView::doSearch()
{
	inputField->search();
}

void KSqlAnalyzerView::doReplace()
{
	inputField->replace();
}

int KSqlAnalyzerView::focusWidg()
{
	if ( inputField->hasFocus() )
		return 0;

	for ( int i=0 ; i <= resultset ; i++ ) {
		if ( outputList[i]->hasFocus() )
			return 1;
	}
	return 0;
}

void KSqlAnalyzerView::setReadyText()
{
	if ( parser || (messageText->text().stripWhiteSpace().isEmpty() || messageText->text().stripWhiteSpace() == "" ) )
		messageText->insert(i18n("The command(s) completed successfuly."));
}

void KSqlAnalyzerView::saveOutput()
{
	int rows, cols;
	QString toSave = "";

	for ( int i=0 ; i <= resultset ; i++ ) {
		if ( outputList[i]->hasFocus() ) {
			rows = outputList[i]->numRows();
			cols = outputList[i]->numCols();
			for ( int r=0 ; r < rows ; r++ ) {
				for ( int c=0 ; c < cols ; c++ ) {
					toSave += outputList[i]->text(r,c).right(512);
					if ( c < cols-1 )
						toSave += ",";
				}
				toSave += "\n";
			}

			KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
				i18n("*.csv|Export Files (*.csv)\n*|All files (*)"), this, i18n("Save as..."));
			if(!url.isEmpty()) {
				QString filename;
				filename = url.encodedPathAndQuery();

				QFile f( filename );
				if ( f.exists() ) {
					switch ( QMessageBox::information(this, i18n("File exists ?"),
						i18n("Do you want to overwrite the file ?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Abort) ) {
						case QMessageBox::Yes:
							break;
						case QMessageBox::No:
						case QMessageBox::Abort:
							return;
							break;
					}
				}
				if ( !f.open( IO_WriteOnly ) ) {
					KMessageBox::error (this,i18n("Could not save the file !"), i18n("I/O Error !"));
					return;
				}
				QTextStream stream( &f );
				stream << toSave;
				f.close();
			}
			return;
		}
	}
}
