/***************************************************************************
                          ksqlanalyzerview.h  -  description
                             -------------------
    begin                : Sun Jan 26 13:05:56 CET 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSQLANALYZERVIEW_H
#define KSQLANALYZERVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt
#include <qwidget.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qvaluelist.h>
#include <qtimer.h>
#include<qdatetime.h>
#include <qcursor.h>
#include <qpaintdevicemetrics.h>
#include <qclipboard.h>
#include <qtabbar.h>

// KDE-Includes
#include <ktextedit.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <kmessagebox.h>

// Own includes
#include "myscrollview.h"
#include "mydataviewer.h"
#include "tsqlcont.h"
#include "logindialog.h"
#include <mytable.h>

#include "kwrite/kwview.h"

class KSqlAnalyzerDoc;

class KSqlAnalyzerView : public QWidget
{
  Q_OBJECT

friend class KSqlAnalyzerDoc;

public:
    KSqlAnalyzerView(KSqlAnalyzerDoc* pDoc, bool new_conn, QWidget* parent, const char *name, int wflags);
    ~KSqlAnalyzerView();

	KWrite *inputField;
	MyScrollView *outputField;
	KSqlAnalyzerDoc *getDocument() const;
	void update(KSqlAnalyzerView* pSender);
	void print(QPrinter *pPrinter);
	QString returnQuery();
	void showOutputWindow(bool);
	void setColumns(QStringList &);
	void insertRow(QValueList<QStringList> &);
	void clearResults();
	void prepareTable(QStringList &);
	QValueList<QStringList> &getDBList();
	QStringList getDBNames();
	QStringList getLoginInfo();
	void setLoginInfo(QStringList &);
	const QString & getDbName();
	void changeDb(const QString &);
	QString getSaveText();
	void setText(const QString &);
	void setTextOutput(bool);
	bool isModified();
	void setModified(bool);
	QString getDefaultDB() { return default_db; };
	void doSearch();
	void doReplace();
	int focusWidg();
	bool isTextOutput();

protected:
	virtual void closeEvent(QCloseEvent* e);
	virtual void resizeEvent(QResizeEvent *re);
	virtual void timerEvent(QTimerEvent *te);
	KSqlAnalyzerDoc *doc;

private:
	QSplitter *s1, *s2;
	MyTable *outputList[2481];
	QString clickTxt;
	QStringList outputCols[2481];

	QValueList<QStringList> dbList;
	QString curSelection;

	QTabBar *winTab;

	QString server, login, pass, db, dbuser, default_db;
	int port, grid_id, msg_id;
	bool conn_error, conn_ready, timer_started, new_connection, all_canceled,
				popup_focus, text_output, was_grid_output, was_text_output,
				modified, is_error, parser;
	LoginDialog *loginDialog;
	TSQLCont *tsql;
	bool query_ready, is_init;
	int resultset, num_rows, s2height, cnt;
	void createOutputField();
	QValueList<int> s2sizes;
	QAccel *accel;
	KStatusBar *statBar;
	KTextEdit *outputText, *messageText;

	/** opens the Login-Screen */
	void openLoginDialog();
	void startSession();

public slots:
	void resizeOutput(bool);
	void runQuery();
	void runQuery(bool parse_only);
	void parseQuery();
	void stopQuery();
	void copySelection();

private slots:
	void focusIt();
	void resetFocus();
	void showCurPos();
	void showOverwriteMode(int);
	void setDBName(QValueList<QStringList> &);
	void setReadyText();

	void setServerData(QString, QString, QString, bool new_conn=TRUE);
	void columnsReceived(int, QStringList &);
	void rowReceived(QValueList<QStringList> &);
	void queryReady(int, int);
	void fatal_error(const QString &);
	void sql_message(const QString &);
	void normal_error(const QString &);
	void tableInfoReceived(QStringList &);
	void storeDBList(QValueList<QStringList> &);
	void loginCanceled();
	void showTableContext(int row, int col, const QPoint &pos);
	void initSelection();
	void removeSelection(QTable *);
	void selectAll();
	void saveOutput();

	/** Query-Actions **/
	void firstSessionReady(int, int);
	void secondSessionReady(int, int);
	void queryError(const QString &);
	void freeQuery();
	void emitServerInfo();
	void switchOutput(int);
	void showData();

signals:
	void dbReceived(QValueList<QStringList> &);
	void toggleAppBar();
	void emitDb();
	void man_dbchange(const QString &);
	void queryRunning(bool);
	void serverInfo(const QString &, const QString &, const QString &, int); //, QValueList<QStringList>);
	void closeIt();
	void textModified(bool);
};

#endif // KSQLANALYZERVIEW_H
