#include "highlight.h"

#include "kwdoc.h"

//#include <kapp.h>
#include <kcharsets.h>
#include <kfontdialog.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kmimemagic.h>

#include <qfile.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qlist.h>
#include <qgrpbox.h>
#include <qtstream.h>
//#include <qregexp.h>
#include <qlabel.h>
#include <qstringlist.h>
#include <qfileinfo.h>
#include <kbuttonbox.h>


//#include <X11/Xlib.h> //used in getXFontList()

// ISO/IEC 9899:1990 (aka ANSI C)
// "interrupt" isnt an ANSI keyword, but an extension of some C compilers
/*
const char *cKeywords[] = {
  "break", "case", "continue", "default", "do", "else", "enum", "extern",
  "for", "goto", "if", "interrupt",  "return", "sizeof", "struct", "switch",
  "typedef", "union", "while", 0L};
*/

const char *sqlKeywords[] = {
	"WRITETEXT","WITH","WHILE","WHERE","WHEN","WAITFOR","VIEW","VARYING","VALUES",
	"USER","USE","UPDATETEXT","UPDATE","UNIQUE","UNION","TSEQUAL","TRUNCATE","TRIGGER",
	"TRANSACTION","TRAN","TOP","TO","THEN","TEXTSIZE","TABLE","SYSTEM_USER","STATISTICS",
	"SOME","SHUTDOWN","SETUSER","SET","SESSION_USER","SELECT","SCHEMA","SAVE","RULE","ROWGUIDCOL",
	"ROWCOUNT","ROLLBACK","RIGHT","REVOKE","RETURN","RESTRICT","RESTORE","REPLICATION","REFERENCES",
	"RECONFIGURE","READTEXT","READ","RAISERROR","PUBLIC","PROCEDURE","PROC","PRINT","PRIMARY","PRECISION",
	"PLAN","PERCENT","OVER","OUTER","ORDER","OR","OPTION","OPENXML","OPENROWSET","OPENQUERY","OPENDATASOURCE",
	"OPEN","ON","OFFSETS","OFF","OF","NULLIF","NULL","NOT","NONCLUSTERED","NOCHECK","NATIONAL","LOAD",
	"LINENO","LIKE","LEFT","KILL","KEY","JOIN","IS","INTO","INTERSECT","INSERT","INNER","INDEX","IN",
	"IF","IDENTITYCOL","IDENTITY_INSERT","IDENTITY","HOLDLOCK","HAVING","GROUP","GRANT","GOTO","FUNCTION",
	"FULL","FROM","FREETEXTTABLE","FREETEXT","FOREIGN","FOR","FILLFACTOR","FILE","FETCH","EXIT","EXISTS",
	"EXECUTE","EXEC","EXCEPT","ESCAPE","ERRLVL","END","ELSE","DUMP","DUMMY","DROP","DOUBLE","DISTRIBUTED",
	"DISTINCT","DISK","DESC","DENY","DELETE","DEFAULT","DECLARE","DEALLOCATE","DBCC","DATABASE","CURSOR",
	"CURRENT_USER","CURRENT_TIMESTAMP","CURRENT_TIME","CURRENT_DATE","CURRENT","CROSS","CREATE","CONVERT",
	"CONTINUE","CONTAINSTABLE","CONTAINS","CONSTRAINT","COMPUTE","COMMIT","COLUMN","COLLATE","COALESCE",
	"CLUSTERED","CLOSE","CHECKIDENT","CHECKPOINT","CHECK","CASE","CASCADE","BY","BULK","BROWSE","BREAK","BETWEEN","BEGIN",
	"BACKUP","AUTHORIZATION","ASC","AS","ANY","AND","ALTER","ALL","ADD", 0L};

const char *sqlTypes[] = {
	"bigint","Binary","bit","char","cursor","datetime","Decimal","float","image",
	"int","money","Nchar","ntext","nvarchar","real","smalldatetime","Smallint",
	"smallmoney","text","timestamp","tinyint","Varbinary","Varchar","uniqueidentifier",0L};

const char *sqlFunctions[] = {
"sysaltfiles","syslockinfo","syscacheobjects","syslogins","syscharsets",
"sysmessages","sysconfigures","sysoledbusers","syscurconfigs","sysperfinfo",
"sysdatabases","sysprocesses","sysdevices","sysremotelogins","syslanguages",
"sysservers","syscolumns","sysindexkeys","syscomments","sysmembers",
"sysconstraints","sysobjects","sysdepends","syspermissions","sysfilegroups",
"sysprotects","sysfiles","sysreferences","sysforeignkeys","systypes",
"sysfulltextcatalogs","sysusers","sysindexes","sysalerts","sysjobsteps",
"syscategories","sysnotifications","sysdownloadlist","sysoperators","sysjobhistory",
"systargetservergroupmembers","sysjobs","systargetservergroups","sysjobschedules","systargetservers",
"sysjobservers","systaskids","sysdatabases","sysservers","sysreplicationalerts",
"sysmergearticles","sysmergepublications","sysmergeschemachange","sysmergesubscriptions","sysmergesubsetfilters",
"sysarticles","syspublications","sysarticleupdates","syssubscriptions",0L};

const char fontSizes[] = {4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,22,24,26,28,32,48,64,0};

//default item style indexes
const int dsNormal = 0;
const int dsKeyword = 1;
const int dsDataType = 2;
const int dsDecVal = 3;
const int dsBaseN = 4;
const int dsFloat = 5;
const int dsChar = 6;
const int dsString = 7;
const int dsComment = 8;
const int dsOthers = 9;


bool testWw(char c) {
  static unsigned char data[] = {0,0,0,0,0,0,255,3,254,255,255,135,254,255,255,7};
  if (c & 128) return false;
  return !(data[c >> 3] & (1 << (c & 7)));
}

// ---------------------------------------------------------------------------

HlItem::HlItem(int attribute, int context)
  : attr(attribute), ctx(context) {
}

// ---------------------------------------------------------------------------
HlItemWw::HlItemWw(int attribute, int context)
  : HlItem(attribute,context) {
}
// ---------------------------------------------------------------------------

HlCharDetect::HlCharDetect(int attribute, int context, char c)
  : HlItem(attribute,context), sChar(c) {
}

const char *HlCharDetect::checkHgl(const char *str) {
    if (*str == sChar) return str + 1;
   return 0L;
  

}

// ---------------------------------------------------------------------------

Hl2CharDetect::Hl2CharDetect(int attribute, int context, const char *s)
  : HlItem(attribute,context) {
  sChar[0] = s[0];
  sChar[1] = s[1];
}

const char *Hl2CharDetect::checkHgl(const char *str) {
  if (str[0] == sChar[0] && str[1] == sChar[1]) return str + 2;
  return 0L;
}

// ---------------------------------------------------------------------------

KeywordData::KeywordData(const char *str) {
  len = strlen(str);
  s = new char[len];
  memcpy(s,str,len);
}

KeywordData::~KeywordData() {
  delete s;
}

// ---------------------------------------------------------------------------

HlKeyword::HlKeyword(int attribute, int context)
  : HlItemWw(attribute,context) {
  words.setAutoDelete(true);
}

HlKeyword::~HlKeyword() {
}


void HlKeyword::addWord(const char *s) {
  KeywordData *word;
  word = new KeywordData(s);
  words.append(word);
}

void HlKeyword::addList(const char **list) {

  while (*list) {
    addWord(*list);
    list++;
  }
}

const char *HlKeyword::checkHgl(const char *s) {
 int z, count, len;
  KeywordData *word;

  count = words.count();
  len = strlen(s);
  for (z = 0; z < count; z++) {
      word = words.at(z);
      if (len>=word->len && memcmp(s,word->s,word->len) == 0) {
          return s + word->len;
      }
  }
  return 0L;
}

// ---------------------------------------------------------------------------

HlCaseInsensitiveKeyword::HlCaseInsensitiveKeyword(int attribute, int context)
  : HlKeyword(attribute,context) {
}

HlCaseInsensitiveKeyword::~HlCaseInsensitiveKeyword() {
}


const char *HlCaseInsensitiveKeyword::checkHgl(const char *s) {
  int z, count;
  KeywordData *word;

  count = words.count();
  for (z = 0; z < count; z++) {
    word = words.at(z);
    if (strncasecmp(s,word->s,word->len) == 0) {
      return s + word->len;
    }
  }
  return 0L;
}

// ---------------------------------------------------------------------------
/*
HlInt::HlInt(int attribute, int context)
  : HlItemWw(attribute,context) {
}

const char *HlInt::checkHgl(const char *str) {
  const char *s;

  s = str;
  while (*s >= '0' && *s <= '9') s++;
  if (s > str) return s;
  return 0L;
}

// ---------------------------------------------------------------------------

HlFloat::HlFloat(int attribute, int context)
  : HlItemWw(attribute,context) {
}

const char *HlFloat::checkHgl(const char *s) {
  bool b, p;

  b = false;
  while (*s >= '0' && *s <= '9') {
    s++;
    b = true;
  }
  if (p = (*s == '.')) {
    s++;
    while (*s >= '0' && *s <= '9') {
      s++;
      b = true;
    }
  }
  if (!b) return 0L;
  if (*s == 'E' || *s == 'e') s++; else return (p) ? s : 0L;
  if (*s == '-') s++;
  b = false;
  while (*s >= '0' && *s <= '9') {
    s++;
    b = true;
  }
  if (b) return s; else return 0L;
}

// ---------------------------------------------------------------------------

HlCInt::HlCInt(int attribute, int context)
  : HlInt(attribute,context) {
}

const char *HlCInt::checkHgl(const char *s) {

  if (*s == '0') s++; else s = HlInt::checkHgl(s);
  if (s && (*s == 'L' || *s == 'l' || *s == 'U' || *s == 'u')) s++;
  return s;
}

// ---------------------------------------------------------------------------

HlCOct::HlCOct(int attribute, int context)
  : HlItemWw(attribute,context) {
}

const char *HlCOct::checkHgl(const char *str) {
  const char *s;

  if (*str == '0') {
    str++;
    s = str;
    while (*s >= '0' && *s <= '7') s++;
    if (s > str) {
      if (*s == 'L' || *s == 'l' || *s == 'U' || *s == 'u') s++;
      return s;
    }
  }
  return 0L;
}

// ---------------------------------------------------------------------------

HlCHex::HlCHex(int attribute, int context)
  : HlItemWw(attribute,context) {
}

const char *HlCHex::checkHgl(const char *str) {
  const char *s;

  if (str[0] == '0' && (str[1] == 'x' || str[1] == 'X')) {
    str += 2;
    s = str;
    while ((*s >= '0' && *s <= '9') || (*s >= 'A' && *s <= 'F') || (*s >= 'a' && *s <= 'f')) s++;
    if (s > str) {
      if (*s == 'L' || *s == 'l' || *s == 'U' || *s == 'u') s++;
      return s;
    }
  }
  return 0L;
}
 
// ---------------------------------------------------------------------------

HlCFloat::HlCFloat(int attribute, int context)
  : HlFloat(attribute,context) {
}

const char *HlCFloat::checkHgl(const char *s) {

  s = HlFloat::checkHgl(s);
  if (s && (*s == 'F' || *s == 'f')) s++;
  return s;
}
*/

//checks for hex and oct (for example \x1b or \033)
const char *checkCharHexOct(const char *str) {
  const char *s;
  int n;

  s = str;
  if (*s == 'x') {
    n = 0;
    do {
      s++;
      n *= 16;
      if (*s >= '0' && *s <= '9') n += *s - '0';
      else if (*s >= 'A' && *s <= 'F') n += *s - 'A' + 10;
      else if (*s >= 'a' && *s <= 'f') n += *s - 'a' + 10;
      else break;
      if (n >= 256) return 0L;
    } while (true);
    if (s - str == 1) return 0L;
  } else {
    if (!(*s >= '0' && *s <= '7')) return 0L;
    n = *s - '0';
    do {
      s++;
      n *= 8;
      if (*s >= '0' && *s <= '7') n += *s - '0'; else break;
      if (n >= 256) return s;
    } while (s - str < 3);
  }
  return s;
}

//checks for C escape chars like \n
const char *checkEscapedChar(const char *s) {

  if (s[0] == '\\' && s[1] != 0) {
    s++;
    if (strchr("abefnrtv\"\'\\",*s)) {
      s++;
    } else return checkCharHexOct(s);
    return s;
  }
  return 0L;
}

// ---------------------------------------------------------------------------

ItemStyle::ItemStyle() : selCol(Qt::white), bold(false), italic(false) {
}

ItemStyle::ItemStyle(const QColor &col, const QColor &selCol,
  bool bold, bool italic)
  : col(col), selCol(selCol), bold(bold), italic(italic) {
}

// ---------------------------------------------------------------------------

ItemFont::ItemFont() :
  family("courier"),
  size(12),
  charset("")
{
}

// ---------------------------------------------------------------------------

ItemData::ItemData(const QString& name, int defStyleNum) :
  name(name),
  defStyleNum(defStyleNum),
  defStyle(true),
  defFont(true)
{
}

ItemData::ItemData(const QString& name, int defStyleNum,
                    const QColor &col, const QColor &selCol,
                    bool bold, bool italic) :
  ItemStyle(col,selCol,bold,italic),
  name(name),
  defStyleNum(defStyleNum),
  defStyle(false),
  defFont(true) {
}

// ---------------------------------------------------------------------------

HlData::HlData(const QString& wildcards, const QString& mimetypes) :
  wildcards(wildcards),
  mimetypes(mimetypes)
{
  itemDataList.setAutoDelete(true);
}

// ---------------------------------------------------------------------------

Highlight::Highlight(const QString& name) :
	iName(name),
	refCount(0)
{
}

Highlight::~Highlight() {
}

KConfig *Highlight::getKConfig() {
  KConfig *config = KGlobal::config();
  config->setGroup((QString) iName + " Highlight");
  return config;
}

QString Highlight::getWildcards()
{
  KConfig *config = getKConfig();
	if (config)
  	return config->readEntry("Wildcards", dw);
	return dw;
}


QString Highlight::getMimetypes()
{
  KConfig *config = getKConfig();
	if (config)
	  return config->readEntry("Mimetypes", dm);
	return dm;
}


HlData *Highlight::getData() {
  KConfig *config;
  HlData *hlData;

  config = getKConfig();

  hlData = new HlData(
    config->readEntry("Wildcards",dw),config->readEntry("Mimetypes",dm));
  getItemDataList(hlData->itemDataList,config);
  return hlData;
}

void Highlight::setData(HlData *hlData) {
  KConfig *config;

  config = getKConfig();

  config->writeEntry("Wildcards",hlData->wildcards);
  config->writeEntry("Mimetypes",hlData->mimetypes);

  setItemDataList(hlData->itemDataList,config);
}

void Highlight::getItemDataList(ItemDataList &list) {
  KConfig *config;

  config = getKConfig();
  getItemDataList(list,config);
}

void Highlight::getItemDataList(ItemDataList &list, KConfig *config) {
  ItemData *p;
  QString s;
  QRgb col, selCol;
  char family[96];
  char charset[48];

  list.clear();
  list.setAutoDelete(true);
  createItemData(list);

  for (p = list.first(); p != 0L; p = list.next()) {
    s = config->readEntry(p->name);
    p->size=p->printSize=10;
    if (!s.isEmpty()) {
      sscanf(s,"%d,%X,%X,%d,%d,%d,%95[^,],%d,%47[^,],%d",
        &p->defStyle,&col,&selCol,&p->bold,&p->italic,
        &p->defFont,family,&p->size,charset, &p->printSize);
      p->col.setRgb(col);
      p->selCol.setRgb(selCol);
      p->family = family;
      p->charset = charset;
    }
  }
}

void Highlight::setItemDataList(ItemDataList &list, KConfig *config) {
  ItemData *p;
  char s[200];

  for (p = list.first(); p != 0L; p = list.next()) {
    sprintf(s,"%d,%X,%X,%d,%d,%d,%1.95s,%d,%1.47s,%d",
      p->defStyle,p->col.rgb(),p->selCol.rgb(),p->bold,p->italic,
      p->defFont,p->family.data(),p->size,p->charset.data(),p->printSize);
    config->writeEntry(p->name,s);
  }
}

QString Highlight::name() {
  return iName;
}

void Highlight::use() {
  if (refCount == 0) init();
  refCount++;
}

void Highlight::release() {
  refCount--;
  if (refCount == 0) done();
}

int Highlight::doHighlight(int, TextLine *textLine) {

  if (textLine)
  {
    textLine->setAttribs(0,0,textLine->length());
    textLine->setAttr(0);
  }
  return 0;
}

int Highlight::doPreHighlight( QList<TextLine> & /*contents*/ ) {
  return 0;
}

void Highlight::createItemData(ItemDataList &list) {

  list.append(new ItemData("Normal Text",dsNormal));
}


void Highlight::init() {
}

void Highlight::done() {
}

bool Highlight::containsFiletype(const QString& ext)
{
  QString w = getWildcards();
  if (w.isEmpty())
    return false;

  w = ";" + w + ";";
  return w.find(";*."+ext+ ";") != -1;
}

bool Highlight::containsMimetype(const QString& mimetype)
{
  QString w = getMimetypes();
  if (w.isEmpty())
    return false;

  w = ";" + w + ";";
  return w.find(";"+mimetype+ ";") != -1;
}

// ---------------------------------------------------------------------------

HlContext::HlContext(int attribute, int lineEndContext)
  : attr(attribute), ctx(lineEndContext) {
  items.setAutoDelete(true);
}

// ---------------------------------------------------------------------------

GenHighlight::GenHighlight(const char *name) : Highlight(name) {
  for ( int z = 0; z < nContexts; z++) contextList[z] = 0L;
}


int GenHighlight::doHighlight(int ctxNum, TextLine *textLine){
  if (!textLine) return 0;

  HlContext *context;
  const char *str, *s1, *s2;
  char lastChar;
  HlItem *item;

  context = contextList[ctxNum];

  str = textLine->getString();
  lastChar = 0;

  s1 = str;
  while (*s1) {
    for (item = context->items.first(); item != 0L; item = context->items.next()) {
      if (item->startEnable(lastChar) ) {
        s2 = item->checkHgl(s1);
        if (s2 > s1) {
          if (item->endEnable(*s2) ) {
            textLine->setAttribs(item->attr,s1 - str,s2 - str);
            ctxNum = item->ctx;
            context = contextList[ctxNum];
            s1 = s2 - 1;
            goto found;
          }
        }
      }
    }
    // nothing found: set attribute of one char
    textLine->setAttribs(context->attr,s1 - str,s1 - str + 1);

    found:
    lastChar = *s1;
    s1++;
  }
  //set "end of line"-properties
  textLine->setAttr(context->attr);
  //return new context
  return context->ctx;
}

int GenHighlight::doPreHighlight( QList<TextLine> &contents )
{
  HlContext *context;
  HlItem *item;
  TextLine *textLine;
  const char *str, *s1, *s2;
  char lastChar;
  int ctxNum= 0;

  context = contextList[0];

#define LOCAL_CONTEXT
#ifdef LOCAL_CONTEXT
  // cache only those context items locally that lead to a context switch
  // test only those possibilities that lead to a context switch
  HlContext *localContext[nContexts];
  for (int i= 0; i < nContexts; ++i )
  {
    if ( contextList[i] ) {
      HlContext *c;
      context = contextList[i];
      localContext[i] = c = new HlContext( context->attr, context->ctx );
      c->items.setAutoDelete(false);
      for (item = context->items.first(); item != 0L; item = context->items.next()) {
        if ( item->ctx == i )
          continue; // not interested in this one
        c->items.append(item);
      }
    } else
      localContext[i] = 0L;
  }
#endif

  int lastLine = (int) contents.count() -1;
  for (int line= 0; line < lastLine; ++line ) {
    textLine = contents.at(line);
    str = textLine->getString();
    s1 = str;
    lastChar = 0;

#ifdef LOCAL_CONTEXT
    context = localContext[ctxNum];
#else
    context = contextList[ctxNum];
#endif
    while (*s1) {
      for (item = context->items.first(); item != 0L; item = context->items.next()) {
        if (item->startEnable(lastChar) ) {
          s2 = item->checkHgl(s1);
          if (s2 > s1) {
            if (item->endEnable(*s2) ) {
              ctxNum = item->ctx;
#ifdef LOCAL_CONTEXT
              context = localContext[ctxNum];
#else
              context = contextList[ctxNum];
#endif
              s1 = s2 - 1;
              goto found;
            }
          }
        }
      }
found:
      lastChar = *s1;
      s1++;
    }
    //set "end of line"-properties
    ctxNum= context->ctx;
    textLine->setContext(ctxNum);
  }
#ifdef LOCAL_CONTEXT
  for (int i = 0; i < nContexts; ++i )
  {
    delete localContext[i];
    localContext[i] = 0L;
  }
#endif
  return 0;
}

void GenHighlight::init() {
  int z;

  for (z = 0; z < nContexts; z++) {
    delete contextList[z];
    contextList[z] = 0L;
  }
  makeContextList();
}

void GenHighlight::done() {
  int z;

  for (z = 0; z < nContexts; z++) { delete contextList[z]; contextList[z] = 0L; }
}

// ---------------------------------------------------------------------------

SqlHighlight::SqlHighlight(const QString& name) :
  GenHighlight(name)
{
  dw = "*.c";
  dm = "text/x-csrc";
}

SqlHighlight::~SqlHighlight() {
}

void SqlHighlight::createItemData(ItemDataList &list) {

  list.append(new ItemData("Normal Text",dsNormal));
  list.append(new ItemData("Keyword",dsKeyword));
  list.append(new ItemData("Data Type",dsDataType));
  list.append(new ItemData("Decimal",dsDecVal));
  list.append(new ItemData("Octal",dsBaseN));
  list.append(new ItemData("Hex",dsBaseN));
  list.append(new ItemData("Float",dsFloat));
  list.append(new ItemData("Char",dsChar));
  list.append(new ItemData("String",dsString));
  list.append(new ItemData("String Char",dsChar));
  list.append(new ItemData("Comment",dsComment));
  list.append(new ItemData("Preprocessor",dsOthers));
  list.append(new ItemData("Prep. Lib",dsOthers,Qt::darkYellow,Qt::yellow,false,false));
  list.append(new ItemData("Symbol",dsNormal));
}

void SqlHighlight::makeContextList() {
  HlContext *c;
  HlKeyword *keyword, *dataType, *sysfunc;

  //normal context
  contextList[0] = c = new HlContext(0,0);
    c->items.append(keyword = new HlCaseInsensitiveKeyword(1,0));
    c->items.append(dataType = new HlCaseInsensitiveKeyword(2,0));
    c->items.append(sysfunc = new HlCaseInsensitiveKeyword(4,0));
    c->items.append(new HlCharDetect(8,1,'\''));
    c->items.append(new Hl2CharDetect(10,2,"--"));
    c->items.append(new Hl2CharDetect(10,3,"/*"));

  //string context
  contextList[1] = c = new HlContext(8,1);
    c->items.append(new HlCharDetect(8,0,'\''));

  //one line comment context
  contextList[2] = new HlContext(10,0);

  //multi line comment context
  contextList[3] = c = new HlContext(10,3);
    c->items.append(new Hl2CharDetect(10,0,"*/"));

  setKeywords(keyword, dataType, sysfunc);
}

void SqlHighlight::setKeywords(HlKeyword *keyword, HlKeyword *dataType, HlKeyword *sqlFuncs)
{
  keyword->addList(sqlKeywords);
  dataType->addList(sqlTypes);
  sqlFuncs->addList(sqlFunctions);
}

// ---------------------------------------------------------------------------

HlManager::HlManager() : QObject(0L) {

  hlList.setAutoDelete(true);
  hlList.append(new SqlHighlight("C"));
}

HlManager::~HlManager() {
}

Highlight *HlManager::getHl(int n) {
  if (n < 0 || n >= (int) hlList.count())
    n = 0;
  return hlList.at(n);
}

int HlManager::defaultHl() {
  KConfig *config;

  config = KGlobal::config();
  config->setGroup("General Options");
  return nameFind(config->readEntry("Highlight"));
}


int HlManager::nameFind(const QString& name) {
  int z;

  for (z = hlList.count() - 1; z > 0; z--) {
    if (hlList.at(z)->iName == name)
      break;
  }
  return z;
}

int HlManager::findByFile(const QString& filename)
{
  if (!filename.isEmpty())
  {
    QFileInfo fileInfo(filename);
    QString ext = fileInfo.extension(false);

    if (!ext.isEmpty())
    {
      for (Highlight *highlight = hlList.first(); highlight != 0L; highlight = hlList.next())
      {
        if (highlight->containsFiletype(ext))
          return hlList.at();
      }
    }
  }
  return -1;
}

int HlManager::findByMimetype(const QString& filename)
{
  if (!filename.isEmpty())
  {
    KMimeMagic* magic = KMimeMagic::self();
    magic->setFollowLinks(true);
    KMimeMagicResult *result = magic->findFileType(filename);
    if (result && result->mimeType())
    {
      for (Highlight *highlight = hlList.first(); highlight != 0L; highlight = hlList.next())
      {
        if (highlight->containsMimetype(result->mimeType()))
          return hlList.at();
      }
    }
  }
  return -1;
}


int HlManager::getHighlight(const QString& filename)
{
  int hl;
  if ((hl = findByFile(filename)) != -1)
    return hl;

  if ((hl = findByMimetype(filename)) != -1)
    return hl;

  return defaultHl();
}


void HlManager::makeAttribs(Highlight *highlight, Attribute *a, int n) {
  ItemStyleList defaultStyleList;
  ItemStyle *defaultStyle;
  ItemFont defaultFont;
  ItemDataList itemDataList;
  ItemData *itemData;
  int z;
  QFont font, printFont;
  KCharsets * charsets = KGlobal::charsets();

  defaultStyleList.setAutoDelete(true);
  getDefaults(defaultStyleList,defaultFont);

  itemDataList.setAutoDelete(true);
  highlight->getItemDataList(itemDataList);
  for (z = 0; z < (int) itemDataList.count(); z++) {
    itemData = itemDataList.at(z);
    if (itemData->defStyle) {
      defaultStyle = defaultStyleList.at(itemData->defStyleNum);
      a[z].col = defaultStyle->col;
      a[z].selCol = defaultStyle->selCol;
      font.setBold(defaultStyle->bold);
      font.setItalic(defaultStyle->italic);
    } else {
      a[z].col = itemData->col;
      a[z].selCol = itemData->selCol;
      font.setBold(itemData->bold);
      font.setItalic(itemData->italic);
    }

    if (itemData->defFont) {
      font.setFamily(defaultFont.family);
      font.setPointSize(defaultFont.size);
      printFont=font;
      printFont.setPointSize(defaultFont.printSize);

    } else {
      font.setFamily(itemData->family);
      font.setPointSize(itemData->size);
      printFont=font;
      printFont.setPointSize(itemData->printSize);
    }

    a[z].setFont(font);
    a[z].setPrintFont(printFont);
  }
  for (; z < n; z++) {
    a[z].col = black;
    a[z].selCol = black;
    a[z].setFont(font);
    a[z].setPrintFont(printFont);
  }
}

int HlManager::defaultStyles() {
  return 10;
}

const char *HlManager::defaultStyleName(int n) {
  static const char *names[] = {
    "Normal","Keyword","Data Type","Decimal/Value","Base-N Integer",
    "Floating Point","Character","String","Comment","Others"};

  return names[n];
}

void HlManager::getDefaults(ItemStyleList &list, ItemFont &font) {
  KConfig *config;
  int z;
  ItemStyle *i;
  QString s;
  QRgb col, selCol;

  list.setAutoDelete(true);
  //ItemStyle(color, selected color, bold, italic)
  list.append(new ItemStyle(black,white,false,false));     //normal
  list.append(new ItemStyle(blue,white,false,false));      //keyword
  list.append(new ItemStyle(blue,white,false,false));   //datatype
  list.append(new ItemStyle(blue,cyan,false,false));       //decimal/value
  list.append(new ItemStyle(darkGreen,darkGreen,false,false));   //base n
  list.append(new ItemStyle(darkMagenta,cyan,false,false));//float
  list.append(new ItemStyle(magenta,magenta,false,false)); //char
  list.append(new ItemStyle(red,red,false,false));         //string
  list.append(new ItemStyle(darkGray,gray,false,false));    //comment
  list.append(new ItemStyle(darkGreen,green,false,false)); //others

  config = KGlobal::config();
  config->setGroup("Default Item Styles");
  for (z = 0; z < defaultStyles(); z++) {
    i = list.at(z);
    s = config->readEntry(defaultStyleName(z));
    if (!s.isEmpty()) {
      sscanf(s,"%X,%X,%d,%d",&col,&selCol,&i->bold,&i->italic);
      i->col.setRgb(col);
      i->selCol.setRgb(selCol);
    }
  }

  config->setGroup("Default Font");
  QFont defaultFont = KGlobalSettings::fixedFont();
  font.family = config->readEntry("Family", defaultFont.family());
  font.size = config->readNumEntry("Size", defaultFont.pointSize());
  font.printSize = config->readNumEntry("PrintSize", defaultFont.pointSize());
#if QT_VERSION < 300
  // ### doesn't compile with Qt3
  font.charset = config->readEntry("Charset", QFont::encodingName(QFont::ISO_8859_1));//  "ISO-8859-1");
#else
  font.charset = config->readEntry("Charset", "ISO-8859-1");
#endif
}

void HlManager::setDefaults(ItemStyleList &list, ItemFont &font) {
  KConfig *config;
  int z;
  ItemStyle *i;
  char s[64];

  config = KGlobal::config();
  config->setGroup("Default Item Styles");
  for (z = 0; z < defaultStyles(); z++) {
    i = list.at(z);
    sprintf(s,"%X,%X,%d,%d",i->col.rgb(),i->selCol.rgb(),i->bold, i->italic);
    config->writeEntry(defaultStyleName(z),s);
  }

  config->setGroup("Default Font");
  config->writeEntry("Family",font.family);
  config->writeEntry("Size",font.size);
  config->writeEntry("PrintSize",font.printSize);
  config->writeEntry("Charset",font.charset);

  emit changed();
}


int HlManager::highlights() {
  return (int) hlList.count();
}

const char *HlManager::hlName(int n) {
  return hlList.at(n)->iName;
}

void HlManager::getHlDataList(HlDataList &list) {
  int z;

  for (z = 0; z < (int) hlList.count(); z++) {
    list.append(hlList.at(z)->getData());
  }
}

void HlManager::setHlDataList(HlDataList &list) {
  int z;

  for (z = 0; z < (int) hlList.count(); z++) {
    hlList.at(z)->setData(list.at(z));
  }

  emit changed();
}

#include "highlight.moc"
