#ifndef _KWVIEV_H_
#define _KWVIEV_H_

#include <qpoint.h>

#include <qscrbar.h>
#include <qiodev.h>
#include <qdragobject.h>

#include <kpopupmenu.h>
#include <kconfig.h>

#include "kwdialog.h"

class KWriteDoc;
class Highlight;
class KURL;

//icon-paint
const int iconBorderWidth		= 16;
const int iconBorderHeight	= 800;

//search flags
const int sfCaseSensitive   = 1;
const int sfWholeWords      = 2;
const int sfFromCursor      = 4;
const int sfBackward        = 8;
const int sfSelected        = 16;
const int sfPrompt          = 32;
const int sfReplace         = 64;
const int sfAgain           = 128;
const int sfWrapped         = 256;
const int sfFinished        = 512;
//dialog results
const int srYes             = QDialog::Accepted;
const int srNo              = 10;
const int srAll             = 11;
const int srCancel          = QDialog::Rejected;

//config flags
const int cfAutoIndent        = 1;
const int cfBackspaceIndent   = 2;
const int cfWordWrap          = 4;
const int cfReplaceTabs       = 8;
const int cfRemoveSpaces      = 16;
const int cfWrapCursor        = 32;
const int cfAutoBrackets      = 64;

const int cfPersistent        = 128;
const int cfKeepSelection     = 256;
const int cfVerticalSelect    = 512;
const int cfDelOnInput        = 1024;
const int cfXorSelect         = 2048;

const int cfOvr               = 4096;
const int cfMark              = 8192;

const int cfAutoCopy          = 16384;

const int cfShowTabs          = 32768;
const int cfHighlightBrackets = 65536;
const int cfTabIndent         = 1L << 17;

const int cfIndentBraces      = 1L << 18;
const int cfIndentParentheses = 1L << 19;


//update flags
const int ufDocGeometry     = 1;
const int ufUpdateOnScroll  = 2;
const int ufPos             = 4;

//load flags
const int lfInsert          = 1;
const int lfNewFile         = 2;
const int lfNoAutoHl        = 4;

//end of line settings
const int eolUnix           = 0;
const int eolMacintosh      = 1;
const int eolDos            = 2;

void resizeBuffer(void *user, int w, int h);

struct PointStruc {
  PointStruc() {}
  PointStruc(int _x, int _y) : x(_x), y(_y) {}
  int x;
  int y;
};

struct BracketMark {
  PointStruc cursor;
  int sXPos;
  int eXPos;
};

struct VConfig {
  PointStruc cursor;
  int flags;
  int wrapAt;
};

struct SConfig {
  PointStruc cursor;
  PointStruc startCursor;
  int flags;
};

extern int kw_bookmark_keys[];

class KWrite;
class KWriteView;

class KWriteView : public QWidget {
    Q_OBJECT
    friend class KWriteDoc;
    friend class KWrite;
    friend class KIconBorder;
  public:
    KWriteView(KWrite *, KWriteDoc *);
    ~KWriteView();

    void cursorLeft(VConfig &);
    void cursorRight(VConfig &);
    void cursorLeftWord(VConfig &);
    void cursorRightWord(VConfig &);
    void cursorUp(VConfig &);
    void cursorDown(VConfig &);
    void home(VConfig &);
    void end(VConfig &);
    void pageUp(VConfig &);
    void pageDown(VConfig &);
    void top(VConfig &);
    void bottom(VConfig &);
		int lines();


    int getXPos() { return xPos;}
    int getYPos() { return yPos;}
    int getRange(int midline);
    QPoint cursorPosition() { return QPoint(cursor.x, cursor.y); };
    QPoint getCursorCoordinates() const;

protected slots:
    void changeXPos(int);
    void changeYPos(int);

  protected:
    virtual bool event ( QEvent * );

    void paintBracketMark();

    void getVConfig(VConfig &);
    void update(VConfig &);
    void insLine(int line);
    void delLine(int line);
    void updateCursor();
    void updateCursor(PointStruc &newCursor);
    void updateView(int flags, int newXPos = 0, int newYPos = 0);
    void scroll(int, int);
    void wheelEvent(QWheelEvent *);
    void tagLines(int start, int end);
    void tagAll();

    void paintTextLines(int xPos, int yPos);
    void paintCursor();

    void placeCursor(int x, int y, int flags);

    virtual void focusInEvent(QFocusEvent *);
    virtual void focusOutEvent(QFocusEvent *);
    virtual void keyPressEvent(QKeyEvent *e);
    virtual void mousePressEvent(QMouseEvent *);
    virtual void mouseDoubleClickEvent(QMouseEvent *);
    virtual void mouseReleaseEvent(QMouseEvent *);
    virtual void mouseMoveEvent(QMouseEvent *);
    virtual void paintEvent(QPaintEvent *);
    virtual void resizeEvent(QResizeEvent *);
    virtual void timerEvent(QTimerEvent *);
    virtual bool focusNextPrevChild( bool next);
		virtual void dropEvent(QDropEvent *);
		virtual void dragEnterEvent(QDragEnterEvent *);
		virtual void dragLeaveEvent(QDragLeaveEvent *);
		virtual void dragMoveEvent(QDragMoveEvent *);

    KWrite *kWrite;
    KWriteDoc *kWriteDoc;
    QScrollBar *xScroll;
    QScrollBar *yScroll;

    int xPos;
    int yPos;
		bool active_drag;

    int mouseX;
    int mouseY;
    int scrollX;
    int scrollY;
    int scrollTimer;

    BracketMark bm;

    PointStruc cursor;
    bool cursorOn;
    int cursorTimer;
    int cXPos;
    int cOldXPos;
    bool exposeCursor;//cursorMoved;

    int startLine;
    int endLine;
    int updateState;
    int updateLines[2];
    bool bIsPainting;

    QPixmap *drawBuffer;
	  bool m_hasFocus;

signals:
	void replace();
	void search();
	void gotoLine();
};

/** An edit widget with many options, document/view architecture and syntax
    highlight.
    @author Jochen Wilhelmy
*/

class KWrite : public QWidget {
    Q_OBJECT
    friend class KWriteView;
    friend class KWriteDoc;
    friend class KIconBorder;

  public:
    /** The document can be used by more than one KWrite objects
    */
    KWrite(QWidget *parent = 0L, const char *name = 0L);
    /** The destructor does not delete the document
    */
    ~KWrite();

  /** */
  QString textLine( int ) const;

  /** */
  void insertText( const QString& text, bool mark=FALSE );

  QPoint getCursorCoordinates() const;

//status functions
    /** Returns the current line number, that is the line the cursor is on.
        For the first line it returns 0. Signal newCurPos() is emitted on
        cursor position changes.
    */
    int currentLine();
    /** Returns the current column number. It handles tab's correctly.
        For the first column it returns 0.
    */
    int currentColumn();
    /** Sets the current cursor position
    */
    void setCursorPosition(int line, int col);
    /** Returns the config flags. See the cfXXX constants in the .h file.
    */
    int config();
    /** Sets the config flags
    */
    void setConfig(int);
    /** Returns true if the document has been modified.
    */
    bool isModified();
    /** Sets the modification status of the document
    */
    void setModified(bool);
    /** Returns true if this editor is the only owner of its document
    */
    bool isLastView();
    /** Returns the document object
    */
    KWriteDoc *doc();
    /** Bit 0 : undo possible, Bit 1 : redo possible.
        Used to enable/disable undo/redo menu items and toolbar buttons
    */
    int undoState();
    void copySettings(KWrite *);

    int flags() const		{ return configFlags; }

  public slots:
    /** Toggles "Vertical Selections" option
    */
    void toggleVertical();
    /** Toggles Overwrite mode
    */
    void toggleOverwrite();

    /**  */
    void pasteStr(QString s);
    /**  */
    void delMarkedText();

  signals:
    /** The cursor position has changed. Get the values with currentLine()
        and currentColumn()
    */
    void newCurPos();
    /** Modified flag or config flags have changed
    */
    void newStatus();
    /** reflects the mark state
    */
    void markStatus(KWriteView *, bool);
    /** reflects the 'text in clipboard' state
    */
    void clipboardStatus(KWriteView *, bool);
    /** Emits messages for the status line
    */
    void statusMsg(const char *);
    /** The file name has changed. The main window can use this to change
        its caption
    */
    void newCaption();
    /** The undo/redo enable status has changed
    */
    void newUndo();

    void deleteLine( int );
		void overwriteMode(int);

  protected:
    int configFlags;
    int wrapAt;

//text access
  public:
     /** Gets the complete document content as string
     */
     QCString text();
     /** Gets the word where the cursor is on
     */
     QString currentWord();
     /** Gets the word at position x, y. Can be used to find
         the word under the mouse cursor
     */
     QString word(int x, int y);
     /** Discard old text without warning and set new text
     */
     void setText(const char *);
     /** Gets the marked text as string
     */
     QString markedText();

	protected:
		void initPopup();

//edit functions
  public slots:
    /** Clears the document without any warnings or requesters.
    */
    void clear();
    /** Moves the marked text into the clipboard
    */
    void cut();
    /** Copies the marked text into the clipboard
    */
    void copy();
    /** Inserts text from the clipboard at the actual cursor position
    */
    void paste();
    /** Undoes the last operation. The number of undo steps is configurable
    */
    void undo();
    /** Repeats an operation which has been undone before.
    */
    void redo();
    /** Moves the current line or the selection one position to the right
    */
    void indent();
    /** Moves the current line or the selection one position to the left
    */
    void unIndent();
    /** Adds "//" at the beginning of the current line or of each line in the selection
    */
    void comment();
    /** Removes "//" at the beginning of the current line or of each line in the selection
    */
    void unComment();
    /** Selects all text
    */
    void selectAll();
    /** Deselects all text
    */
    void deselectAll();
    /** Inverts the current selection
    */
    void invertSelection();

//search/replace functions
  public slots:
    /** Presents a search dialog to the user
    */
    void search();
    /** Presents a replace dialog to the user
    */
    void replace();
    /** Repeasts the last search or replace operation. On replace, the
        user is prompted even if the "Prompt On Replace" option was off.
    */
    void searchAgain(bool back=false);
    /** Presents a "Goto Line" dialog to the user
    */
    void gotoLine();
  protected:
    void initSearch(SConfig &, int flags);
    void continueSearch(SConfig &);
    void searchAgain(SConfig &);
    void replaceAgain();
    void doReplaceAction(int result, bool found = false);
    void exposeFound(PointStruc &cursor, int slen, int flags, bool replace);
    void deleteReplacePrompt();
    bool askReplaceEnd();
		void gotoPos(int cursorXPos, int cursorYPos);

  protected slots:
    void replaceSlot();

  protected:
    QStrList searchForList;
    QStrList replaceWithList;
    int searchFlags;
    int replaces;
    SConfig s;
    QDialog *replacePrompt;

  public:
    /** Is there a bookmark on the line?
    */
    bool bookmarked(int line);
    /** Get a pointer to the bookmark list
    */

public slots:
    /** Toggle a bookmark at the actual edit position
    */
    void toggleBookmark();
    /** Toggle a bookmark at line
    */
    void toggleBookmark(int line);
    /** Go to the next bookmark
    */
    void nextBookmark();
    /** Go to the previous bookmark
    */
    void previousBookmark();
    /** Clears all bookmarks
    */
    void clearBookmarks();
    /** Sets the cursor to the bookmark n
    */
    void gotoBookmark(int n);

//config file / session management functions
  public:
    /** Writes config entries into the KConfig object
    */
    void writeConfig(KConfig *);
    /** Reads session config out of the KConfig object. This also includes
        the actual cursor position and the bookmarks.
    */
    void readSessionConfig(KConfig *);
    /** Writes session config into the KConfig object
    */
    void writeSessionConfig(KConfig *);

		void setFocus();
		int lines();

//syntax highlight
  public slots:
    /** Gets the highlight number
    */
    int getHl();
    /** Sets the highlight number n
    */
    void setHl(int n);
    /** Get the end of line mode (Unix, Macintosh or Dos)
    */
    int getEol();
    /** Set the end of line mode (Unix, Macintosh or Dos)
    */
    void setEol(int);

//internal
  protected:
    virtual void mousePressEvent(QMouseEvent *);
    virtual void paintEvent(QPaintEvent *);
    virtual void resizeEvent(QResizeEvent *);

    KWriteView *kWriteView;
    KWriteDoc *kWriteDoc;

    int stepLine;

	private:
		QPopupMenu *pop;
};


#endif //KWVIEV_H


