/***************************************************************************
                          logindialog.cpp  -  description
                             -------------------
    begin                : Sun Jan 26 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "logindialog.h"

// Load pixmap
#include "pix/sql_pic.xpm"

LoginDialog::LoginDialog(QWidget *parent, const char *name, bool modal )
					: QDialog(parent,name, modal)
{
	config = kapp->config();
	cp = FALSE;

	this->setCaption(I18N_NOOP("Connect to SQL-Server"));

	//---------------------------------

	QFrame *picFrame = new QFrame(this);
	picFrame->setGeometry(10,10,40,38);
	picFrame->setBackgroundPixmap((const char**)sql_pic_xpm);
	picFrame->setFrameStyle(QFrame::Panel | QFrame::Raised);

	QLabel *serverLab = new QLabel(I18N_NOOP("SQL Server"), this);
	serverLab->setGeometry(70,20,100,20);

	serverLine = new QComboBox(this);
	serverLine->setEditable(TRUE);
	serverLine->setGeometry(170,20,180,20);
	connect(serverLine, SIGNAL(activated(int)), this, SLOT(changeLogin(int)));

	QFrame *splitFrame = new QFrame(this);
	splitFrame->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	splitFrame->setGeometry(5, 75, 390,2);

	//---------------------------------

	QLabel *loginLab = new QLabel(I18N_NOOP("Login Name:"), this);
	loginLab->setGeometry(30,90,170,20);

	loginLine = new KLineEdit(this);
	loginLine->setText("sa");
	loginLine->setGeometry(170,90,220,20);

	QLabel *passLab = new QLabel(I18N_NOOP("Password:"), this);
	passLab->setGeometry(30,120,170,20);

	passLine = new KLineEdit(this);
	passLine->setEchoMode(QLineEdit::Password);
	passLine->setGeometry(170,120,220,20);

	QFrame *split2Frame = new QFrame(this);
	split2Frame->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	split2Frame->setGeometry(5,155,390,2);

	//---------------------------------

	okBut = new QPushButton(I18N_NOOP("OK"), this);
	okBut->setGeometry(130,165,80,30);
	connect(okBut, SIGNAL(clicked()), this, SLOT(okPressed()));

	cancelBut = new QPushButton(I18N_NOOP("Cancel"), this);
	cancelBut->setGeometry(220,165,80,30);
	connect(cancelBut, SIGNAL(clicked()), this, SLOT(cancelPressed()));

	helpBut = new QPushButton(I18N_NOOP("Help"), this);
	helpBut->setGeometry(310,165,80,30);
	helpBut->setDisabled(true);

	getRecentLogins();
}

LoginDialog::~LoginDialog()
{
}

void LoginDialog::getRecentLogins()
{
	QStringList loginList;
	QString tmp;
  config->setGroup("Logins");
	cnt = 0;

	loginList = config->readListEntry("Recent Logins", '|');

	for ( QStringList::Iterator it=loginList.begin() ; it != loginList.end() ; ++it ) {
		tmp = (*it).latin1();
		server[cnt] = QStringList::split(",", tmp,FALSE).first();
		user[cnt] = QStringList::split(",", tmp,FALSE).last();
		if ( !server[cnt].isEmpty() ) {
			serverLine->insertItem(server[cnt], cnt);
			loginLine->setText(user[cnt]);
		}
		cnt++;
	}
	if ( loginList.count() ) {
		serverLine->setCurrentItem(0);
		loginLine->setText(user[0]);
		passLine->setFocus();
	}
}

void LoginDialog::saveLogin()
{
	QString tmp, tmp2;
  config->setGroup("Logins");

	tmp.sprintf("%s,%s", (const char*)serverLine->currentText(), (const char*)loginLine->text());
	for ( int i=0 ; i <= cnt ; i++ ) {
		if ( serverLine->currentText() != server[i] && !server[i].isEmpty() ) {
			tmp += "|";
			tmp2.sprintf("%s,%s", (const char*)server[i], (const char*)user[i]);
			tmp += tmp2;
		}
	}
	config->writeEntry("Recent Logins", tmp);
	config->sync();
}

void LoginDialog::changeLogin(int num)
{
	serverLine->setCurrentItem(num);
	loginLine->setText(user[num]);
	passLine->clear();
	passLine->setFocus();
}

void LoginDialog::cancelPressed()
{
	cp = TRUE;
	emit canceled();
	this->close();
}

void LoginDialog::okPressed()
{
	if (serverLine->currentText().length() > 0 && loginLine->text().length() > 0 ) {
		cp = TRUE;
		saveLogin();
		emit login_data(serverLine->currentText(), loginLine->text(), passLine->text());
	} else {
		QMessageBox::information(this, I18N_NOOP("Not enough informations !"),
						I18N_NOOP("You have to fill out at least the\nServer- and "
											"Login-Fields !\n"));
 	}
}

void LoginDialog::closeEvent(QCloseEvent *c)
{
	if ( !cp )
		emit canceled();
	QDialog::closeEvent(c);
}

void LoginDialog::keyPressEvent(QKeyEvent *k)
{
	if ( k->key() == Qt::Key_Escape )
		cancelPressed();
	QDialog::keyPressEvent(k);
}
