/***************************************************************************
                          myappbar.cpp  -  description
                             -------------------
    begin                : Thu Feb 13 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "myappbar.h"

// Picmaps
#include "pix/objects.xpm"
#include "pix/templates.xpm"
#include "pix/server_pix.xpm"
#include "pix/db_pix.xpm"
#include "pix/folder_pix.xpm"
#include "pix/tab_pix.xpm"
#include "pix/view_pix.xpm"
#include "pix/sp_pix.xpm"
#include "pix/col_pix.xpm"
#include "pix/index_pix.xpm"
#include "pix/constr_pix.xpm"
#include "pix/func_pix.xpm"
#include "pix/param_pix.xpm"
#include "pix/deps_pix.xpm"
#include "pix/trigger_pix.xpm"
#include "pix/co_pix.xpm"
#include "pix/sdt_pix.xpm"

MyAppBar::MyAppBar(QWidget *parent, const char *name )
					: QFrame(parent,name)
{
	this->setBackgroundColor(gray);

	act_db = "master";
	activeItem = 0L;
	rows_received = FALSE;
	c_count = -1;
	dbl_conn = FALSE;

	// Table-Childs
	tabChildName[0] = i18n("Columns");
	tabChildName[1] = i18n("Indexes");
	tabChildName[2] = i18n("Constraints");
	tabChildName[3] = i18n("Dependencies");
	tabChildName[4] = i18n("Triggers");
	tabChildName[5] = i18n("Parameters");
	tabChildName[6] = i18n("");

	objectBox = new QComboBox(this);
	connect(objectBox, SIGNAL(activated(int)), this, SLOT(buildList(int)));

	objectWidg = new ObjectView(0L);
	objectWidg->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	objectWidg->setMargin(1);
	objectWidg->addColumn(0L,-1);
	objectWidg->setSorting(-1);
	QHeader *oHead = objectWidg->header();
	oHead->hide();
	connect(objectWidg, SIGNAL(expanded(QListViewItem *)), this, SLOT(getFurther(QListViewItem *)));
	connect(objectWidg, SIGNAL(refresh()), this, SLOT(refreshView()));

	templateWidg = new ObjectView(0L);
	templateWidg->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	templateWidg->setMargin(1);
	templateWidg->addColumn(0L,-1);
	templateWidg->setSorting(-1);

	QHeader *tHead = templateWidg->header();
	tHead->hide();

	QIconSet objIcon ((const char**)objects_xpm);
	QIconSet templIcon ((const char**)templates_xpm);

	tabWidg = new QTabWidget(this);
	tabWidg->setTabPosition(QTabWidget::Bottom);
	tabWidg->addTab(objectWidg, objIcon, i18n("Objects"));
	tabWidg->addTab(templateWidg, templIcon, i18n("Templates"));
	tabWidg->setBackgroundColor(white);

	initObjects();
	initTemplates();
}

MyAppBar::~MyAppBar()
{
}

void MyAppBar::resizeEvent(QResizeEvent *re)
{
	objectBox->setGeometry(0,0,this->width(), 25);
	tabWidg->setGeometry(0,25,this->width(), (this->height()-25));
}

void MyAppBar::setServerInfo(const QString &servername, const QString &log, const QString &pwd, int prt) //, QValueList<QStringList> dbnames)
{
	for ( int i=0 ; i <= c_count ; i++ ) {
		if ( servername == server[i] && log == login[i] && prt == port[i] ) {
			dbl_conn = TRUE;
			return;
		}
	}
	c_count++;

	// Store Serverdata and Login-Information
	server[c_count] = servername;
	login[c_count] = log;
	pass[c_count] = pwd;
	port[c_count] = prt;
	tsql = new TSQLCont();

	// Insert the servername into the objectBox
	QString tmp, ptmp;
	ptmp.sprintf(":%d", port[c_count]);
	tmp = server[c_count];
	tmp += ptmp;
	tmp += " (";
	tmp += login[c_count];
	tmp += ")";
	objectBox->insertItem(tmp, c_count);
	objectBox->setCurrentItem(c_count);

	objectWidg->setFocus();
}

void MyAppBar::setDbNames(QStringList dbn)
{
	if ( dbl_conn ) {
		dbl_conn = FALSE;
		return;
	}

	if ( c_count < 0 ) return;

	if ( dbn == dbnames[c_count] )
		return;

	dbnames[c_count] = dbn;
	buildList(c_count);
}

void MyAppBar::buildList(int num)
{
	// First we have to clear the List completly !
	objectWidg->clear();
	objectWidg->hide();

	QStringList db;
	QString dbChildName[6];
	QString tmp, tmp2;
	int c_func, c_param;

	dbChildName[0] = i18n("User Tables");
	dbChildName[1] = i18n("System Tables");
	dbChildName[2] = i18n("Views");
	dbChildName[3] = i18n("Stored Procedures");
	dbChildName[4] = i18n("Extended Stored Procedures");
	dbChildName[5] = i18n("Functions");

	// Show the Common-Objects
	QListViewItem *cObjItem = new QListViewItem(objectWidg);
	cObjItem->setPixmap(0, QPixmap((const char**)folder_pix_xpm));
	cObjItem->setText(0,i18n("Common Objects"));

	for ( int i=12 ; i >= 0 ; i-- ) {
		QListViewItem *cChildItem = new QListViewItem(cObjItem);
		cChildItem->setPixmap(0, QPixmap((const char**)folder_pix_xpm));

		tmp = cObjects[i][0][0];
		c_func = tmp.left(2).toInt();
		tmp = tmp.right(tmp.length()-3);

		cChildItem->setText(0, tmp);
		for ( int f=c_func ; f > 0 ; f-- ) {
			tmp = cObjects[i][f][0];
			c_param = tmp.left(2).toInt();
			tmp = tmp.right(tmp.length()-3);
			QListViewItem *coFuncItem = new QListViewItem(cChildItem);
			coFuncItem->setText(0,tmp);
			coFuncItem->setText(4,tmp);
			coFuncItem->setDragEnabled(TRUE);
			if ( !c_param ) {
				coFuncItem->setPixmap(0,QPixmap((const char**)sdt_pix_xpm));
			} else {
				coFuncItem->setPixmap(0,QPixmap((const char**)co_pix_xpm));
				QListViewItem *coPHItem = new QListViewItem(coFuncItem);
				coPHItem->setPixmap(0, QPixmap((const char**)folder_pix_xpm));
				coPHItem->setText(0, i18n("Parameters"));
				for ( int p=c_param ; p>0 ; p-- ) {
					tmp2 = cObjects[i][f][p];
					tmp2 = tmp2.left(tmp2.find(' ',0));
					QListViewItem *coParamItem = new QListViewItem(coPHItem);
					coParamItem->setPixmap(0, QPixmap((const char**)param_pix_xpm));
					coParamItem->setText(0, cObjects[i][f][p]);
					coParamItem->setText(4, tmp2);
					coParamItem->setDragEnabled(TRUE);
				}
			}
		}
	}
	cObjItem->setOpen(TRUE);

	// Show the server and its Objects
	QListViewItem *serverItem = new QListViewItem(objectWidg);
	serverItem->setPixmap(0, QPixmap((const char**)server_pix_xpm));
	serverItem->setText(0,server[num]);
	serverItem->setText(4,server[num]);

	for ( QStringList::Iterator it=dbnames[num].begin() ; it != dbnames[num].end() ; ++it ) {
		// Databases OPEN
//		db = *it;
//		tmp = db.first();
		if ( *it != "" ) {
			QListViewItem *dbItem = new QListViewItem(serverItem);
			dbItem->setPixmap(0, QPixmap((const char**)db_pix_xpm));
			dbItem->setText(0,(*it).latin1());
			dbItem->setText(1, 0L);
			dbItem->setText(2, "dbase");
			tmp = (*it).latin1();
			dbItem->setText(4, tmp);
			dbItem->setDragEnabled(TRUE);

			for ( int i=5 ; i >= 0 ; i-- ) {
				if ( i==4 && tmp != "master" )
					continue;

				// Database-Folders OPEN
				QString ph_txt;
				QListViewItem *dbChild = new QListViewItem(dbItem);
				dbChild->setPixmap(0, QPixmap((const char**)folder_pix_xpm));
				dbChild->setText(0, dbChildName[i]);
				dbChild->setText(1, 0L);

				ph_txt = "PH1_";
				switch ( i ) {
					case 0:
						ph_txt.append("US");
						break;
					case 1:
						ph_txt.append("SY");
						break;
					case 2:
						ph_txt.append("VI");
						break;
					case 3:
						ph_txt.append("ST");
						break;
					case 4:
						ph_txt.append("EX");
						break;
					case 5:
						ph_txt.append("FU");
						break;
				}
				dbChild->setText(2, ph_txt);
				dbChild->setExpandable(TRUE);

				repaint();
				qApp->processEvents();
			}	// Database-Folders CLOSE
		}
	} // Databases CLOSE
	serverItem->setOpen(TRUE);
	objectWidg->show();

	tsql->tsql_login(server[num], port[num], login[num], pass[num]);
	if ( tsql->tsql_connect() ) {
		tsql->tsql_doQuery("set quoted_identifier off");
		tsql->tsql_doQuery("SET TEXTSIZE 64512");
		tsql->tsql_doQuery("set showplan_text off");
		tsql->tsql_doQuery("SET NOEXEC OFF SET PARSEONLY OFF");
		tsql->tsql_doQuery("set showplan_all off");
		tsql->tsql_doQuery("set nocount off;set arithabort off;set concat_null_yields_null off;"
												"set ansi_nulls off;set cursor_close_on_commit off;set ansi_null_dflt_on off;"
												"set implicit_transactions off;set ansi_padding off;set ansi_warnings off;"
												"set quoted_identifier off");
		tsql->tsql_doQuery("set lock_timeout -1");
		tsql->tsql_doQuery("set nocount off;set arithabort on;set concat_null_yields_null on;"
												"set ansi_nulls on;set cursor_close_on_commit off;set ansi_null_dflt_on on;"
												"set implicit_transactions off;set ansi_padding on;set ansi_warnings on;"
												"set quoted_identifier on");
		}
	connect(tsql, SIGNAL(tsql_queryrow_received(QValueList<QStringList> &)),
				 this, SLOT(insertQueryResults(QValueList<QStringList> &)));
	connect(tsql, SIGNAL(tsql_querycolumns_received(int, QStringList  &)),
				 this, SLOT(insertQueryColumns(int, QStringList &)));
	connect(tsql, SIGNAL(tsql_queryReady(int,int)),
				 this, SLOT(queryReady(int,int)));
}

void MyAppBar::getFurther(QListViewItem *parItem)
{
	// Set the active Item
	activeItem = parItem;
	
	QString parName, parId, parType, parStr, query_string, tmp, tmp_user;
	int depth;
	depth = -1;
	vdep = FALSE;

	parName = parItem->text(0);
	parId = parItem->text(1);
	parType = parItem->text(2).upper();

	if ( parType.isNull() ) {
		return;
	} else if ( parType == "DBASE" ) {
			act_db = parName;
			return;
	}

	// We only need to get the Objects once
	if ( activeItem->childCount() )
		return;

	// Set the waiting cursor...
	objectWidg->setCursor(QCursor::waitCursor);

	if ( parType ==  "PH1_US" ) {
		// Get User-Tables
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select dbname = user_name(uid)+'.'+name, id, 'user_table' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'U' order by name desc");
	} else if ( parType ==  "PH1_SY" ) {
		// Get System-Tables
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select dbname = user_name(uid)+'.'+name, id, 'system_table' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'S' order by name desc");
	} else if ( parType ==  "PH1_VI" ) {
		// Get Views
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select dbname = user_name(uid)+'.'+name, id, 'view' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'V' order by name desc");
	} else if ( parType ==  "PH1_ST" ) {
		// Get Stored-Procedures
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select dbname = user_name(uid)+'.'+name, id, 'stored_procedure' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'P' order by name desc");
	} else if ( parType ==  "PH1_EX" ) {
		// Get Extended Stored-Procedures
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select dbname = user_name(uid)+'.'+name, id, 'xstored_procedure' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'X' order by name desc");
	} else if ( parType ==  "PH1_FU" ) {
		// Get Functions
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select dbname = user_name(uid)+'.'+name, id, 'function' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'TF' or type = N'IF' or type = N'FN' order by name desc");
	} else if ( parType == "TAB_COL" ) {
		// Get Columns of one Table
		tmp = activeItem->parent()->text(0);
		tmp_user = tmp.left(tmp.find('.',0));
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("Create Table #MyColTab ( name VARCHAR(420) NULL, id INT NULL, type VARCHAR(12) NULL, colid INT NULL);\n");
		query_string.append("DECLARE @name VARCHAR(384)\n");
		query_string.append("DECLARE @isnullable INT\n");
		query_string.append("DECLARE @len INT\n");
		query_string.append("DECLARE @xtype INT\n");
		query_string.append("DECLARE @xname VARCHAR(384)\n");
		query_string.append("DECLARE @nulltext VARCHAR(20)\n");
		query_string.append("DECLARE @cid INT\n");
		query_string.append("DECLARE col CURSOR FOR select name, xtype, isnullable, length, colid from [");
		query_string.append(act_db);
		query_string.append("]..syscolumns where id = ");
		query_string.append(parId);
		query_string.append(" order by colorder\n");
		query_string.append("open col\n");
		query_string.append("FETCH NEXT FROM col INTO @name, @xtype, @isnullable, @len, @cid\n");
		query_string.append("WHILE ( @@FETCH_STATUS=0 )\n");
		query_string.append("BEGIN\n");
		query_string.append(" IF @isnullable=0\n");
		query_string.append(" BEGIN\n");
		query_string.append("   SET @nulltext = 'Not Null'\n");
		query_string.append(" END   \n");
		query_string.append(" SELECT @xname = name FROM [");
		query_string.append(act_db);
		query_string.append("]..systypes where xtype=@xtype\n");
		query_string.append("INSERT INTO #MyColTab (name, id, type, colid) \n");
		query_string.append("SELECT CONVERT(VARCHAR(420), @name+' ('+@xname+'('+CONVERT(VARCHAR(100), @len)+'), '+@nulltext+')'), 0, 'table_column', @cid\n");
		query_string.append(" FETCH NEXT FROM col INTO @name, @xtype, @isnullable, @len, @cid\n");
		query_string.append("END\n");
		query_string.append("CLOSE col\n");
		query_string.append("DEALLOCATE col\n");
		query_string.append("select name, id, type from #MyColTab order by colid desc\n");
		query_string.append("DROP TABLE #MyColTab");
	} else if ( parType == "TAB_IND" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("];\nselect I.name, '0', 'table_index'  from [");
		query_string.append(act_db);
		query_string.append("]..sysindexes I where I.id = ");
		query_string.append(parId);
		query_string.append(" and I.indid > 0 and I.indid < 255 and INDEXPROPERTY(I.id, I.name, N'IsStatistics') = 0 and INDEXPROPERTY(I.id, I.name, N'IsHypothetical') = 0 and I.name not in (select O.name from sysobjects O where O.parent_obj = I.id and OBJECTPROPERTY(O.id, N'isConstraint') = 1)");
	} else if ( parType == "TAB_CON" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("Create Table #MyConTab ( name VARCHAR(420) NULL, id INT NULL, type VARCHAR(50) NULL);\n");
		query_string.append("DECLARE @name VARCHAR(384)\n");
		query_string.append("DECLARE @xtype VARCHAR(20)\n");
		query_string.append("DECLARE @xname VARCHAR(384)\n");
		query_string.append("DECLARE col CURSOR FOR select name, xtype from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where parent_obj = ");
		query_string.append(parId);
		query_string.append(" and (type = N'C' or type = N'D' or type = N'F' or type = N'K')\n");
		query_string.append("open col\n");
		query_string.append("FETCH NEXT FROM col INTO @name, @xtype\n");
		query_string.append("WHILE ( @@FETCH_STATUS=0 )\n");
		query_string.append("BEGIN\n");
		query_string.append(" IF @xtype='PK'\n");
		query_string.append("   BEGIN\n");
		query_string.append("     SET @name = @name+' (primary key constraint)'\n");
		query_string.append("   END\n");
		query_string.append(" ELSE IF @xtype='D'\n");
		query_string.append("    BEGIN\n");
		query_string.append("     SET @name = @name+' (default constraint)'\n");
		query_string.append("    END\n");
		query_string.append(" ELSE IF @xtype='UQ'\n");
		query_string.append("    BEGIN\n");
		query_string.append("     SET @name = @name+' (unique key constraint)'\n");
		query_string.append("    END\n");
		query_string.append("INSERT INTO #MyConTab (name, id, type) \n");
		query_string.append("SELECT CONVERT(VARCHAR(420), @name), 0, 'table_constraint'\n");
		query_string.append(" FETCH NEXT FROM col INTO @name, @xtype\n");
		query_string.append("END\n");
		query_string.append("CLOSE col\n");
		query_string.append("DEALLOCATE col\n");
		query_string.append("select name, id, type from #MyConTab\n");
		query_string.append("DROP TABLE #MyConTab\n");
	} else if ( parType == "TAB_DEP" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select distinct o.name, '0', 'table_depends' from [");
		query_string.append(act_db);
		query_string.append("]..sysdepends d, [");
		query_string.append(act_db);
		query_string.append("]..sysobjects o where d.depid = ");
		query_string.append(parId);
		query_string.append(" and d.id = o.id");
	} else if ( parType == "TAB_TRI" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select name, '0', 'table_triggers' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects ");
		query_string.append(" where type = N'TR' and parent_obj = ");
		query_string.append(parId);
	} else if ( parType == "VIEW_COL" ) {
		// Special ! We need to get the column-names here !!!
		vdep = TRUE;
		tmp = activeItem->parent()->text(0);
		tmp_user = tmp.left(tmp.find('.',0));
		tmp = tmp.right((tmp.length()-tmp.find('.',0))-1);
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select top 0 * from [");
		query_string.append(act_db);
		query_string.append("].[");
		query_string.append(tmp_user);
		query_string.append("].[");
		query_string.append(tmp);
		query_string.append("]");
	} else if ( parType == "VIEW_IND" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select I.name, '0', 'view_index' from [");
		query_string.append(act_db);
		query_string.append("]..sysindexes I where I.id = ");
		query_string.append(parId);
		query_string.append(" and I.indid > 0 and I.indid < 255");
	} else if ( parType == "VIEW_DEP" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select distinct o.name, '0', 'view_depends' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects o, [");
		query_string.append(act_db);
		query_string.append("]..sysdepends d where d.id = ");
		query_string.append(parId);
		query_string.append(" and o.id = d.depid");
	} else if ( parType == "VIEW_TRI" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select name, '0', 'view_trigger' from [");
		query_string.append(act_db);
		query_string.append("]..sysobjects where type = N'TR' and parent_obj = ");
		query_string.append(parId);
		query_string.append(" order by name desc");
	} else if ( parType == "SP_PAR" ) {
		// Get the name of the Stored Procedure
		tmp = activeItem->parent()->text(0);
		tmp_user = tmp.left(tmp.find('.',0));
		tmp = tmp.right((tmp.length()-tmp.find('.',0))-1);
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("CREATE TABLE #MySpColumns (qual NVARCHAR(384) NULL, owner NVARCHAR(384) NULL, name NVARCHAR(384) NULL,");
		query_string.append(" cname NVARCHAR(384) NULL, ctype NVARCHAR(255) NULL, dtype INT, tname NVARCHAR(384) NULL, prec INT,");
		query_string.append(" length INT, scale INT, radix INT, nullable INT, rem NVARCHAR(384) NULL, cdef NVARCHAR(384) NULL, sdtype INT,");
		query_string.append(" sdtypesub NVARCHAR(384) NULL, col NVARCHAR(384) NULL, op INT, is_nullable NVARCHAR(10) NULL, ssdtype INT);\n");
		query_string.append("INSERT INTO #MySpColumns exec ");
		query_string.append(act_db);
		query_string.append("..sp_sproc_columns N'");
		query_string.append(tmp);
		query_string.append("', N'%', N'");
		query_string.append(act_db);
		query_string.append("', NULL, @ODBCVer = 3\n");
		query_string.append("UPDATE #MySpColumns set ctype='Return Value' where ctype='5'\n");
		query_string.append("UPDATE #MySpColumns set ctype='Input' where ctype='1'\n");
		query_string.append("select cname+' ('+tname+'('+CONVERT(VARCHAR(100),length)+'),'+ctype+')', '0', 'sp_params' from #myspcolumns\n");
		query_string.append("DROP TABLE #MySpColumns\n");
	} else if ( parType == "SP_DEP" ) {
		query_string = "use [";
		query_string.append(act_db);
		query_string.append("]\n");
		query_string.append("select distinct name = user_name(o.uid)+'.'+o.name, type, 'sp_depends' from [");
		query_string.append(act_db);
		query_string.append("]..sysdepends d, [");
		query_string.append(act_db);
		query_string.append("]..sysobjects o where (d.depid = ");
		query_string.append(parId);
		query_string.append(" and d.id = o.id) or (d.id = ");
		query_string.append(parId);
		query_string.append(" and o.id=d.depid) order by name desc");
	}

debug("QUERY START\n-------------\n%s\n--------------\nQUERY END", (const char*)query_string);
	tsql->tsql_doQuery(query_string);

//debug("%s", (const char*)query_string);
}

/**
** Receive database-informations and create new Items.
*/
void MyAppBar::insertQueryResults(QValueList<QStringList> &rs)
{
	if ( vdep )
		return;
	QStringList row;
	QString col, row_id, tmp;
	int rs_cnt;

	for ( QValueList<QStringList>::Iterator it=rs.begin() ; it != rs.end() ; ++it ) {
		row = *it;
		rs_cnt = 0;
		QListViewItem *child = new QListViewItem(activeItem);
		rows_received = TRUE;
		for ( QStringList::Iterator rit=row.begin() ; rit != row.end() ; ++rit ) {
			if ( !rs_cnt ) {
				tmp = (*rit).latin1();
				child->setText(rs_cnt, tmp);
			} else
				child->setText(rs_cnt, (*rit).latin1());

			// Store ID of the new Item
			if ( rs_cnt == 1 )
				row_id = (*rit).latin1();

			// Set Pixmaps and Childs
			if ( rs_cnt == 2 ) {
				col = (*rit).latin1();
				int tabnstart = 4;
				int tabnend = 0;

				if ( col == "system_table" ) {
					tabnstart = 3;
					child->setPixmap(0, QPixmap((const char**)tab_pix_xpm));
					child->setText(4,tmp);
				} else if ( col == "view" ) {
					child->setPixmap(0, QPixmap((const char**)view_pix_xpm));
					child->setText(4,tmp);
				} else if ( col == "stored_procedure" ) {
					child->setPixmap(0, QPixmap((const char**)sp_pix_xpm));
					child->setText(4,tmp);
					tabnstart = 5;
					tabnend = 3;
				} else if ( col == "xstored_procedure") {
					child->setPixmap(0, QPixmap((const char**)sp_pix_xpm));
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else if ( col == "function") {
					child->setPixmap(0, QPixmap((const char**)func_pix_xpm));
					child->setText(4,tmp);
					tabnstart = 5;
					tabnend = 3;
				} else if ( col == "table_column" ) {
					child->setPixmap(0, QPixmap((const char**)col_pix_xpm));
					tmp = tmp.left( tmp.find('(',0) );
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else if ( col == "table_index" || col == "view_index" ) {
					child->setPixmap(0, QPixmap((const char**)index_pix_xpm));
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else if ( col == "table_constraint" ) {
					child->setPixmap(0, QPixmap((const char**)constr_pix_xpm));
					tmp = tmp.left( tmp.find('(',0) );
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else if ( col == "table_depends" || col == "view_depends" || col == "sp_depends" ) {
					child->setPixmap(0, QPixmap((const char**)deps_pix_xpm));
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else if ( col == "table_triggers" || col == "view_triggers" ) {
					child->setPixmap(0, QPixmap((const char**)trigger_pix_xpm));
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else if ( col == "sp_params" ) {
					child->setPixmap(0, QPixmap((const char**)param_pix_xpm));
					tmp = tmp.left( tmp.find('(',0) );
					child->setText(4,tmp);
					tabnstart = -1;
					tabnend = 0;
				} else {
					child->setPixmap(0, QPixmap((const char**)tab_pix_xpm));
					child->setText(4,tmp);
				}

				for ( int i=tabnstart ; i >= tabnend ; i-- ) {
					if ( col == "view" && i == 2 ) continue;
					if ( (col == "stored_procedure" || col == "xstored_procedure"  || col == "function" ) && i == 4 ) continue;
					QListViewItem *tabChild = new QListViewItem(child);
					tabChild->setText(0, tabChildName[i]);
					tabChild->setText(1, row_id);
					if ( col == "view" )
						tmp = "VIEW_";
					else if ( col == "stored_procedure" || col == "function" )
						tmp = "SP_";
					else
						tmp = "TAB_";

					switch ( i ) {
						case 0:
							tmp.append("COL");
							break;
						case 1:
							tmp.append("IND");
							break;
						case 2:
							tmp.append("CON");
							break;
						case 3:
							tmp.append("DEP");
							break;
						case 4:
							tmp.append("TRI");
							break;
						case 5:
							tmp.append("PAR");
							break;
						case 6:
							tmp.append("");
							break;
					}
					tabChild->setText(2, tmp);
					tabChild->setPixmap(0, QPixmap((const char**)folder_pix_xpm));
					tabChild->setExpandable(TRUE);
				}
				if ( col == "user_table" || col == "system_table" || col == "view"  || col == "stored_procedure" || col == "function" ) {
					child->setExpandable(TRUE);
				}
				child->setDragEnabled(TRUE);
			}
			rs_cnt++;
		}
	}
}

void MyAppBar::insertQueryColumns(int cols, QStringList &clist)
{
	if ( !vdep )
		return;

	for ( QStringList::Iterator it=clist.begin() ; it != clist.end() ; ++it ) {
		QListViewItem *child = new QListViewItem(activeItem);
		child->setText(0, (*it).latin1());
		child->setText(1, "0");
		child->setText(2, "view_columns");
		child->setPixmap(0, QPixmap((const char**)col_pix_xpm));
		child->setDragEnabled(TRUE);
	}
}

void MyAppBar::queryReady(int r, int t)
{
	if ( !rows_received )
		activeItem->setExpandable(FALSE);
	rows_received = FALSE;

	// Set the arrow cursor...
	objectWidg->setCursor(QCursor::arrowCursor);
}

void MyAppBar::initObjects()
{
	cObjects[0][0][0] = i18n("15,Configuration Functions");
		cObjects[0][1][0] = "01,@@CONNECTIONS";
			cObjects[0][1][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[0][2][0] = "01,@@DATEFIRST";
			cObjects[0][2][1] = "RETURN_VALUE (tinyint, Return Value)";
		cObjects[0][3][0] = "01,@@DBTS";
			cObjects[0][3][1] = "RETURN_VALUE (varbinary, Return Value)";
		cObjects[0][4][0] = "01,@@LANGUAGE";
			cObjects[0][4][1] = "RETURN_VALUE (nvarchar, Return Value)";
		cObjects[0][5][0] = "01,@@LANGID";
			cObjects[0][5][1] = "RETURN_VALUE (smallint, Return Value)";
		cObjects[0][6][0] = "01,@@LOCK_TIMEOUT";
			cObjects[0][6][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[0][7][0] = "01,@@MAX_CONNECTIONS";
			cObjects[0][7][1] = "RETURN_VALUE (tinyint, Return Value)";
		cObjects[0][8][0] = "01,@@NESTLEVEL";
			cObjects[0][8][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[0][9][0] = "01,@@OPTIONS";
			cObjects[0][9][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[0][10][0] = "01,@@REMSERVER";
			cObjects[0][10][1] = "RETURN_VALUE (nvarchar(128), Return Value)";
		cObjects[0][11][0] = "01,@@SPID";
			cObjects[0][11][1] = "RETURN_VALUE (smallint, Return Value)";
		cObjects[0][12][0] = "01,@@SERVERNAME";
			cObjects[0][12][1] = "RETURN_VALUE (nvarchar, Return Value)";
		cObjects[0][13][0] = "01,@@SERVICENAME";
			cObjects[0][13][1] = "RETURN_VALUE (nvarchar, Return Value)";
		cObjects[0][14][0] = "01,@@TEXTSIZE";
			cObjects[0][14][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[0][15][0] = "01,@@VERSION";
			cObjects[0][15][1] = "RETURN_VALUE (nvarchar, Return Value)";

	cObjects[1][0][0] = i18n("03,Cursor Functions");
		cObjects[1][1][0] = "01,@@CURSOR_ROWS";
			cObjects[1][1][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[1][2][0] = "01,@@FETCH_STATUS";
			cObjects[1][2][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[1][3][0] = "03,CURSOR_STATUS";
			cObjects[1][3][1] = "RETURN_VALUE (smallint, Return Value)";
			cObjects[1][3][2] = "@param1 (varchar, Input)";
			cObjects[1][3][3] = "cursor_name (nvarchar, Input)";

	cObjects[2][0][0] = i18n("09,Date and Time Functions");
		cObjects[2][1][0] = "04,DATEADD";
			cObjects[2][1][1] = "RETURN_VALUE (datetime, Return Value)";
			cObjects[2][1][2] = "datepart (Input)";
			cObjects[2][1][3] = "number (int, Input)";
			cObjects[2][1][4] = "date (datetime, Input)";
		cObjects[2][2][0] = "04,DATEDIFF";
			cObjects[2][2][1] = "RETURN_VALUE (int, Return Value)";
			cObjects[2][2][2] = "datepart (Input)";
			cObjects[2][2][3] = "startdate (datetime, Input)";
			cObjects[2][2][4] = "enddate (datepart, Input)";
		cObjects[2][3][0] = "03,DATENAME";
			cObjects[2][3][1] = "RETURN_VALUE (nvarchar, Return Value)";
			cObjects[2][3][2] = "datepart (Input)";
			cObjects[2][3][3] = "date (datetime, Input)";
		cObjects[2][4][0] = "03,DATEPART";
			cObjects[2][4][1] = "RETURN_VALUE (int, Return Value)";
			cObjects[2][4][2] = "datepart (Input)";
			cObjects[2][4][3] = "date (datetime, Input)";
		cObjects[2][5][0] = "02,DAY";
			cObjects[2][5][1] = "RETURN_VALUE (int, Return Value)";
			cObjects[2][5][2] = "date (datetime, Input)";
		cObjects[2][6][0] = "01,GETDATE";
			cObjects[2][6][1] = "RETURN_VALUE (datetime, Return Value)";
		cObjects[2][7][0] = "02,MONTH";
			cObjects[2][7][1] = "RETURN_VALUE (int, Return Value)";
			cObjects[2][7][2] = "date (datetime, Input)";
		cObjects[2][8][0] = "02,YEAR";
			cObjects[2][8][1] = "RETURN_VALUE (int, Return Value)";
			cObjects[2][8][2] = "date (datetime, Input)";
		cObjects[2][9][0] = "01,GETUTCDATE";
			cObjects[2][9][1] = "RETURN_VALUE (datetime, Return Value)";

	cObjects[3][0][0] = i18n("23,Mathematical Functions");
		cObjects[3][1][0] = "02,ABS";
			cObjects[3][1][1] = "RETURN_VALUE (numeric, Return Value)";
			cObjects[3][1][2] = "numeric_expression (numeric, Input)";
		cObjects[3][2][0] = "02,ACOS";
			 cObjects[3][2][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][2][2] = "float_expression (real, Input)";
		cObjects[3][3][0] = "02,ASIN";
			 cObjects[3][3][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][3][2] = "float_expression (real, Input)";
		cObjects[3][4][0] = "02,ATAN";
			 cObjects[3][4][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][4][2] = "float_expression (float, Input)";
		cObjects[3][5][0] = "03,ATN2";
			 cObjects[3][5][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][5][2] = "float_expression1 (float, Input)";
			 cObjects[3][5][3] = "float_expression2 (float, Input)";
		cObjects[3][6][0] = "02,CEILING";
			 cObjects[3][6][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[3][6][2] = "numeric_expression (numeric, Input)";
		cObjects[3][7][0] = "02,COS";
			 cObjects[3][7][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][7][2] = "float_expression (float, Input)";
		cObjects[3][8][0] = "02,COT";
			 cObjects[3][8][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][8][2] = "float_expression (float, Input)";
		cObjects[3][9][0] = "02,DEGREES";
			 cObjects[3][9][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[3][9][2] = "numeric_expression (numeric, Input)";
		cObjects[3][10][0] = "02,EXP";
			 cObjects[3][10][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][10][2] = "float_expression (float, Input)";
		cObjects[3][11][0] = "02,FLOOR";
			 cObjects[3][11][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[3][11][2] = "numeric_expression (numeric, Input)";
		cObjects[3][12][0] = "02,LOG";
			 cObjects[3][12][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][12][2] = "float_expression (float, Input)";
		cObjects[3][13][0] = "02,LOG1";
			 cObjects[3][13][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][13][2] = "float_expression (float, Input)";
		cObjects[3][14][0] = "01,PI";
			 cObjects[3][14][1] = "RETURN_VALUE (float, Return Value)";
			cObjects[3][15][0] = "03,POWER";
		 cObjects[3][15][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[3][15][2] = "numeric_expression (numeric, Input)";
			 cObjects[3][15][3] = "Y (numeric, Input)";
		cObjects[3][16][0] = "02,RADIANS";
			 cObjects[3][16][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[3][16][2] = "numeric_expression (numeric, Input)";
		cObjects[3][17][0] = "02,RAND";
			 cObjects[3][17][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][17][2] = "seed (int, Input)";
		cObjects[3][18][0] = "04,ROUND";
			 cObjects[3][18][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[3][18][2] = "numeric_expression (numeric, Input)";
		 cObjects[3][18][3] = "length (int, Input)";
			 cObjects[3][18][4] = "function (int, Input)";
			cObjects[3][19][0] = "02,SIGN";
		 cObjects[3][19][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][19][2] = "numeric_expression (numeric, Input)";
			cObjects[3][20][0] = "02,SIN";
		 cObjects[3][20][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][20][2] = "float_expression (float, Input)";
			cObjects[3][21][0] = "02,SQUARE";
		 cObjects[3][21][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][21][2] = "float_expression (float, Input)";
			cObjects[3][22][0] = "02,SQRT";
		 cObjects[3][22][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][22][2] = "float_expression (float, Input)";
			cObjects[3][23][0] = "02,TAN";
		 cObjects[3][23][1] = "RETURN_VALUE (float, Return Value)";
			 cObjects[3][23][2] = "float_expression (real, Input)";


	cObjects[4][0][0] = i18n("16,Aggregate Functions");
		cObjects[4][1][0] = "02,SUM";
			 cObjects[4][1][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[4][1][2] = "expression (numeric, Input)";
		cObjects[4][2][0] = "02,MIN";
			 cObjects[4][2][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[4][2][2] = "expression (numeric, Input)";
		cObjects[4][3][0] = "02,MAX";
			 cObjects[4][3][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[4][3][2] = "expression (numeric, Input)";
		cObjects[4][4][0] = "02,COUNT";
			 cObjects[4][4][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[4][4][2] = "expression (Input)";
		cObjects[4][5][0] = "01,COUNT(*)";
			 cObjects[4][5][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[4][6][0] = "02,COUNT_BIG(*)";
			 cObjects[4][6][1] = "RETURN_VALUE(bigint, Return Value)";
			 cObjects[4][6][1] = "expression (Input)";
		cObjects[4][7][0] = "01,COUNT_BIG(*)";
			 cObjects[4][7][1] = "RETURN_VALUE(bigint, Return Value)";
		cObjects[4][8][0] = "02,AVG";
			 cObjects[4][8][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][8][2] = "expression (numeric, Input)";
		cObjects[4][9][0] = "02,STDEV";
			 cObjects[4][9][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][9][2] = "expression (numeric, Input)";
		cObjects[4][10][0] = "02,STDEVP";
			 cObjects[4][10][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][10][2] = "expression (numeric, Input)";
		cObjects[4][11][0] = "02,VAR";
			 cObjects[4][11][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][11][2] = "expression (numeric, Input)";
		cObjects[4][12][0] = "02,VARP";
			 cObjects[4][12][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][12][2] = "expression (numeric, Input)";
		cObjects[4][13][0] = "02,BINARY_CHECKSUM";
			 cObjects[4][13][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][13][2] = "expression (numeric, Input)";
		cObjects[4][14][0] = "02,CHECKSUM";
			 cObjects[4][14][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][14][2] = "expression (numeric, Input)";
		cObjects[4][15][0] = "02,CHECKSUM_AGG";
			 cObjects[4][15][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][15][2] = "expression (numeric, Input)";
		cObjects[4][16][0] = "02,GROUPING";
			 cObjects[4][16][1] = "RETURN_VALUE(numeric, Return Value)";
			 cObjects[4][16][2] = "column_name (sysname, Input)";	

	cObjects[5][0][0] = i18n("26,Metadata Functions");
		cObjects[5][1][0] = "03,COL_LENGTH";
			 cObjects[5][1][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][1][2] = "table (nvarchar, Input)";
			 cObjects[5][1][3] = "column (nvarchar, Input)";
		cObjects[5][2][0] = "03,COL_NAME";
			 cObjects[5][2][1] = "RETURN_VALUE (sysname, Return Value)";
			 cObjects[5][2][2] = "table_id (int, Input)";
			 cObjects[5][2][3] = "column_id (int, Input)";
		cObjects[5][3][0] = "04,COLUMNPROPERTY";
			 cObjects[5][3][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][3][2] = "id (int, Input)";
			 cObjects[5][3][3] = "column (nvarchar, Input)";
			 cObjects[5][3][4] = "property (nvarchar, Input)";
		cObjects[5][4][0] = "03,DATABASEPROPERTY";
			 cObjects[5][4][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][4][2] = "database (nvarchar(128), Input)";
			 cObjects[5][4][3] = "property (nvarchar(128), Input)";
		cObjects[5][5][0] = "03,DATABASEPROPERTYEX";
			 cObjects[5][5][1] = "RETURN_VALUE (sql_variant, Return Value)";
			 cObjects[5][5][2] = "database (nvarchar(128), Input)";
			 cObjects[5][5][3] = "property (nvarchar(128), Input)";
		cObjects[5][6][0] = "02,DB_ID";
			 cObjects[5][6][1] = "RETURN_VALUE (smallint, Return Value)";
			 cObjects[5][6][2] = "database (nvarchar, Input)";
		cObjects[5][7][0] = "02,DB_NAME";
			 cObjects[5][7][1] = "RETURN_VALUE (nvarchar(128), Return Value)";
			 cObjects[5][7][2] = "database_id (smallint, Input)";
		cObjects[5][8][0] = "02,FILE_ID";
			 cObjects[5][8][1] = "RETURN_VALUE (smallint, Return Value)";
	 		cObjects[5][8][2] = "file_name (nchar(128), Input)";
		cObjects[5][9][0] = "02,FILE_NAME";
			 cObjects[5][9][1] = "RETURN_VALUE (nvarchar(128), Return Value)";
			 cObjects[5][9][2] = "file_id (smallint, Input)";
		cObjects[5][10][0] = "02,FILEGROUP_ID";
			 cObjects[5][10][1] = "RETURN_VALUE (smallint, Return Value)";
			 cObjects[5][10][2] = "filegroup_id (smallint, Input)";
		cObjects[5][11][0] = "02,FILEGROUP_NAME";
			 cObjects[5][11][1] = "RETURN_VALUE (nvarchar(128), Return Value)";
			 cObjects[5][11][2] = "filegroup_id (smallint, Input)";
		cObjects[5][12][0] = "03,FILEGROUPPROPERTY";
			 cObjects[5][12][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][12][2] = "filegroup_name (nvarchar(128), Input)";
			 cObjects[5][12][3] = "property (varchar(128), Input)";
		cObjects[5][13][0] = "03,FILEPROPERTY";
			 cObjects[5][13][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][13][2] = "file_name (nvarchar(128), Input)";
			 cObjects[5][13][3] = "property (varchar(128), Input)";
		cObjects[5][14][0] = "03,FULLTEXTCATALOGPROPERTY";
			 cObjects[5][14][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][14][2] = "catalog_name (nchar(128), Input)";
			 cObjects[5][14][3] = "property (varchar(128), Input)";
		cObjects[5][15][0] = "02,FULLTEXTSERVICEPROPERTY";
			 cObjects[5][15][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][15][2] = "property (varchar(128), Input)";
		cObjects[5][16][0] = "04,INDEX_COL";
			 cObjects[5][16][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[5][16][2] = "table (nvarchar, Input)";
			 cObjects[5][16][3] = "index_id (int, Input)";
			 cObjects[5][16][4] = "key_id (int, Input)";
		cObjects[5][17][0] = "05,INDEXKEY_PROPERTY";
			 cObjects[5][17][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][17][2] = "table_id (int, Input)";
			 cObjects[5][17][3] = "index_id (int, Input)";
			 cObjects[5][17][4] = "key_id (int, Input)";
			 cObjects[5][17][5] = "property (varchar(128), Input)";
		cObjects[5][18][0] = "04,INDEXPROPERTY";
			 cObjects[5][18][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][18][2] = "table_id (int, Input)";
			 cObjects[5][18][3] = "index (nvarchar(128), input)";
			 cObjects[5][18][4] = "property (varchar(128), Input)";
		cObjects[5][19][0] = "02,OBJECT_ID";
			 cObjects[5][19][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][19][2] = "object (nchar, Input)";
		cObjects[5][20][0] = "02,OBJECT_NAME";
			 cObjects[5][20][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[5][20][2] = "object_id (int, Input)";
		cObjects[5][21][0] = "03,OBJECTPROPERTY";
			 cObjects[5][21][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][21][2] = "id (int, Input)";
			 cObjects[5][21][3] = "property (varchar(128), Input)";
		cObjects[5][22][0] = "01,@@PROCID";
			 cObjects[5][22][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[5][23][0] = "03,TYPEPROPERTY";
			 cObjects[5][23][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][23][2] = "type (nvarchar, Input)";
			 cObjects[5][23][3] = "property (varchar(128), Input)";
		cObjects[5][24][0] = "05,INDEXKEY_PROPERTY";
			 cObjects[5][24][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[5][24][2] = "table_id (int, Input)";
			 cObjects[5][24][3] = "index_id (int, Input)";
			 cObjects[5][24][4] = "key_id (int, input)";
			 cObjects[5][24][5] = "property (varchar, Input)";
		cObjects[5][25][0] = "03,SQL_VARIANT_PROPERTY";
			 cObjects[5][25][1] = "RETURN_VALUE (sql_variant, Return Value)";
			 cObjects[5][25][2] = "expression (sql_variant, Input)";
			 cObjects[5][25][3] = "property (varchar(128), Input)";
		cObjects[5][26][0] = "08,fn_listextendedproperty";
			 cObjects[5][26][1] = "TABLE_RETURN_VALUE (Result Column)";
			 cObjects[5][26][2] = "property_name (sysname, Input)";
			 cObjects[5][26][3] = "level0_object_type (varchar(128), Input)";
			 cObjects[5][26][4] = "level0_object_name (sysname, input)";
			 cObjects[5][26][5] = "level1_object_type (varchar(128), Input)";
			 cObjects[5][26][6] = "level1_object_name (sysname, input)";
			 cObjects[5][26][7] = "level2_object_type (varchar(128), Input)";
			 cObjects[5][26][8] = "level2_object_name (sysname, Input)";

	cObjects[6][0][0] = i18n("14,Security Functions");
		cObjects[6][1][0] = "02,IS_MEMBER";
			 cObjects[6][1][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[6][1][2] = "group_or_role (sysname, Input)";
		cObjects[6][2][0] = "03,IS_SRVROLEMEMBER";
			 cObjects[6][2][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[6][2][2] = "role (sysname, Input)";
			 cObjects[6][2][3] = "login (sysname, Input)";
		cObjects[6][3][0] = "02,SUSER_ID";
			 cObjects[6][3][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[6][3][2] = "login (nchar, Input)";
		cObjects[6][4][0] = "02,SUSER_NAME";
			 cObjects[6][4][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[6][4][2] = "server_user_id (int, Input)";
		cObjects[6][5][0] = "02,SUSER_SID";
			 cObjects[6][5][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[6][5][2] = "login (sysname, Input)";
		cObjects[6][6][0] = "02,SUSER_SNAME";
			 cObjects[6][6][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[6][6][2] = "server_user_id (is varbinary(85), Input)";
		cObjects[6][7][0] = "02,USER_ID";
			 cObjects[6][7][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[6][7][2] = "user (nchar, input)";
		cObjects[6][8][0] = "02,USER_NAME";
			 cObjects[6][8][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[6][8][2] = "id (int, Input)";
		cObjects[6][9][0] = "01,USER";
			 cObjects[6][9][1] = "RETURN_VALUE (char, Return Value)";
		cObjects[6][10][0] = "02,HAS_DBACCESS";
			 cObjects[6][10][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[6][10][2] = "database_name (sysname, Input)";
		cObjects[6][11][0] = "02,fn_trace_getenvinfo";
			 cObjects[6][11][1] = "TABLE_RETURN_VALUE (Result Column)";
			 cObjects[6][11][2] = "trace_id (int, Input)";
		cObjects[6][12][0] = "02,fn_trace_getfilterinfo";
			 cObjects[6][12][1] = "TABLE_RETURN_VALUE (Result Column)";
			 cObjects[6][12][2] = "trace_id (int, Input)";
		cObjects[6][13][0] = "02,fn_trace_getinfo";
			 cObjects[6][13][1] = "TABLE_RETURN_VALUE (Result Column)";
			 cObjects[6][13][2] = "trace_id (int, Input)";
		cObjects[6][14][0] = "03,fn_trace_gettable";
			 cObjects[6][14][1] = "TABLE_RETURN_VALUE (Result Column)";
			 cObjects[6][14][2] = "filename (nvarchar(256), Input)";
			 cObjects[6][14][3] = "numfiles (int, Input)";

	cObjects[7][0][0] = i18n("23,String Functions");
		cObjects[7][1][0] = "02,ASCII";
 			cObjects[7][1][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[7][1][2] = "character_expression (varchar, Input)";
		cObjects[7][2][0] = "02,CHAR";
			 cObjects[7][2][1] = "RETURN_VALUE (char(1), Return Value)";
			 cObjects[7][2][2] = "integer_expression (int, Input)";
		cObjects[7][3][0] = "04,CHARINDEX";
			 cObjects[7][3][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[7][3][2] = "expression1 (varchar, Input)";
			 cObjects[7][3][3] = "expression2 (varchar, Input)";
			 cObjects[7][3][4] = "start_location (int, Input)";
		cObjects[7][4][0] = "03,DIFFERENCE";
			 cObjects[7][4][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[7][4][2] = "character_expression1 (varchar, Input)";
			 cObjects[7][4][3] = "character_expression2 (varchar, Input)";
		cObjects[7][5][0] = "03,LEFT";
			 cObjects[7][5][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][5][2] = "character_expression (varchar, Input)";
			 cObjects[7][5][3] = "integer_expression (int, Input)";
		cObjects[7][6][0] = "02,LEN";
			 cObjects[7][6][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[7][6][2] = "string_expression (varchar, Input)";
		cObjects[7][7][0] = "02,LOWER";
			 cObjects[7][7][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][7][2] = "character_expression (varchar, Input)";
		cObjects[7][8][0] = "02,LTRIM";
			 cObjects[7][8][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][8][2] = "character_expression (varchar, Input)";
		cObjects[7][9][0] = "02,NCHAR";
			 cObjects[7][9][1] = "RETURN_VALUE (nchar(1), Return Value)";
			 cObjects[7][9][2] = "integer_expression (int, Input)";
		cObjects[7][10][0] = "03,PATINDEX";
			 cObjects[7][10][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[7][10][2] = "pattern (int, Input)";
			 cObjects[7][10][3] = "expression (varchar, Input)";
		cObjects[7][11][0] = "04,REPLACE";
			 cObjects[7][11][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][11][2] = "string_expression1 (varchar, Input)";
			 cObjects[7][11][3] = "string_expression2 (varchar, Input)";
			 cObjects[7][11][4] = "string_expression3 (varchar, Input)";
		cObjects[7][12][0] = "03,QUOTENAME";
			 cObjects[7][12][1] = "RETURN_VALUE (nvarchar(129), Return Value)";
			 cObjects[7][12][2] = "character_string (sysname, Input)";
			 cObjects[7][12][3] = "quote_character (varchar, Input)";
		cObjects[7][13][0] = "03,REPLICATE";
			 cObjects[7][13][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][13][2] = "character_expression (varchar, Input)";
			 cObjects[7][13][3] = "integer_expression (int, Input)";
		cObjects[7][14][0] = "02,REVERSE";
			 cObjects[7][14][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][14][2] = "character_expression (varchar, Input)";
		cObjects[7][15][0] = "03,RIGHT";
			 cObjects[7][15][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][15][2] = "character_expression (varchar, Input)";
			 cObjects[7][15][3] = "integer_expression (int, Input)";
		cObjects[7][16][0] = "02,RTRIM";
			 cObjects[7][16][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][16][2] = "character_expression (varchar, Input)";
		cObjects[7][17][0] = "02,SOUNDEX";
			 cObjects[7][17][1] = "RETURN_VALUE (char, Return Value)";
			 cObjects[7][17][2] = "character_expression (varchar, Input)";
		cObjects[7][18][0] = "02,SPACE";
			 cObjects[7][18][1] = "RETURN_VALUE (char, Return Value)";
			 cObjects[7][18][2] = "integer_expression (int, Input)";
		cObjects[7][19][0] = "04,STR";
			 cObjects[7][19][1] = "RETURN_VALUE (char, Return Value)";
			 cObjects[7][19][2] = "float_expression (float, Input)";
			 cObjects[7][19][3] = "length (int, Input)";
			 cObjects[7][19][4] = "decimal (int, input)";
		cObjects[7][20][0] = "05,STUFF";
			 cObjects[7][20][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][20][2] = "character_expression1 (varchar, Input)";
			 cObjects[7][20][3] = "start (int, input)";
			 cObjects[7][20][4] = "length (int, Input)";
			 cObjects[7][20][5] = "character_expression2 (varchar, Input)";
		cObjects[7][21][0] = "04,SUBSTRING";
			 cObjects[7][21][1] = "RETURN_VALUE (varchar_nvarchar_varbinary, Return Value)";
			 cObjects[7][21][2] = "expression1 (varchar_nvarchar_varbinary, Input)";
			 cObjects[7][21][3] = "start (int, Input)";
			 cObjects[7][21][4] = "length (int, input)";
		cObjects[7][22][0] = "02,UNICODE";
			 cObjects[7][22][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[7][22][2] = "ncharacter_expression (nvarchar, Input)";
		cObjects[7][23][0] = "02,UPPER";
			 cObjects[7][23][1] = "RETURN_VALUE (varchar, Return Value)";
			 cObjects[7][23][2] = "character_expression (varchar, Input)";


	cObjects[8][0][0] = i18n("38,System Functions");
		cObjects[8][1][0] = "01,APP_NAME";
			 cObjects[8][1][1] = "RETURN_VALUE (nvarchar(128), Return Value)";
		cObjects[8][2][0] = "02,CASE";
			 cObjects[8][2][1] = "RETURN_VALUE (boolean, Return Value)";
			 cObjects[8][2][2] = "input_expression (expression , Input)";
		cObjects[8][3][0] = "03,CAST";
			 cObjects[8][3][1] = "RETURN_VALUE (Return Value)";
			 cObjects[8][3][2] = "expression (Input)";
			 cObjects[8][3][3] = "type_name (Input)";
		cObjects[8][4][0] = "02,COALESCE";
			 cObjects[8][4][1] = "RETURN_VALUE (Return Value)";
			 cObjects[8][4][2] = "expression (Input)";
		cObjects[8][5][0] = "04,CONVERT";
			 cObjects[8][5][1] = "RETURN_VALUE (Return Value)";
			 cObjects[8][5][2] = "data_type (Input)";
			 cObjects[8][5][3] = "expression (Input)";
			 cObjects[8][5][4] = "style (varchar, Input)";
		cObjects[8][6][0] = "01,CURRENT_TIMESTAMP";
			 cObjects[8][6][1] = "RETURN_VALUE (datetime, Return Value)";
		cObjects[8][7][0] = "01,CURRENT_USER";
			 cObjects[8][7][1] = "RETURN_VALUE (sysname, Return Value)";
		cObjects[8][8][0] = "02,DATALENGTH";
			 cObjects[8][8][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[8][8][2] = "expression (Input)";
		cObjects[8][9][0] = "01,@@ERROR";
			 cObjects[8][9][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[8][10][0] = "03,FORMATMESSAGE";
			 cObjects[8][10][1] = "RETURN_VALUE (nvarchar, Return Value)";
			 cObjects[8][10][2] = "msg_number (int, Input)";
			 cObjects[8][10][3] = "param_value (Input)";
		cObjects[8][11][0] = "02,GETANSINULL";
			 cObjects[8][11][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[8][11][2] = "database (nchar, Input)";
		cObjects[8][12][0] = "01,HOST_ID";
			 cObjects[8][12][1] = "RETURN_VALUE (char(8), Return Value)";
		cObjects[8][13][0] = "01,HOST_NAME";
			 cObjects[8][13][1] = "RETURN_VALUE (nchar, Return Value)";
		cObjects[8][14][0] = "02,IDENT_INCR";
			 cObjects[8][14][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[8][14][2] = "table_or_view (nvarchar, Input)";
		cObjects[8][15][0] = "02,IDENT_SEED";
			 cObjects[8][15][1] = "RETURN_VALUE (numeric, Return Value)";
			 cObjects[8][15][2] = "table_or_view (nvarchar, Input)";
		cObjects[8][16][0] = "01,@@IDENTITY";
			 cObjects[8][16][1] = "RETURN_VALUE (numeric, Return Value)";
		cObjects[8][17][0] = "04,IDENTITY";
			 cObjects[8][17][1] = "RETURN_VALUE (nvarchar, Return Value)";
			 cObjects[8][17][2] = "data_type (Input)";
			 cObjects[8][17][3] = "seed (int, Input)";
			 cObjects[8][17][4] = "increment (int, Input)";
		cObjects[8][18][0] = "02,ISDATE";
			 cObjects[8][18][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[8][18][2] = "expression (int, Input)";
		cObjects[8][19][0] = "03,ISNULL";
			 cObjects[8][19][1] = "RETURN_VALUE (Return Value)";
			 cObjects[8][19][2] = "check_expression (Input)";
			 cObjects[8][19][3] = "replacement_value (Input)";
		cObjects[8][20][0] = "02,ISNUMERIC";
			 cObjects[8][20][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[8][20][2] = "expression (Input)";
		cObjects[8][21][0] = "01,NEWID";
			 cObjects[8][21][1] = "RETURN_VALUE (uniqueidentifier, Return Value)";
		cObjects[8][22][0] = "03,NULLIF";
			 cObjects[8][22][1] = "RETURN_VALUE (Return Value)";
			 cObjects[8][22][2] = "expression1 (Input)";
			 cObjects[8][22][3] = "expression2 (Input)";
		cObjects[8][23][0] = "03,PARSENAME";
			 cObjects[8][23][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[8][23][2] = "object_name (sysname, Input)";
			 cObjects[8][23][3] = "object_piece (int, Input)";
		cObjects[8][24][0] = "03,PERMISSIONS";
			 cObjects[8][24][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[8][24][2] = "objectid (int, Input)";
			 cObjects[8][24][3] = "column (nvarchar, Input)";
		cObjects[8][25][0] = "01,@@ROWCOUNT";
			 cObjects[8][25][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[8][26][0] = "01,ROWCOUNT_BIG";
			 cObjects[8][26][1] = "RETURN_VALUE (bigint, Return Value)";
		cObjects[8][27][0] = "01,SESSION_USER";
			 cObjects[8][27][1] = "RETURN_VALUE (nchar, Return Value)";
		cObjects[8][28][0] = "03,STATS_DATE";
			 cObjects[8][28][1] = "RETURN_VALUE (datetime, Return Value)";
			 cObjects[8][28][2] = "table_id (int, input)";
			 cObjects[8][28][3] = "index_id (int, Input)";
		cObjects[8][29][0] = "01,SYSTEM_USER";
			 cObjects[8][29][1] = "RETURN_VALUE (nchar, Return Value)";
		cObjects[8][30][0] = "01,@@TRANCOUNT";
			 cObjects[8][30][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[8][31][0] = "02,USER_NAME";
			 cObjects[8][31][1] = "RETURN_VALUE (nchar, Return Value)";
			 cObjects[8][31][2] = "id (int, Input)";
		cObjects[8][32][0] = "03,COLLATIONPROPERTY";
			 cObjects[8][32][1] = "RETURN_VALUE (sql_variant, Return Value)";
			 cObjects[8][32][2] = "collation_name (nvarchar(128), Input)";
			 cObjects[8][32][3] = "property (varchar(128), Input)";
		cObjects[8][33][0] = "01,SCOPE_IDENTITY";
			 cObjects[8][33][1] = "RETURN_VALUE (sql_variant, Return Value)";
		cObjects[8][34][0] = "02,SERVERPROPERTY";
			 cObjects[8][34][1] = "RETURN_VALUE (sql_variant, Return Value)";
			 cObjects[8][34][2] = "propertyname (nvarchar, input)";
		cObjects[8][35][0] = "02,SESSIONPROPERTY";
			 cObjects[8][35][1] = "RETURN_VALUE (sql_variant, Return Value)";
			 cObjects[8][35][2] = "option (nvarchar, input)";
		cObjects[8][36][0] = "01,fn_helpcollations";
			 cObjects[8][36][1] = "TABLE_RETURN_VALUE (Result Column)";
		cObjects[8][37][0] = "01,fn_serversharddrives";
			 cObjects[8][37][1] = "TABLE_RETURN_VALUE (Result Column)";
		cObjects[8][38][0] = "01,fn_virtualservernodes";
			 cObjects[8][38][1] = "TABLE_RETURN_VALUE (Result Column)";

	cObjects[9][0][0] = i18n("11,System Statistical Functions");
		cObjects[9][1][0] = "01,@@CPU_BUSY";
			 cObjects[9][1][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][2][0] = "01.IDLE";
			 cObjects[9][2][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][3][0] = "01,@@IO_BUSY";
			 cObjects[9][3][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][4][0] = "01,@@PACK_RECEIVED";
			 cObjects[9][4][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][5][0] = "01,@@PACK_SEND";
			 cObjects[9][5][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][6][0] = "01,@@PACKET_ERRORS";
			 cObjects[9][6][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][7][0] = "01,@@TIMETICKS";
			 cObjects[9][7][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][8][0] = "01,@@TOTAL_ERRORS";
			 cObjects[9][8][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][9][0] = "01,@@TOTAL_READ";
			 cObjects[9][9][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][10][0] = "01,@@TOTAL_WRITE";
			 cObjects[9][10][1] = "RETURN_VALUE (int, Return Value)";
		cObjects[9][11][0] = "03,fn_virtualfilestats";
			 cObjects[9][11][1] = "TABLE_RETURN_VALUE (Result Column)";
			 cObjects[9][11][2] = "database_id (int, Input)";
			 cObjects[9][11][3] = "file_id (int, Input)";

	cObjects[10][0][0] = i18n("03,Text and Image Functions");
		cObjects[10][1][0] = "03,PATINDEX";
			 cObjects[10][1][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[10][1][2] = "pattern (nvarchar, Input)";
			 cObjects[10][1][3] = "expression (nvarchar, Input)";
		cObjects[10][2][0] = "02,TEXTPTR";
			 cObjects[10][2][1] = "RETURN_VALUE (varbinary, Return Value)";
			 cObjects[10][2][2] = "column (Input)";
		cObjects[10][3][0] = "03,TEXTVALID";
			 cObjects[10][3][1] = "RETURN_VALUE (int, Return Value)";
			 cObjects[10][3][2] = "table.column (nvarchar, Input)";
			 cObjects[10][3][3] = "text_ptr (Input)";

	cObjects[11][0][0] = i18n("06,Rowset");
		cObjects[11][1][0] = "05,CONTAINSTABLE";
			 cObjects[11][1][1] = "RETURN_ROWSET (Result Column)";
			 cObjects[11][1][2] = "table (Input)";
			 cObjects[11][1][3] = "column (Input)";
			 cObjects[11][1][4] = "contains_search_condition (varchar, Input)";
			 cObjects[11][1][5] = "top_n_by_rank (int, Input)";
		cObjects[11][2][0] = "05,FREETEXTTABLE";
			 cObjects[11][2][1] = "RETURN_ROWSET (Result Column)";
			 cObjects[11][2][2] = "table (Input)";
			 cObjects[11][2][3] = "column (Input)";
			 cObjects[11][2][4] = "freetext_string (varchar, Input)";
			 cObjects[11][2][5] = "top_n_by_rank (int, Input)";
		cObjects[11][3][0] = "03,OPENQUERY";
			 cObjects[11][3][1] = "RETURN_ROWSET (Result Column)";
			 cObjects[11][3][2] = "linked_server (Input)";
			 cObjects[11][3][3] = "query (varchar, Input)";
		cObjects[11][4][0] = "05,OPENROWSET";
			 cObjects[11][4][1] = "RETURN_ROWSET (Result Column)";
			 cObjects[11][4][2] = "provider_name (varchar, Input)";
			 cObjects[11][4][3] = "provider_string (varchar, Input)";
			 cObjects[11][4][4] = "Object (Input)";
			 cObjects[11][4][5] = "query (varchar, Input)";
		cObjects[11][5][0] = "03,OPENDATASOURCE";
			 cObjects[11][5][1] = "RETURN_ROWSET (Result Column)";
			 cObjects[11][5][2] = "provider_name (varchar, Input)";
			 cObjects[11][5][3] = "init_string (varchar, Input)";
		cObjects[11][6][0] = "04,OPENXML";
			 cObjects[11][6][1] = "RETURN_ROWSET (Result Column)";
			 cObjects[11][6][2] = "idoc (int, Input)";
			 cObjects[11][6][3] = "rowpattern (nvarchar, Input)";
			 cObjects[11][6][4] = "flags (byte, Input)";

	cObjects[12][0][0] = i18n("29,System Data Types");
		cObjects[12][1][0] = "00,bigint";
		cObjects[12][2][0] = "00,binary";
		cObjects[12][3][0] = "00,bit";
		cObjects[12][4][0] = "00,char";
		cObjects[12][5][0] = "00,cursor";
		cObjects[12][6][0] = "00,datetime";
		cObjects[12][7][0] = "00,decimal";
		cObjects[12][8][0] = "00,float";
		cObjects[12][9][0] = "00,image";
		cObjects[12][10][0] = "00,int";
		cObjects[12][11][0] = "00,money";
		cObjects[12][12][0] = "00,nchar";
		cObjects[12][13][0] = "00,ntext";
		cObjects[12][14][0] = "00,numeric";
		cObjects[12][15][0] = "00,nvarchar";
		cObjects[12][16][0] = "00,real";
		cObjects[12][17][0] = "00,rowversion";
		cObjects[12][18][0] = "00,smalldatetime";
		cObjects[12][19][0] = "00,smallint";
		cObjects[12][20][0] = "00,smallmoney";
		cObjects[12][21][0] = "00,sql_variant";
		cObjects[12][22][0] = "00,sysname";
		cObjects[12][23][0] = "00,table";
		cObjects[12][24][0] = "00,text";
		cObjects[12][25][0] = "00,timestamp";
		cObjects[12][26][0] = "00,tinyint";
		cObjects[12][27][0] = "00,uniqueidentifier";
		cObjects[12][28][0] = "00,varbinary";
		cObjects[12][29][0] = "00,varchar";
}

void MyAppBar::initTemplates()
{
	QString tmp;
	int t_func;

	tObjects[0][0][0] = "03,Attach and Detach Database";
		 tObjects[0][1][0] = "Attach_Database_Template";
			tObjects[0][1][1] = "-- =============================================\n";
			tObjects[0][1][1] += "-- Attach database via sp_attach_db\n";
			tObjects[0][1][1] += "-- =============================================\n";
			tObjects[0][1][1] += "EXECUTE sp_attach_db @dbname    = N'<database_name, sysname, test_db>', \n";
			tObjects[0][1][1] += "           @filename1 = N'<filename1, nvarchar(260), c:\\program files\\microsoft sql server\\mssql\\data\\test_db.mdf>',\n";
			tObjects[0][1][1] += "           @filename2 = N'<filename2, nvarchar(260), c:\\program files\\microsoft sql server\\mssql\\data\\test_db_log.ldf>'\n";
			tObjects[0][1][1] += "GO";

		 tObjects[0][2][0] = "Attach_Single_File_Database_Template";
			tObjects[0][2][1] = "-- =============================================\n";
			tObjects[0][2][1] += "-- Attach database via sp_attach_single_file_db\n";
			tObjects[0][2][1] += "-- =============================================\n";
			tObjects[0][2][1] += "-- note: it builds a new log file and performs additional cleanup work \n";
			tObjects[0][2][1] += "--   to remove replication from the newly attached database\n";
			tObjects[0][2][1] += "EXECUTE sp_attach_single_file_db @dbname   = N'<database_name, sysname, test_db>', \n";
			tObjects[0][2][1] += "               @physname = N'<physname, nvarchar(260), c:\\program files\\microsoft sql server\\mssql\\data\\test_db.mdf>'\n";
			tObjects[0][2][1] += "GO\n";

		 tObjects[0][3][0] = "Detach_Database_Template";
			tObjects[0][3][1] = "-- =============================================\n";
			tObjects[0][3][1] += "-- Detach database via sp_detach_db\n";
			tObjects[0][3][1] += "-- =============================================\n";
			tObjects[0][3][1] += "EXECUTE sp_detach_db @dbname     = N'<database_name, sysname, test_db>', \n";
			tObjects[0][3][1] += "            @skipchecks = N'<skipchecks, nvarchar(10), TRUE>'\n";
			tObjects[0][3][1] += "GO\n";

	tObjects[1][0][0] = "05,Create Database";
		 tObjects[1][1][0] = "Create Database Basic Template";
			tObjects[1][1][1] = "-- =============================================\n";
			tObjects[1][1][1] += "-- Basic Create Database Template\n";
			tObjects[1][1][1] += "-- =============================================\n";
			tObjects[1][1][1] += "IF EXISTS (SELECT * \n";
			tObjects[1][1][1] += "	   FROM   master..sysdatabases \n";
			tObjects[1][1][1] += "	   WHERE  name = N'<database_name, sysname, test_db>')\n";
			tObjects[1][1][1] += "	DROP DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][1][1] += "GO\n";
			tObjects[1][1][1] += "\n";
			tObjects[1][1][1] += "CREATE DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][1][1] += "GO\n";

		 tObjects[1][2][0] = "Create Database for ATTACH";
			tObjects[1][2][1] = "-- =============================================\n";
			tObjects[1][2][1] += "-- Create database for attach\n";
			tObjects[1][2][1] += "-- =============================================\n";
			tObjects[1][2][1] += "IF EXISTS (SELECT * \n";
			tObjects[1][2][1] += "	   FROM   master..sysdatabases \n";
			tObjects[1][2][1] += "	   WHERE  name = N'<database_name, sysname, test_db>')\n";
			tObjects[1][2][1] += "	DROP DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][2][1] += "GO\n";
			tObjects[1][2][1] += "\n";
			tObjects[1][2][1] += "CREATE DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][2][1] += "ON PRIMARY \n";
			tObjects[1][2][1] += "	(FILENAME = N'<os_file_name, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db.mdf>')\n";
			tObjects[1][2][1] += "FOR ATTACH\n";
			tObjects[1][2][1] += "GO\n";
			tObjects[1][2][1] += "\n";

		 tObjects[1][3][0] = "Create Database on Multiple File Groups";
			tObjects[1][3][1] = "-- =============================================\n";
			tObjects[1][3][1] += "-- Create database on mulitple file groups\n";
			tObjects[1][3][1] += "-- =============================================\n";
			tObjects[1][3][1] += "IF EXISTS (SELECT * \n";
			tObjects[1][3][1] += "	   FROM   master..sysdatabases \n";
			tObjects[1][3][1] += "	   WHERE  name = N'<database_name, sysname, test_db>')\n";
			tObjects[1][3][1] += "	DROP DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][3][1] += "GO\n";
			tObjects[1][3][1] += "\n";
			tObjects[1][3][1] += "CREATE DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][3][1] += "ON PRIMARY\n";
			tObjects[1][3][1] += "	( NAME = <logical_file_name_1, , test_db_file_1>,\n";
			tObjects[1][3][1] += "	  FILENAME = N'<os_file_name_1, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_1.mdf>',\n";
			tObjects[1][3][1] += "          SIZE = 1MB,\n";
			tObjects[1][3][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][3][1] += "          FILEGROWTH = 10%),\n";
			tObjects[1][3][1] += "	\n";
			tObjects[1][3][1] += "	( NAME = <logical_file_name_2, , test_db_file_2>,\n";
			tObjects[1][3][1] += "	  FILENAME = N'<os_file_name_2, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_2.ndf>',\n";
			tObjects[1][3][1] += "          SIZE = 1MB,\n";
			tObjects[1][3][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][3][1] += "          FILEGROWTH = 10%),\n";
			tObjects[1][3][1] += "\n";
			tObjects[1][3][1] += "FILEGROUP <file_group_name_1, , test_db_group_1>\n";
			tObjects[1][3][1] += "	( NAME = <logical_file_name_3, , test_db_file_3>,\n";
			tObjects[1][3][1] += "	  FILENAME = N'<os_file_name_3, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_3.ndf>',\n";
			tObjects[1][3][1] += "          SIZE = 1MB,\n";
			tObjects[1][3][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][3][1] += "          FILEGROWTH = 10%),\n";
			tObjects[1][3][1] += "	\n";
			tObjects[1][3][1] += "	( NAME = <logical_file_name_4, , test_db_file_4>,\n";
			tObjects[1][3][1] += "	  FILENAME = N'<os_file_name_4, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_4.ndf>',\n";
			tObjects[1][3][1] += "          SIZE = 1MB,\n";
			tObjects[1][3][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][3][1] += "          FILEGROWTH = 10%),\n";
			tObjects[1][3][1] += "\n";
			tObjects[1][3][1] += "FILEGROUP <file_group_name_2, , test_db_group_2>\n";
			tObjects[1][3][1] += "	( NAME = <logical_file_name_5, , test_db_file_5>,\n";
			tObjects[1][3][1] += "	  FILENAME = N'<os_file_name_5, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_5.ndf>',\n";
			tObjects[1][3][1] += "          SIZE = 1MB,\n";
			tObjects[1][3][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][3][1] += "          FILEGROWTH = 10%)\n";
			tObjects[1][3][1] += "\n";
			tObjects[1][3][1] += "LOG ON\n";
			tObjects[1][3][1] += "	( NAME = <logical_log_file_name_1, , test_db_log_file_1>,\n";
			tObjects[1][3][1] += "	  FILENAME = N'<os_log_file_name_1, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_log_1.ldf>',\n";
			tObjects[1][3][1] += "          SIZE = 1MB,\n";
			tObjects[1][3][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][3][1] += "          FILEGROWTH = 10%)\n";
			tObjects[1][3][1] += "GO\n";

		 tObjects[1][4][0] = "Create Database on Multiple Data and Transaction Log Files";
			tObjects[1][4][1] = "-- =============================================\n";
			tObjects[1][4][1] += "-- Create database on multiple data and transaction log files\n";
			tObjects[1][4][1] += "-- =============================================\n";
			tObjects[1][4][1] += "IF EXISTS (SELECT * \n";
			tObjects[1][4][1] += "	   FROM   master..sysdatabases \n";
			tObjects[1][4][1] += "	   WHERE  name = N'<database_name, sysname, test_db>')\n";
			tObjects[1][4][1] += "	DROP DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][4][1] += "GO\n";
			tObjects[1][4][1] += "\n";
			tObjects[1][4][1] += "CREATE DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][4][1] += "ON PRIMARY \n";
			tObjects[1][4][1] += "	( NAME = <logical_file_name_1, , test_db_file_1>,\n";
			tObjects[1][4][1] += "          FILENAME = N'<os_file_name_1, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_1.mdf>',\n";
			tObjects[1][4][1] += "          SIZE = 1MB,\n";
			tObjects[1][4][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][4][1] += "          FILEGROWTH = 10%),\n";
			tObjects[1][4][1] += "\n";
			tObjects[1][4][1] += "	( NAME = <logical_file_name_2, ,test_db_file_2>,\n";
			tObjects[1][4][1] += "	  FILENAME = N'<os_file_name_2, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_2.ndf>',\n";
			tObjects[1][4][1] += "          SIZE = 1MB,\n";
			tObjects[1][4][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][4][1] += "          FILEGROWTH = 10%)\n";
			tObjects[1][4][1] += "\n";
			tObjects[1][4][1] += "LOG ON \n";
			tObjects[1][4][1] += "	( NAME = <logical_log_file_name_1, ,test_db_log_file_1>,\n";
			tObjects[1][4][1] += "	  FILENAME = N'<os_log_file_name_1, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_log_1.ldf>',\n";
			tObjects[1][4][1] += "          SIZE = 512KB,\n";
			tObjects[1][4][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][4][1] += "          FILEGROWTH = 10%),\n";
			tObjects[1][4][1] += "	\n";
			tObjects[1][4][1] += "	( NAME = <logical_log_file_name_2, ,test_db_log_file_2>,\n";
			tObjects[1][4][1] += "	  FILENAME = N'<os_log_file_name_2, , c:\\program files\\microsoft sql server\\mssql\\data\\test_db_log_2.ldf>',\n";
			tObjects[1][4][1] += "          SIZE = 512KB,\n";
			tObjects[1][4][1] += "          MAXSIZE = 200MB,\n";
			tObjects[1][4][1] += "          FILEGROWTH = 10%)\n";
			tObjects[1][4][1] += "GO\n";

		 tObjects[1][5][0] = "Create Database Specifying Collation";
			tObjects[1][5][1] = "-- =============================================\n";
			tObjects[1][5][1] += "-- Create database specifying collation\n";
			tObjects[1][5][1] += "-- =============================================\n";
			tObjects[1][5][1] += "IF EXISTS (SELECT * \n";
			tObjects[1][5][1] += "	   FROM   master..sysdatabases \n";
			tObjects[1][5][1] += "	   WHERE  name = N'<database_name, sysname, test_db>')\n";
			tObjects[1][5][1] += "	DROP DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][5][1] += "GO\n";
			tObjects[1][5][1] += "\n";
			tObjects[1][5][1] += "CREATE DATABASE <database_name, sysname, test_db>\n";
			tObjects[1][5][1] += "COLLATE <collation_name, , Latin1_General_CI_AI>\n";
			tObjects[1][5][1] += "GO\n";
			tObjects[1][5][1] += "\n";
			tObjects[1][5][1] += "-- ==========================================================\n";
			tObjects[1][5][1] += "-- sample query to retrieve all types of collations available\n";
			tObjects[1][5][1] += "-- ==========================================================\n";
			tObjects[1][5][1] += "-- SELECT * \n";
			tObjects[1][5][1] += "-- FROM   ::fn_helpcollations()\n";
			tObjects[1][5][1] += "-- GO\n";

	tObjects[2][0][0] = "03,Create Function";
		 tObjects[2][1][0] = "Create Inline Function";
			tObjects[2][1][1] = "-- =============================================\n";
			tObjects[2][1][1] += "-- Create inline function (IF)\n";
			tObjects[2][1][1] += "-- =============================================\n";
			tObjects[2][1][1] += "IF EXISTS (SELECT * \n";
			tObjects[2][1][1] += "	   FROM   sysobjects \n";
			tObjects[2][1][1] += "	   WHERE  name = N'<inline_function_name, sysname, test_function>')\n";
			tObjects[2][1][1] += "	DROP FUNCTION <inline_function_name, sysname, test_function>\n";
			tObjects[2][1][1] += "GO\n";
			tObjects[2][1][1] += "\n";
			tObjects[2][1][1] += "CREATE FUNCTION <inline_function_name, sysname, test_function> \n";
			tObjects[2][1][1] += "	(<@param1, sysname, @p1> <data_type_for_param1, , int>, \n";
			tObjects[2][1][1] += "	 <@param2, sysname, @p2> <data_type_for_param2, , char>)\n";
			tObjects[2][1][1] += "RETURNS TABLE \n";
			tObjects[2][1][1] += "AS\n";
			tObjects[2][1][1] += "	RETURN SELECT   @p1 AS c1, \n";
			tObjects[2][1][1] += "			@p2 AS c2\n";
			tObjects[2][1][1] += "GO\n";
			tObjects[2][1][1] += "\n";
			tObjects[2][1][1] += "-- =============================================\n";
			tObjects[2][1][1] += "-- Example to execute function\n";
			tObjects[2][1][1] += "-- =============================================\n";
			tObjects[2][1][1] += "SELECT * \n";
			tObjects[2][1][1] += "FROM <owner, , dbo>.<inline_function_name, sysname, test_function> \n";
			tObjects[2][1][1] += "	(<value_for_@param1, , 1>, \n";
			tObjects[2][1][1] += "	 <value_for_@param2, , 'a'>)\n";
			tObjects[2][1][1] += "GO\n";

		 tObjects[2][2][0] = "Create Scalar Function";
         tObjects[2][2][1] = " =============================================\n";
         tObjects[2][2][1] += "-- Create scalar function (FN)\n";
         tObjects[2][2][1] += "-- =============================================\n";
         tObjects[2][2][1] += "IF EXISTS (SELECT * \n";
         tObjects[2][2][1] += "	   FROM   sysobjects \n";
         tObjects[2][2][1] += "	   WHERE  name = N'<scalar_function_name, sysname, test_function>')\n";
         tObjects[2][2][1] += "	DROP FUNCTION <scalar_function_name, sysname, test_function>\n";
         tObjects[2][2][1] += "GO\n";
         tObjects[2][2][1] += "\n";
         tObjects[2][2][1] += "CREATE FUNCTION <scalar_function_name, sysname, test_function> \n";
         tObjects[2][2][1] += "	(<@param1, sysname, @p1> <data_type_for_param1, , int>, \n";
         tObjects[2][2][1] += "	 <@param2, sysname, @p2> <data_type_for_param2, , int>)\n";
         tObjects[2][2][1] += "RETURNS <function_data_type, ,int>\n";
         tObjects[2][2][1] += "AS\n";
         tObjects[2][2][1] += "BEGIN\n";
         tObjects[2][2][1] += "	<function_body, , RETURN @p1 + @p2 >\n";
         tObjects[2][2][1] += "--	eg.\n";
         tObjects[2][2][1] += "--	DECLARE @sum AS int\n";
         tObjects[2][2][1] += "--	SELECT @sum = @p1 + @P2\n";
         tObjects[2][2][1] += "--	RETURN @sum\n";
         tObjects[2][2][1] += "END\n";
         tObjects[2][2][1] += "GO\n";
         tObjects[2][2][1] += "\n";
         tObjects[2][2][1] += "-- =============================================\n";
         tObjects[2][2][1] += "-- Example to execute function\n";
         tObjects[2][2][1] += "-- =============================================\n";
         tObjects[2][2][1] += "SELECT <owner, , dbo>.<scalar_function_name, sysname, test_function> \n";
         tObjects[2][2][1] += "	(<value_for_@param1, , 1>, \n";
         tObjects[2][2][1] += "	 <value_for_@param2, , 2>)\n";
         tObjects[2][2][1] += "GO\n";

		 tObjects[2][3][0] = "Create Table Function";
        tObjects[2][3][1] = " =============================================\n";
        tObjects[2][3][1] += "-- Create table function (TF)\n";
        tObjects[2][3][1] += "-- =============================================\n";
        tObjects[2][3][1] += "IF EXISTS (SELECT * \n";
        tObjects[2][3][1] += "	   FROM   sysobjects \n";
        tObjects[2][3][1] += "	   WHERE  name = N'<table_function_name, sysname, test_function>')\n";
        tObjects[2][3][1] += "	DROP FUNCTION <table_function_name, sysname, test_function>\n";
        tObjects[2][3][1] += "GO\n";
        tObjects[2][3][1] += "\n";
        tObjects[2][3][1] += "CREATE FUNCTION <table_function_name, sysname, test_function> \n";
        tObjects[2][3][1] += "	(<@param1, sysname, @p1> <data_type_for_param1, , int>, \n";
        tObjects[2][3][1] += "	 <@param2, sysname, @p2> <data_type_for_param2, , char>)\n";
        tObjects[2][3][1] += "RETURNS <@table_variable_name, sysname, @table_var> TABLE \n";
        tObjects[2][3][1] += "	(<column_1, sysname, c1> <data_type_for_column1, , int>, \n";
        tObjects[2][3][1] += "	 <column_2, sysname, c2> <data_type_for_column2, , int>)\n";
        tObjects[2][3][1] += "AS\n";
        tObjects[2][3][1] += "BEGIN\n";
        tObjects[2][3][1] += "	INSERT <@table_variable_name, sysname, @table_var>\n";
        tObjects[2][3][1] += "	SELECT 1, 2\n";
        tObjects[2][3][1] += "	RETURN \n";
        tObjects[2][3][1] += "END\n";
        tObjects[2][3][1] += "GO\n";
        tObjects[2][3][1] += "\n";
        tObjects[2][3][1] += "-- =============================================\n";
        tObjects[2][3][1] += "-- Example to execute function\n";
        tObjects[2][3][1] += "-- =============================================\n";
        tObjects[2][3][1] += "SELECT * FROM <owner, , dbo>.<table_function_name, sysname, test_function>\n";
        tObjects[2][3][1] += "	(<value_for_@param1, , 1>, \n";
        tObjects[2][3][1] += "	 <value_for_@param2, , 2>)\n";
        tObjects[2][3][1] += "GO\n";

	tObjects[3][0][0] = "02,Create Index";
		 tObjects[3][1][0] = "Create Index Basic";
			tObjects[3][1][1] = " =============================================\n";
			tObjects[3][1][1] += "-- Create index basic template\n";
			tObjects[3][1][1] += "-- =============================================\n";
			tObjects[3][1][1] += "CREATE INDEX <index_name, sysname, ind_test>\n";
			tObjects[3][1][1] += "ON <database_name, sysname, pubs>.<owner, sysname, dbo>.<table_or_view_name, sysname, authors> \n";
			tObjects[3][1][1] += "	(<column_1, sysname, au_lname>, \n";
			tObjects[3][1][1] += "	 <column_2, sysname, au_fname>)\n";
			tObjects[3][1][1] += "GO\n";
			tObjects[3][1][1] += "\n";
			tObjects[3][1][1] += "\n";

		 tObjects[3][2][0] = "Create Index Full Syntax";
			tObjects[3][2][1] = " =============================================\n";
			tObjects[3][2][1] += "-- Create index full syntax template\n";
			tObjects[3][2][1] += "-- =============================================\n";
			tObjects[3][2][1] += "CREATE UNIQUE CLUSTERED INDEX <index_name, sysname, ind_test>\n";
			tObjects[3][2][1] += "ON <database_name, sysname, pubs>.<owner, sysname, dbo>.<table_or_view_name, sysname, authors> \n";
			tObjects[3][2][1] += "	(<column_1, sysname, au_lname> DESC, \n";
			tObjects[3][2][1] += "	 <column_2, sysname, au_fname>)\n";
			tObjects[3][2][1] += "WITH\n";
			tObjects[3][2][1] += "PAD_INDEX,\n";
			tObjects[3][2][1] += "FILLFACTOR = <fillfactor_value, , 50>,\n";
			tObjects[3][2][1] += "IGNORE_DUP_KEY,\n";
			tObjects[3][2][1] += "DROP_EXISTING,\n";
			tObjects[3][2][1] += "STATISTICS_NORECOMPUTE\n";
			tObjects[3][2][1] += "ON <name_of_filegroup, , [PRIMARY]>\n";
			tObjects[3][2][1] += "GO\n";

	tObjects[4][0][0] = "03,Create Procedure";
		 tObjects[4][1][0] = "Create Procedure Basic Template";
			tObjects[4][1][1] = " =============================================\n";
			tObjects[4][1][1] += "-- Create procedure basic template\n";
			tObjects[4][1][1] += "-- =============================================\n";
			tObjects[4][1][1] += "-- creating the store procedure\n";
			tObjects[4][1][1] += "IF EXISTS (SELECT name \n";
			tObjects[4][1][1] += "	   FROM   sysobjects \n";
			tObjects[4][1][1] += "	   WHERE  name = N'<procedure_name, sysname, proc_test>' \n";
			tObjects[4][1][1] += "	   AND 	  type = 'P')\n";
			tObjects[4][1][1] += "    DROP PROCEDURE <procedure_name, sysname, proc_test>\n";
			tObjects[4][1][1] += "GO\n";
			tObjects[4][1][1] += "\n";
			tObjects[4][1][1] += "CREATE PROCEDURE <procedure_name, sysname, proc_test> \n";
			tObjects[4][1][1] += "	<@param1, sysname, @p1> <datatype_for_param1, , int> = <default_value_for_param1, , 0>, \n";
			tObjects[4][1][1] += "	<@param2, sysname, @p2> <datatype_for_param2, , int> = <default_value_for_param2, , 0>\n";
			tObjects[4][1][1] += "AS\n";
			tObjects[4][1][1] += "	SELECT @p1, @p2\n";
			tObjects[4][1][1] += "GO\n";
			tObjects[4][1][1] += "\n";
			tObjects[4][1][1] += "-- =============================================\n";
			tObjects[4][1][1] += "-- example to execute the store procedure\n";
			tObjects[4][1][1] += "-- =============================================\n";
			tObjects[4][1][1] += "EXECUTE <procedure_name, sysname, proc_test> <value_for_param1, , 1>, <value_for_param2, , 2>\n";
			tObjects[4][1][1] += "GO\n";

		 tObjects[4][2][0] = "Create Procedure with CURSOR OUTPUT Parameter";
			tObjects[4][2][1] = " =============================================\n";
			tObjects[4][2][1] += "-- Create procedure with CURSOR OUTPUT Parameters\n";
			tObjects[4][2][1] += "-- =============================================\n";
			tObjects[4][2][1] += "-- drop the store procedure if it already exists\n";
			tObjects[4][2][1] += "IF EXISTS (SELECT name \n";
			tObjects[4][2][1] += "	   FROM   sysobjects \n";
			tObjects[4][2][1] += "	   WHERE  name = N'<procedure_name, sysname, proc_test>' \n";
			tObjects[4][2][1] += "	   AND 	  type = 'P')\n";
			tObjects[4][2][1] += "	DROP PROCEDURE <procedure_name, sysname, proc_test>\n";
			tObjects[4][2][1] += "GO\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "-- create the store procedure \n";
			tObjects[4][2][1] += "CREATE PROCEDURE <procedure_name, sysname, proc_test> \n";
			tObjects[4][2][1] += "	<@proc_cursor_name, , @proc_test_cursor> CURSOR VARYING OUTPUT\n";
			tObjects[4][2][1] += "AS\n";
			tObjects[4][2][1] += "SET <@proc_cursor_name, , @proc_test_cursor> = CURSOR FOR\n";
			tObjects[4][2][1] += "	<select_statement, , select 1>\n";
			tObjects[4][2][1] += "OPEN <@proc_cursor_name, , @proc_test_cursor>\n";
			tObjects[4][2][1] += "GO\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "-- =============================================\n";
			tObjects[4][2][1] += "-- example to execute the store procedure\n";
			tObjects[4][2][1] += "-- =============================================\n";
			tObjects[4][2][1] += "DECLARE <@variable_cursor_name, , @test_cursor_variable> CURSOR\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "EXEC <procedure_name, sysname, proc_test> <@proc_cursor_name, , @proc_test_cursor> = <@variable_cursor_name, , @test_cursor_variable> OUTPUT\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "WHILE (@@FETCH_STATUS = 0)\n";
			tObjects[4][2][1] += "BEGIN\n";
			tObjects[4][2][1] += "    FETCH NEXT FROM <@variable_cursor_name, , @test_cursor_variable>\n";
			tObjects[4][2][1] += "    PRINT 'put user defined code here'\n";
			tObjects[4][2][1] += "END\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "CLOSE <@variable_cursor_name, , @test_cursor_variable>\n";
			tObjects[4][2][1] += "\n";
			tObjects[4][2][1] += "DEALLOCATE <@variable_cursor_name, , @test_cursor_variable>\n";
			tObjects[4][2][1] += "GO\n";

		 tObjects[4][3][0] = "Create Procedure with OUTPUT Parameter";
          tObjects[4][3][1] += "-- Create procedure with OUTPUT Parameters\n";
          tObjects[4][3][1] += "-- =============================================\n";
          tObjects[4][3][1] += "-- creating the store procedure\n";
          tObjects[4][3][1] += "IF EXISTS (SELECT name \n";
          tObjects[4][3][1] += "	   FROM   sysobjects \n";
          tObjects[4][3][1] += "	   WHERE  name = N'<procedure_name, sysname, proc_test>' \n";
          tObjects[4][3][1] += "	   AND 	  type = 'P')\n";
          tObjects[4][3][1] += "    DROP PROCEDURE <procedure_name, sysname, proc_test>\n";
          tObjects[4][3][1] += "GO\n";
          tObjects[4][3][1] += "\n";
          tObjects[4][3][1] += "CREATE PROCEDURE <procedure_name, sysname, proc_test> \n";
          tObjects[4][3][1] += "	<@param1, sysname, @p1> <datatype_for_param1, , int> = <default_value_for_param1, , 0>, \n";
          tObjects[4][3][1] += "	<@param2, sysname, @p2> <datatype_for_param2, , int>  OUTPUT\n";
          tObjects[4][3][1] += "AS\n";
          tObjects[4][3][1] += "	SELECT @p2 = @p2 + @p1\n";
          tObjects[4][3][1] += "GO\n";
          tObjects[4][3][1] += "\n";
          tObjects[4][3][1] += "-- =============================================\n";
          tObjects[4][3][1] += "-- example to execute the store procedure\n";
          tObjects[4][3][1] += "-- =============================================\n";
          tObjects[4][3][1] += "DECLARE <@variable_for_output_parameter, sysname, @p2_output> <datatype_for_output_parameter, , int>\n";
          tObjects[4][3][1] += "EXECUTE <procedure_name, sysname, proc_test> <value_for_param1, , 1>, <@variable_for_output_parameter, sysname, @p2_output> OUTPUT\n";
          tObjects[4][3][1] += "SELECT <@variable_for_output_parameter, sysname, @p2_output>\n";
          tObjects[4][3][1] += "GO\n";

	tObjects[5][0][0] = "01,Create Statistics";
		 tObjects[5][1][0] = "Create Statistics Template";
         tObjects[5][1][1] = " =============================================\n";
         tObjects[5][1][1] += "-- Create Statistics Template\n";
         tObjects[5][1][1] += "-- =============================================\n";
         tObjects[5][1][1] += "CREATE STATISTICS <statistics_name, sysname, stats_test> \n";
         tObjects[5][1][1] += "ON <table_name, sysname, t1> \n";
         tObjects[5][1][1] += "	(<column_list, sysname, c1>)\n";
         tObjects[5][1][1] += "WITH\n";
         tObjects[5][1][1] += "SAMPLE <percentage, , 50> PERCENT\n";
         tObjects[5][1][1] += "GO\n";

	tObjects[6][0][0] = "07,Create Table";
		 tObjects[6][1][0] = "Create Table Basic Template";
         tObjects[6][1][1] = " =============================================\n";
         tObjects[6][1][1] += "-- Create table basic template\n";
         tObjects[6][1][1] += "-- =============================================\n";
         tObjects[6][1][1] += "IF EXISTS(SELECT name \n";
         tObjects[6][1][1] += "	  FROM 	 sysobjects \n";
         tObjects[6][1][1] += "	  WHERE  name = N'<table_name, sysname, test_table>' \n";
         tObjects[6][1][1] += "	  AND 	 type = 'U')\n";
         tObjects[6][1][1] += "    DROP TABLE <table_name, sysname, test_table>\n";
         tObjects[6][1][1] += "GO\n";
         tObjects[6][1][1] += "\n";
         tObjects[6][1][1] += "CREATE TABLE <table_name, sysname, test_table> (\n";
         tObjects[6][1][1] += "<column_1, sysname, c1> <datatype_for_column_1, , int> NULL, \n";
         tObjects[6][1][1] += "<column_2, sysname, c2> <datatype_for_column_2, , int> NOT NULL)\n";
         tObjects[6][1][1] += "GO\n";

		 tObjects[6][2][0] = "Create Table Specifying Constraints";
         tObjects[6][2][1] = " =============================================\n";
         tObjects[6][2][1] += "-- Create table specify constraints\n";
         tObjects[6][2][1] += "-- =============================================\n";
         tObjects[6][2][1] += "IF EXISTS (SELECT name \n";
         tObjects[6][2][1] += "	   FROM   sysobjects \n";
         tObjects[6][2][1] += "	   WHERE  name = N'<table_name, sysname, test_table>' \n";
         tObjects[6][2][1] += "	   AND    type = 'U')\n";
         tObjects[6][2][1] += "    DROP TABLE <table_name, sysname, test_table>\n";
         tObjects[6][2][1] += "GO\n";
         tObjects[6][2][1] += "\n";
         tObjects[6][2][1] += "CREATE TABLE <table_name, sysname, test_table> (\n";
         tObjects[6][2][1] += "<column_1, sysname, c1> <datatype_for_column_1, , int> NOT NULL, \n";
         tObjects[6][2][1] += "<column_2, sysname, c2> <datatype_for_column_2, , int> NOT NULL\n";
         tObjects[6][2][1] += "	CONSTRAINT <column_constraint, sysname, check_c2> CHECK(c2 > 0),\n";
         tObjects[6][2][1] += "CONSTRAINT <table_constraint, sysname, pk_c1> PRIMARY KEY (c1) )\n";
         tObjects[6][2][1] += "GO\n";

		 tObjects[6][3][0] = "Create Table Specifying File Groups";
         tObjects[6][3][1] = " =============================================\n";
         tObjects[6][3][1] += "-- Create table specifying filegroups\n";
         tObjects[6][3][1] += "-- =============================================\n";
         tObjects[6][3][1] += "IF EXISTS (SELECT name \n";
         tObjects[6][3][1] += "	   FROM   sysobjects \n";
         tObjects[6][3][1] += "	   WHERE  name = N'<table_name, sysname, test_table>' \n";
         tObjects[6][3][1] += "	   AND 	  type = 'U')\n";
         tObjects[6][3][1] += "    DROP TABLE <table_name, sysname, test_table>\n";
         tObjects[6][3][1] += "GO\n";
         tObjects[6][3][1] += "\n";
         tObjects[6][3][1] += "CREATE TABLE <table_name, sysname, test_table> (\n";
         tObjects[6][3][1] += "<column_1, sysname, c1> <datatype_for_column_1, , int> NOT NULL, \n";
         tObjects[6][3][1] += "<column_2, sysname, c2> <datatype_for_column_2, text|ntext|image, text> )\n";
         tObjects[6][3][1] += "ON <file_group_name_1, , [PRIMARY]>\n";
         tObjects[6][3][1] += "TEXTIMAGE_ON <file_group_name_2, , [PRIMARY]>\n";
         tObjects[6][3][1] += "GO\n";

		 tObjects[6][4][0] = "Create Table with Computed Column";
         tObjects[6][4][1] = " =============================================\n";
         tObjects[6][4][1] += "-- Create table with Computed column\n";
         tObjects[6][4][1] += "-- =============================================\n";
         tObjects[6][4][1] += "IF EXISTS (SELECT name \n";
         tObjects[6][4][1] += "	   FROM   sysobjects \n";
         tObjects[6][4][1] += "	   WHERE  name = N'<table_name, sysname, test_table>' \n";
         tObjects[6][4][1] += "	   AND 	  type = 'U')\n";
         tObjects[6][4][1] += "    DROP TABLE <table_name, sysname, test_table>\n";
         tObjects[6][4][1] += "GO\n";
         tObjects[6][4][1] += "\n";
         tObjects[6][4][1] += "CREATE TABLE <table_name, sysname, test_table> (\n";
         tObjects[6][4][1] += "<column_1, sysname, c1> <datatype_for_column_1, , int> NOT NULL, \n";
         tObjects[6][4][1] += "<column_2, sysname, c2> AS <expression, , 2*c1>)\n";
         tObjects[6][4][1] += "GO\n";

		 tObjects[6][5][0] = "Create Table with IDENTITY Column";
         tObjects[6][5][1] = " =============================================\n";
         tObjects[6][5][1] += "-- Create table with IDENTITY column\n";
         tObjects[6][5][1] += "-- =============================================\n";
         tObjects[6][5][1] += "IF EXISTS (SELECT name \n";
         tObjects[6][5][1] += "	   FROM   sysobjects \n";
         tObjects[6][5][1] += "	   WHERE  name = N'<table_name, sysname, test_table>' \n";
         tObjects[6][5][1] += "	   AND 	  type = 'U')\n";
         tObjects[6][5][1] += "    DROP TABLE <table_name, sysname, test_table>\n";
         tObjects[6][5][1] += "GO\n";
         tObjects[6][5][1] += "\n";
         tObjects[6][5][1] += "create table <table_name, sysname, test_table> (\n";
         tObjects[6][5][1] += "<column_1, sysname, c1> <datatype_for_column_1, , int> IDENTITY(<seed, , 100>, <increment, , 1>), \n";
         tObjects[6][5][1] += "<column_2, sysname, c2> <datatype_for_column_2, , int> NOT NULL)\n";
         tObjects[6][5][1] += "GO\n";

		 tObjects[6][6][0] = "Create Table with ROWGUIDCOL Column";
         tObjects[6][6][1] = " =============================================\n";
         tObjects[6][6][1] += "-- Create table with ROWGUIDCOL column\n";
         tObjects[6][6][1] += "-- =============================================\n";
         tObjects[6][6][1] += "IF EXISTS (SELECT name \n";
         tObjects[6][6][1] += "	   FROM   sysobjects \n";
         tObjects[6][6][1] += "	   WHERE  name = N'<table_name, sysname, test_table>' \n";
         tObjects[6][6][1] += "	   AND 	  type = 'U')\n";
         tObjects[6][6][1] += "    DROP TABLE <table_name, sysname, test_table>\n";
         tObjects[6][6][1] += "GO\n";
         tObjects[6][6][1] += "\n";
         tObjects[6][6][1] += "create table <table_name, sysname, test_table> (\n";
         tObjects[6][6][1] += "<column_1, sysname, c1> uniqueidentifier ROWGUIDCOL \n";
         tObjects[6][6][1] += "	CONSTRAINT <constraint_name, sysname, const_c1> DEFAULT NEWID(), \n";
         tObjects[6][6][1] += "<column_2, sysname, c2> <datatype_for_column_2, , int> NOT NULL)\n";
         tObjects[6][6][1] += "GO\n";

		 tObjects[6][7][0] = "Create Temp Table";
         tObjects[6][7][1] = " =============================================\n";
         tObjects[6][7][1] += "-- Create temp table\n";
         tObjects[6][7][1] += "-- =============================================\n";
         tObjects[6][7][1] += "CREATE TABLE #<temp_table_name, sysname, test_table> (\n";
         tObjects[6][7][1] += "<column_1, sysname, c1> <datatype_for_column_1, , char> NULL, \n";
         tObjects[6][7][1] += "<column_2, sysname, c2> <datatype_for_column_2, , int> NOT NULL)\n";
         tObjects[6][7][1] += "GO\n";

	tObjects[7][0][0] = "04,Create Trigger";
		 tObjects[7][1][0] = "Create INSTEAD IF Trigger Basic Template";
			tObjects[7][1][1] = " =============================================\n";
			tObjects[7][1][1] += "-- Create basic Instead Of Trigger\n";
			tObjects[7][1][1] += "-- =============================================\n";
			tObjects[7][1][1] += "IF EXISTS (SELECT name \n";
			tObjects[7][1][1] += "	   FROM   sysobjects \n";
			tObjects[7][1][1] += "	   WHERE  name = N'<trigger_name, sysname, trig_test>' \n";
			tObjects[7][1][1] += "	   AND 	  type = 'TR')\n";
			tObjects[7][1][1] += "    DROP TRIGGER <trigger_name, sysname, trig_test>\n";
			tObjects[7][1][1] += "GO\n";
			tObjects[7][1][1] += "\n";
			tObjects[7][1][1] += "CREATE TRIGGER <trigger_name, sysname, trig_test>\n";
			tObjects[7][1][1] += "ON <table_or_view_name, sysname, pubs.dbo.sales>\n";
			tObjects[7][1][1] += "INSTEAD OF INSERT\n";
			tObjects[7][1][1] += "AS\n";
			tObjects[7][1][1] += "BEGIN\n";
			tObjects[7][1][1] += "	RAISERROR (50009, 16, 10)\n";
			tObjects[7][1][1] += "	EXEC sp_who\n";
			tObjects[7][1][1] += "END\n";
			tObjects[7][1][1] += "GO\n";

		 tObjects[7][2][0] = "Create Trigger Basic Template";
         tObjects[7][2][1] = " =============================================\n";
         tObjects[7][2][1] += "-- Create trigger basic template(After trigger)\n";
         tObjects[7][2][1] += "-- =============================================\n";
         tObjects[7][2][1] += "IF EXISTS (SELECT name \n";
         tObjects[7][2][1] += "	   FROM   sysobjects \n";
         tObjects[7][2][1] += "	   WHERE  name = N'<trigger_name, sysname, trig_test>' \n";
         tObjects[7][2][1] += "	   AND 	  type = 'TR')\n";
         tObjects[7][2][1] += "    DROP TRIGGER <trigger_name, sysname, trig_test>\n";
         tObjects[7][2][1] += "GO\n";
         tObjects[7][2][1] += "\n";
         tObjects[7][2][1] += "CREATE TRIGGER <trigger_name, sysname, trig_test>\n";
         tObjects[7][2][1] += "ON <table_name, sysname, pubs.dbo.sales>\n";
         tObjects[7][2][1] += "FOR DELETE, INSERT, UPDATE \n";
         tObjects[7][2][1] += "AS \n";
         tObjects[7][2][1] += "BEGIN\n";
         tObjects[7][2][1] += "	RAISERROR (50009, 16, 10)\n";
         tObjects[7][2][1] += "END\n";
         tObjects[7][2][1] += "GO\n";

		 tObjects[7][3][0] = "Create Trigger Contained IF COLUMNS_UPDATED";
         tObjects[7][3][1] = " =============================================\n";
         tObjects[7][3][1] += "-- Create trigger contained If (COLUMNS_UPDATED()) \n";
         tObjects[7][3][1] += "-- =============================================\n";
         tObjects[7][3][1] += "IF EXISTS (SELECT name \n";
         tObjects[7][3][1] += "	   FROM   sysobjects \n";
         tObjects[7][3][1] += "	   WHERE  name = N'<trigger_name, sysname, trig_test>' \n";
         tObjects[7][3][1] += "	   AND 	  type = 'TR')\n";
         tObjects[7][3][1] += "    DROP TRIGGER <trigger_name, sysname, trig_test>\n";
         tObjects[7][3][1] += "GO\n";
         tObjects[7][3][1] += "\n";
         tObjects[7][3][1] += "CREATE TRIGGER <trigger_name, sysname, trig_test>\n";
         tObjects[7][3][1] += "ON <table_or_view_name, sysname, pubs.dbo.sales>\n";
         tObjects[7][3][1] += "FOR INSERT, UPDATE \n";
         tObjects[7][3][1] += "AS\n";
         tObjects[7][3][1] += "--eg. check if all of column 2, 3, 4 are updated\n";
         tObjects[7][3][1] += "IF (COLUMNS_UPDATED() & 14) = 14\n";
         tObjects[7][3][1] += "BEGIN\n";
         tObjects[7][3][1] += "	RAISERROR ('These columns can not be updated at the same time', 16, 1)\n";
         tObjects[7][3][1] += "	ROLLBACK TRANSACTION\n";
         tObjects[7][3][1] += "END\n";
         tObjects[7][3][1] += "GO\n";

		 tObjects[7][4][0] = "Create Trigger Contained IF UPDATE";
         tObjects[7][4][1] = " =============================================\n";
         tObjects[7][4][1] += "-- Create trigger contained If UPDATE(column)\n";
         tObjects[7][4][1] += "-- =============================================\n";
         tObjects[7][4][1] += "IF EXISTS (SELECT name \n";
         tObjects[7][4][1] += "	   FROM   sysobjects \n";
         tObjects[7][4][1] += "	   WHERE  name = N'<trigger_name, sysname, trig_test>' \n";
         tObjects[7][4][1] += "	   AND 	  type = 'TR')\n";
         tObjects[7][4][1] += "    DROP TRIGGER <trigger_name, sysname, trig_test>\n";
         tObjects[7][4][1] += "GO\n";
         tObjects[7][4][1] += "\n";
         tObjects[7][4][1] += "CREATE TRIGGER <trigger_name, sysname, trig_test>\n";
         tObjects[7][4][1] += "ON <table_or_view_name, sysname, pubs.dbo.sales>\n";
         tObjects[7][4][1] += "FOR INSERT, UPDATE \n";
         tObjects[7][4][1] += "AS \n";
         tObjects[7][4][1] += "If UPDATE(<column_1, sysname, stor_id>) OR UPDATE(<column_2, sysname, ord_num>)\n";
         tObjects[7][4][1] += "BEGIN\n";
         tObjects[7][4][1] += "	RAISERROR ('These columns should never be updated', 16, 1)\n";
         tObjects[7][4][1] += "	ROLLBACK TRANSACTION\n";
         tObjects[7][4][1] += "END\n";
         tObjects[7][4][1] += "GO\n";

	tObjects[8][0][0] = "03,Create View";
		 tObjects[8][1][0] = "Create View Basic Template";
         tObjects[8][1][1] = " =============================================\n";
         tObjects[8][1][1] += "-- Create view basic template\n";
         tObjects[8][1][1] += "-- =============================================\n";
         tObjects[8][1][1] += "IF EXISTS (SELECT TABLE_NAME \n";
         tObjects[8][1][1] += "	   FROM   INFORMATION_SCHEMA.VIEWS \n";
         tObjects[8][1][1] += "	   WHERE  TABLE_NAME = N'<view_name, sysname, view_test>')\n";
         tObjects[8][1][1] += "    DROP VIEW <view_name, sysname, view_test>\n";
         tObjects[8][1][1] += "GO\n";
         tObjects[8][1][1] += "\n";
         tObjects[8][1][1] += "CREATE VIEW <view_name, sysname, view_test>\n";
         tObjects[8][1][1] += "AS \n";
         tObjects[8][1][1] += "	<select_statement, , SELECT * FROM authors>\n";
         tObjects[8][1][1] += "GO\n";

		 tObjects[8][2][0] = "Create View with CHECK OPTION";
         tObjects[8][2][1] = " =============================================\n";
         tObjects[8][2][1] += "-- Create view with CHECK OPTION \n";
         tObjects[8][2][1] += "-- =============================================\n";
         tObjects[8][2][1] += "IF EXISTS (SELECT TABLE_NAME \n";
         tObjects[8][2][1] += "	   FROM   INFORMATION_SCHEMA.VIEWS \n";
         tObjects[8][2][1] += "	   WHERE  TABLE_NAME = N'<view_name, sysname, view_test>')\n";
         tObjects[8][2][1] += "    DROP VIEW <view_name, sysname, view_test>\n";
         tObjects[8][2][1] += "GO\n";
         tObjects[8][2][1] += "\n";
         tObjects[8][2][1] += "CREATE VIEW <view_name, sysname, view_test>\n";
         tObjects[8][2][1] += "AS \n";
         tObjects[8][2][1] += "	<select_statement, , SELECT * FROM authors>\n";
         tObjects[8][2][1] += "WITH CHECK OPTION\n";
         tObjects[8][2][1] += "GO\n";

		 tObjects[8][3][0] = "Create View with SCHEMABINDING";
         tObjects[8][3][1] = " =============================================\n";
         tObjects[8][3][1] += "-- Create schemabinding view\n";
         tObjects[8][3][1] += "-- =============================================\n";
         tObjects[8][3][1] += "IF EXISTS (SELECT TABLE_NAME \n";
         tObjects[8][3][1] += "	   FROM   INFORMATION_SCHEMA.VIEWS \n";
         tObjects[8][3][1] += "	   WHERE  TABLE_NAME = N'<view_name, sysname, view_test>')\n";
         tObjects[8][3][1] += "    DROP VIEW <view_name, sysname, view_test>\n";
         tObjects[8][3][1] += "GO\n";
         tObjects[8][3][1] += "\n";
         tObjects[8][3][1] += "CREATE VIEW <view_name, sysname, view_test> WITH SCHEMABINDING\n";
         tObjects[8][3][1] += "AS \n";
         tObjects[8][3][1] += "<select_statement, , SELECT au_id FROM dbo.authors>\n";
         tObjects[8][3][1] += "--note: need to specify specific column names and owner of the table \n";
         tObjects[8][3][1] += "--eg. SELECT column_1, column_2 FROM owner.table_or_view_name WHERE search_condition\n";
         tObjects[8][3][1] += "GO\n";

	tObjects[9][0][0] = "04,Manage Extended Property";
		 tObjects[9][1][0] = "Add Extended Property";
			tObjects[9][1][1] = "-- =============================================\n";
			tObjects[9][1][1] += "-- Add extended property \n";
			tObjects[9][1][1] += "-- =============================================\n";
			tObjects[9][1][1] += "EXEC sp_addextendedproperty N'<property_name, sysname, n1>', <property_value, sql_variant, 8>,  \n";
			tObjects[9][1][1] += "			    N'<level0type, , user>', N'<level0name, sysname, dbo>', \n";
			tObjects[9][1][1] += "			    N'<level1type, , table>', N'<level1name, sysname, authors>', \n";
			tObjects[9][1][1] += "			    N'<level2type, , column>', N'<level2name, sysname, au_id>'\n";
			tObjects[9][1][1] += "GO\n";

		 tObjects[9][2][0] = "Drop Extended Property";
         tObjects[9][2][1] = "-- =============================================\n";
         tObjects[9][2][1] += "-- Drop extended property \n";
         tObjects[9][2][1] += "-- =============================================\n";
         tObjects[9][2][1] += "EXEC sp_dropextendedproperty N'<property_name, sysname, n1>', \n";
         tObjects[9][2][1] += "			     N'<level0type, , user>', N'<level0name, sysname, dbo>', \n";
         tObjects[9][2][1] += "			     N'<level1type, , table>', N'<level1name, sysname, authors>', \n";
         tObjects[9][2][1] += "			     N'<level2type, , column>', N'<level2name, sysname, au_id>'\n";
         tObjects[9][2][1] += "GO\n";

		 tObjects[9][3][0] = "List Extended Property";
         tObjects[9][3][1] = "-- =============================================\n";
         tObjects[9][3][1] += "-- List extended property \n";
         tObjects[9][3][1] += "-- =============================================\n";
         tObjects[9][3][1] += "SELECT * FROM ::fn_listextendedproperty(\n";
         tObjects[9][3][1] += "	N'<property_name, sysname, n1>', \n";
         tObjects[9][3][1] += "	N'<level0type, , user>', N'<level0name, sysname, dbo>', \n";
         tObjects[9][3][1] += "	N'<level1type, , table>', N'<level1name, sysname, authors>', \n";
         tObjects[9][3][1] += "	N'<level2type, , column>', N'<level2name, sysname, au_id>')\n";
         tObjects[9][3][1] += "GO\n";

		 tObjects[9][4][0] = "Update Extended Property";
			tObjects[9][4][1] = "-- =============================================\n";
			tObjects[9][4][1] += "-- Update extended property \n";
			tObjects[9][4][1] += "-- =============================================\n";
			tObjects[9][4][1] += "EXEC sp_updateextendedproperty N'<property_name, sysname, n1>', <property_value, sql_variant, 9>, \n";
			tObjects[9][4][1] += "			       N'<level0type, , user>', N'<level0name, sysname, dbo>', \n";
			tObjects[9][4][1] += "			       N'<level1type, , table>', N'<level1name, sysname, authors>', \n";
			tObjects[9][4][1] += "			       N'<level2type, , column>', N'<level2name, sysname, au_id>'\n";
			tObjects[9][4][1] += "GO\n";

	tObjects[10][0][0] = "06,Manage Linked Server";
		 tObjects[10][1][0] = "Add Linked Server Login";
			tObjects[10][1][1] = "-- =============================================\n";
			tObjects[10][1][1] += "-- Add linked server login\n";
			tObjects[10][1][1] += "-- =============================================\n";
			tObjects[10][1][1] += "sp_addlinkedsrvlogin @rmtsrvname  = N'<server_name, sysname, server1>',\n";
			tObjects[10][1][1] += "    		     @useself 	  = '<useself, varchar(8), TRUE>',\n";
			tObjects[10][1][1] += "   		     @locallogin  = <locallogin, sysname, NULL>,\n";
			tObjects[10][1][1] += " 		     @rmtuser     = <rmtuser, sysname, NULL>,\n";
			tObjects[10][1][1] += "  		     @rmtpassword = <rmtpassword, sysname, NULL>\n";
			tObjects[10][1][1] += "GO\n";

		 tObjects[10][2][0] = "Add non-Sql Server Login";
         tObjects[10][2][1] = " =============================================\n";
         tObjects[10][2][1] += "-- Add another non-Sql Server as linked server\n";
         tObjects[10][2][1] += "-- =============================================\n";
         tObjects[10][2][1] += "sp_addlinkedserver @server     = N'<server_name, sysname, server1>', \n";
         tObjects[10][2][1] += "		   @srvproduct = N'<product_name, nvarchar(128), Microsoft.Jet.OLEDB.4.0>',\n";
         tObjects[10][2][1] += "   		   @provider   = N'<provider_name, nvarchar(128), OLE DB Provider for Jet>',\n";
         tObjects[10][2][1] += "		   @datasrc    = N'<data_source, nvarchar(4000), C:\\MSOffice\\Access\\Samples\\Northwind.mdb>', \n";
         tObjects[10][2][1] += "		   @location   = <location, nvarchar(4000), NULL>, \n";
         tObjects[10][2][1] += "		   @provstr    = <provider_string, nvarchar(4000), NULL>,\n";
         tObjects[10][2][1] += "		   @catalog    = <catalog, sysname, NULL>\n";
         tObjects[10][2][1] += "GO\n";
         tObjects[10][2][1] += "\n";
         tObjects[10][2][1] += "-- =============================================\n";
         tObjects[10][2][1] += "-- List linked server\n";
         tObjects[10][2][1] += "-- =============================================\n";
         tObjects[10][2][1] += "sp_linkedservers\n";
         tObjects[10][2][1] += "GO\n";

		 tObjects[10][3][0] = "Add Sql Server as Linked Server";
			tObjects[10][3][1] = "-- =============================================\n";
			tObjects[10][3][1] += "-- Add another Sql Server as linked server\n";
			tObjects[10][3][1] += "-- =============================================\n";
			tObjects[10][3][1] += "sp_addlinkedserver N'<server_name, sysname, server1>', N'SQL Server'\n";
			tObjects[10][3][1] += "GO\n";
			tObjects[10][3][1] += "\n";
			tObjects[10][3][1] += "-- =============================================\n";
			tObjects[10][3][1] += "-- List linked server\n";
			tObjects[10][3][1] += "-- =============================================\n";
			tObjects[10][3][1] += "sp_linkedservers\n";
			tObjects[10][3][1] += "GO\n";

		 tObjects[10][4][0] = "Configure Linked Server";
			tObjects[10][4][1] = "-- =============================================\n";
			tObjects[10][4][1] += "-- Configure linked server \n";
			tObjects[10][4][1] += "-- =============================================\n";
			tObjects[10][4][1] += "sp_serveroption @server   = N'<server_name, sysname, server1>', \n";
			tObjects[10][4][1] += "    		@optname  = '<option_name, varchar(35), dist>',\n";
			tObjects[10][4][1] += "    		@optvalue = N'<option_value, nvarchar(128), OFF>'\n";
			tObjects[10][4][1] += "GO\n";

		 tObjects[10][5][0] = "Drop Linked Server Login";
         tObjects[10][5][1] = "-- =============================================\n";
         tObjects[10][5][1] += "-- Drop linked server login\n";
         tObjects[10][5][1] += "-- =============================================\n";
         tObjects[10][5][1] += "sp_droplinkedsrvlogin @rmtsrvname = N'<server_name, sysname, server1>',\n";
         tObjects[10][5][1] += "		      @locallogin = N'<locallogin, sysname, test_login>'\n";
         tObjects[10][5][1] += "GO\n";

		 tObjects[10][6][0] = "Drop Linked Server";
         tObjects[10][6][1] = "-- =============================================\n";
         tObjects[10][6][1] += "-- Drop linked server\n";
         tObjects[10][6][1] += "-- =============================================\n";
         tObjects[10][6][1] += "sp_dropserver @server     = N'<server_name, sysname, server1>', \n";
         tObjects[10][6][1] += "	      @droplogins = <option, char(10), NULL>\n";
         tObjects[10][6][1] += "GO\n";
         tObjects[10][6][1] += "\n";
         tObjects[10][6][1] += "-- =============================================\n";
         tObjects[10][6][1] += "-- List linked server\n";
         tObjects[10][6][1] += "-- =============================================\n";
         tObjects[10][6][1] += "sp_linkedservers\n";
         tObjects[10][6][1] += "GO\n";

	tObjects[11][0][0] = "11,Manage Login Role User";
		 tObjects[11][1][0] = "Add Login to Server Role";
			tObjects[11][1][1] = "-- =============================================\n";
			tObjects[11][1][1] += "-- Add login to server roles\n";
			tObjects[11][1][1] += "-- =============================================\n";
			tObjects[11][1][1] += "sp_addsrvrolemember @loginame = N'<Windows_or_Sql_Server_login, sysname, REDMOND\\john>', \n";
			tObjects[11][1][1] += "		    @rolename = N'<server_role, sysname, sysadmin>'\n";
			tObjects[11][1][1] += "GO\n";

		 tObjects[11][2][0] = "Add Sql Server Login";
			tObjects[11][2][1] = "-- =============================================\n";
			tObjects[11][2][1] += "-- Add Sql Server login\n";
			tObjects[11][2][1] += "-- =============================================\n";
			tObjects[11][2][1] += "sp_addlogin @loginame    = N'<Sql_Server_login, sysname, test_login>',\n";
			tObjects[11][2][1] += "	    @passwd      = '<password, sysname, 123#$>',\n";
			tObjects[11][2][1] += "	    @defdb       = N'<database, sysname, pubs>',\n";
			tObjects[11][2][1] += "	    @deflanguage = N'<language, sysname, German>',\n";
			tObjects[11][2][1] += "	    @sid         = <sid, varbinary(16), NULL>,\n";
			tObjects[11][2][1] += "	    @encryptopt  = <encryption_option, varchar(20), NULL>\n";
			tObjects[11][2][1] += "GO\n";

		 tObjects[11][3][0] = "Add User to Database Role";
			tObjects[11][3][1] = "-- =============================================\n";
			tObjects[11][3][1] += "-- Add user to database role\n";
			tObjects[11][3][1] += "-- =============================================\n";
			tObjects[11][3][1] += "sp_addrolemember @rolename = N'<db_role, sysname, db_owner>', \n";
			tObjects[11][3][1] += "		 @membername = N'<db_user_name, sysname, test_user>'\n";
			tObjects[11][3][1] += "GO\n";

		 tObjects[11][4][0] = "Deny Sql Server Access to Windows User Group";
         tObjects[11][4][1] = "-- =============================================\n";
         tObjects[11][4][1] += "-- Deny Sql Server access to Windows user or group\n";
         tObjects[11][4][1] += "-- =============================================\n";
         tObjects[11][4][1] += "sp_denylogin @loginame = N'<Windows_user_or_group, sysname, REDMOND\\john>'\n";
         tObjects[11][4][1] += "GO\n";

		 tObjects[11][5][0] = "Drop Login from Server Role";
         tObjects[11][5][1] = "-- =============================================\n";
         tObjects[11][5][1] += "-- Drop login from server role\n";
         tObjects[11][5][1] += "-- =============================================\n";
         tObjects[11][5][1] += "sp_dropsrvrolemember @loginame = N'<Windows_or_Sql_Server_login, sysname, REDMOND\\john>', \n";
         tObjects[11][5][1] += "		     @rolename = N'<server_role, sysname, sysadmin>'\n";
         tObjects[11][5][1] += "GO\n";

		 tObjects[11][6][0] = "Drop Sql Server Login";
         tObjects[11][6][1] = "-- =============================================";
         tObjects[11][6][1] += "-- Drop Sql Server login";
         tObjects[11][6][1] += "-- =============================================";
         tObjects[11][6][1] += "sp_droplogin @loginame = N'<Sql_Server_login, sysname, test_login>'";
         tObjects[11][6][1] += "GO";

		 tObjects[11][7][0] = "Drop User from Database Role";
          tObjects[11][7][1] = "-- =============================================\n";
          tObjects[11][7][1] += "-- Drop user from database role\n";
          tObjects[11][7][1] += "-- =============================================\n";
          tObjects[11][7][1] += "sp_droprolemember @rolename   = N'<db_role, sysname, db_owner>', \n";
          tObjects[11][7][1] += "		  @membername = N'<db_user_name, sysname, johnny>'\n";
          tObjects[11][7][1] += "GO\n";

		 tObjects[11][8][0] = "Grant Database Access to Windows User or Sql Server Login";
         tObjects[11][8][1] = "-- =============================================\n";
         tObjects[11][8][1] += "-- Grant dbaccess to login (Windows or Sql Server login)\n";
         tObjects[11][8][1] += "-- =============================================\n";
         tObjects[11][8][1] += "sp_grantdbaccess @loginame   = N'<Windows_or_Sql_Server_login, sysname, REDMOND\\john>', \n";
         tObjects[11][8][1] += "		 @name_in_db = N'<db_user_name, sysname, johnny>'\n";
         tObjects[11][8][1] += "GO\n";

		 tObjects[11][9][0] = "Grant Sql Server Access to Windows User or Group";
         tObjects[11][9][1] = "-- =============================================\n";
         tObjects[11][9][1] += "-- Grant Sql Server access to Windows user or group\n";
         tObjects[11][9][1] += "-- =============================================\n";
         tObjects[11][9][1] += "sp_grantlogin @loginame = N'<Windows_user_or_group, sysname, REDMOND\\john>'\n";
         tObjects[11][9][1] += "GO\n";

		 tObjects[11][10][0] = "Revoke Database Access From Windows User or Sql Server Login";
         tObjects[11][10][1] = "-- =============================================\n";
         tObjects[11][10][1] += "-- Revoke dbaccess from User (User name in current Database, Windows User, or Sql Server login)\n";
         tObjects[11][10][1] += "-- =============================================\n";
         tObjects[11][10][1] += "sp_revokedbaccess @name_in_db = N'<db_user_name, sysname, test_user>'\n";
         tObjects[11][10][1] += "GO\n";

		 tObjects[11][11][0] = "Revoke Sql Server Access from Windows User or Group";
         tObjects[11][11][1] = "-- =============================================\n";
         tObjects[11][11][1] += "-- Revoke Sql Server access to Windows user or group\n";
         tObjects[11][11][1] += "-- =============================================\n";
         tObjects[11][11][1] += "sp_revokelogin @loginame = N'<Windows_user_or_group, sysname, REDMOND\\john>'\n";
         tObjects[11][11][1] += "GO\n";

	tObjects[12][0][0] = "04,Using Cursor";
		 tObjects[12][1][0] = "Declare and Use KEYSET Cursor";
         tObjects[12][1][1] = " =============================================\n";
         tObjects[12][1][1] += "-- Declare and using a KEYSET cursor\n";
         tObjects[12][1][1] += "-- =============================================\n";
         tObjects[12][1][1] += "DECLARE <cursor_name, sysname, test_cursor> CURSOR\n";
         tObjects[12][1][1] += "KEYSET\n";
         tObjects[12][1][1] += "FOR <select_statement, , SELECT au_fname FROM pubs.dbo.authors>\n";
         tObjects[12][1][1] += "\n";
         tObjects[12][1][1] += "DECLARE @name varchar(40)\n";
         tObjects[12][1][1] += "\n";
         tObjects[12][1][1] += "OPEN <cursor_name, sysname, test_cursor>\n";
         tObjects[12][1][1] += "\n";
         tObjects[12][1][1] += "FETCH NEXT FROM <cursor_name, sysname, test_cursor> INTO @name\n";
         tObjects[12][1][1] += "WHILE (@@fetch_status <> -1)\n";
         tObjects[12][1][1] += "BEGIN\n";
         tObjects[12][1][1] += "	IF (@@fetch_status <> -2)\n";
         tObjects[12][1][1] += "	BEGIN\n";
         tObjects[12][1][1] += "--		PRINT 'add user defined code here' \n";
         tObjects[12][1][1] += "--		eg.\n";
         tObjects[12][1][1] += "		PRINT 'updating record for ' + @name\n";
         tObjects[12][1][1] += "		UPDATE pubs.dbo.authors \n";
         tObjects[12][1][1] += "		SET phone = replace(phone, ' ', '-')\n";
         tObjects[12][1][1] += "		WHERE CURRENT OF <cursor_name, sysname, test_cursor>\n";
         tObjects[12][1][1] += "	END\n";
         tObjects[12][1][1] += "	FETCH NEXT FROM <cursor_name, sysname, test_cursor> INTO @name\n";
         tObjects[12][1][1] += "END\n";
         tObjects[12][1][1] += "\n";
         tObjects[12][1][1] += "CLOSE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][1][1] += "DEALLOCATE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][1][1] += "GO\n";

		 tObjects[12][2][0] = "Declare and Use Read_ONLY Cursor";
         tObjects[12][2][1] = " =============================================\n";
         tObjects[12][2][1] += "-- Declare and using a READ_ONLY cursor\n";
         tObjects[12][2][1] += "-- =============================================\n";
         tObjects[12][2][1] += "DECLARE <cursor_name, sysname, test_cursor> CURSOR\n";
         tObjects[12][2][1] += "READ_ONLY\n";
         tObjects[12][2][1] += "FOR <select_statement, , SELECT au_fname FROM pubs.dbo.authors>\n";
         tObjects[12][2][1] += "\n";
         tObjects[12][2][1] += "DECLARE @name varchar(40)\n";
         tObjects[12][2][1] += "OPEN <cursor_name, sysname, test_cursor>\n";
         tObjects[12][2][1] += "\n";
         tObjects[12][2][1] += "FETCH NEXT FROM <cursor_name, sysname, test_cursor> INTO @name\n";
         tObjects[12][2][1] += "WHILE (@@fetch_status <> -1)\n";
         tObjects[12][2][1] += "BEGIN\n";
         tObjects[12][2][1] += "	IF (@@fetch_status <> -2)\n";
         tObjects[12][2][1] += "	BEGIN\n";
         tObjects[12][2][1] += "--		PRINT 'add user defined code here'\n";
         tObjects[12][2][1] += "--		eg.\n";
         tObjects[12][2][1] += "		DECLARE @message varchar(100)\n";
         tObjects[12][2][1] += "		SELECT @message = 'my name is: ' + @name\n";
         tObjects[12][2][1] += "		PRINT @message\n";
         tObjects[12][2][1] += "	END\n";
         tObjects[12][2][1] += "	FETCH NEXT FROM <cursor_name, sysname, test_cursor> INTO @name\n";
         tObjects[12][2][1] += "END\n";
         tObjects[12][2][1] += "\n";
         tObjects[12][2][1] += "CLOSE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][2][1] += "DEALLOCATE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][2][1] += "GO\n";

		 tObjects[12][3][0] = "Declare and Use SCROLL Cursor with Various FETCH Options";
         tObjects[12][3][1] = " =============================================\n";
         tObjects[12][3][1] += "-- Declare a SCROLL cursor and use various FETCH options\n";
         tObjects[12][3][1] += "-- =============================================\n";
         tObjects[12][3][1] += "DECLARE <cursor_name, sysname, test_cursor> SCROLL CURSOR FOR\n";
         tObjects[12][3][1] += "<select_statement, , SELECT * FROM pubs.dbo.authors>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "OPEN <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "-- Fetch the first row in the cursor.\n";
         tObjects[12][3][1] += "FETCH FIRST FROM <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "-- Fetch the last row in the cursor.\n";
         tObjects[12][3][1] += "FETCH LAST FROM <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "-- Fetch the row immediately prior to the current row in the cursor.\n";
         tObjects[12][3][1] += "FETCH PRIOR FROM <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "-- Fetch the row immediately after the current row in the cursor.\n";
         tObjects[12][3][1] += "FETCH NEXT FROM <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "-- Fetch the nth row in the cursor.\n";
         tObjects[12][3][1] += "FETCH ABSOLUTE <row_number, int, 8> FROM <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "-- Fetch the row that is n rows before or after the current row.\n";
         tObjects[12][3][1] += "FETCH RELATIVE <-/+ row_number, int, -2> FROM <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "\n";
         tObjects[12][3][1] += "CLOSE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "DEALLOCATE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][3][1] += "GO\n";

		 tObjects[12][4][0] = "Declare and Use UPDATE Cursor";
         tObjects[12][4][1] = "-- =============================================\n";
         tObjects[12][4][1] += "-- Declare and using an UPDATE cursor\n";
         tObjects[12][4][1] += "-- =============================================\n";
         tObjects[12][4][1] += "DECLARE <@variable_1, sysname, @v1> <datatype_for_variable_1, sysname, varchar(20)>, \n";
         tObjects[12][4][1] += "	<@variable_2, sysname, @v2> <datatype_for_variable_2, sysname, varchar(40)>\n";
         tObjects[12][4][1] += "\n";
         tObjects[12][4][1] += "DECLARE	<cursor_name, sysname, test_cursor> CURSOR \n";
         tObjects[12][4][1] += "FOR SELECT <column_1, sysname, au_fname>, <column_2, sysname, au_lname> FROM <table_name, sysname, pubs.dbo.authors>\n";
         tObjects[12][4][1] += "FOR UPDATE of <column_1, sysname, au_fname>\n";
         tObjects[12][4][1] += "\n";
         tObjects[12][4][1] += "DECLARE @count smallint\n";
         tObjects[12][4][1] += "SELECT @count = 1\n";
         tObjects[12][4][1] += "\n";
         tObjects[12][4][1] += "OPEN <cursor_name, sysname, test_cursor>\n";
         tObjects[12][4][1] += "FETCH NEXT FROM <cursor_name, sysname, test_cursor> INTO <@variable_1, sysname, @v1>, <@variable_2, sysname, @v2>\n";
         tObjects[12][4][1] += "\n";
         tObjects[12][4][1] += "WHILE (@@fetch_status <> -1)\n";
         tObjects[12][4][1] += "BEGIN\n";
         tObjects[12][4][1] += "	IF (@@fetch_status <> -2)\n";
         tObjects[12][4][1] += "	BEGIN\n";
         tObjects[12][4][1] += "--		PRINT	'add user-defined code here...'\n";
         tObjects[12][4][1] += "--		eg\n";
         tObjects[12][4][1] += "		PRINT 'updating record of ' + @v1 + ' ' + @v2\n";
         tObjects[12][4][1] += "		UPDATE pubs.dbo.authors\n";
         tObjects[12][4][1] += "		SET au_fname = @v1 + '-' + CAST(@count AS varchar(4))\n";
         tObjects[12][4][1] += "		WHERE au_lname = @v2\n";
         tObjects[12][4][1] += "	END\n";
         tObjects[12][4][1] += "	FETCH NEXT FROM <cursor_name, sysname, test_cursor> INTO <@variable_1, sysname, @v1>, <@variable_2, sysname, @v2>\n";
         tObjects[12][4][1] += "	SELECT @count = @count + 1\n";
         tObjects[12][4][1] += "END\n";
         tObjects[12][4][1] += "\n";
         tObjects[12][4][1] += "CLOSE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][4][1] += "DEALLOCATE <cursor_name, sysname, test_cursor>\n";
         tObjects[12][4][1] += "GO\n";

	QListViewItem *templateItem = new QListViewItem(templateWidg);
	templateItem->setPixmap(0, QPixmap((const char**)folder_pix_xpm));
	templateItem->setText(0, i18n("Templates"));

	for ( int i=12 ; i >= 0 ; i-- ) {
		QListViewItem *tChildItem = new QListViewItem(templateItem);
		tChildItem->setPixmap(0, QPixmap((const char**)folder_pix_xpm));
		tChildItem->setExpandable(TRUE);

		tmp = tObjects[i][0][0];
		t_func = tmp.left(2).toInt();
		tmp = tmp.right(tmp.length()-3);

		tChildItem->setText(0, tmp);
		for ( int f=t_func ; f > 0 ; f-- ) {
			tmp = tObjects[i][f][0];
			QListViewItem *toFuncItem = new QListViewItem(tChildItem);
			toFuncItem->setPixmap(0,QPixmap((const char**)templates_xpm));
			toFuncItem->setText(0,tmp);
			toFuncItem->setText(4, tObjects[i][f][1]);
			toFuncItem->setDragEnabled(TRUE);
		}
	}
	templateItem->setOpen(TRUE);
}

void MyAppBar::refreshView()
{
debug("REFRESH");
	buildList(objectBox->currentItem());
}
