/***************************************************************************
                          tsqlcont.cpp  -  description
                             -------------------
    begin                : Sat Feb 1 2003
    copyright            : (C) 2003 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tsqlcont.h"

int tsql_handle_message(TDSCONTEXT *context, TDSSOCKET *tds, TDSMSGINFO *msg)
{
	return 1;
}

int tsql_handle_error(TDSCONTEXT *context, TDSSOCKET *tds, TDSMSGINFO *msg)
{
	return 1;
}

TSQLCont::TSQLCont()
				: QObject(0,"TSQLCont")
{
	// No login-information is avaible at this moment
	login_ok = FALSE;
	breaked=FALSE;

	// No error, yet
	old_msg = "";
	old_err = "";

	// Get login structure
	login = (TDSLOGIN *) tds_alloc_login();

	context = tds_alloc_context();
	if( context->locale && !context->locale->date_fmt ) {
		/* set default in case there's no locale file */
		context->locale->date_fmt = strdup("%b %e %Y %l:%M%p");
	}

	// Initialize an empty Error-Handler
	context->msg_handler = tsql_handle_message;
	context->err_handler = tsql_handle_error;
}

TSQLCont::~TSQLCont()
{
}

bool TSQLCont::tsql_login(const QString &hostname, int port, const QString &username, const QString &password)
{
	tds_set_user(login, const_cast <char *>( (const char*)username));
	tds_set_app(login, "TSQL");
	tds_set_host(login, "myhost");
	tds_set_library(login, "TDS-Library");
	tds_set_server(login, const_cast <char *>( (const char*)hostname));
	tds_set_port(login, port);
	tds_set_charset(login, "iso_1");
	tds_set_language(login, "us_english");
	tds_set_packet(login, 4096);
	tds_set_passwd(login, const_cast <char *>( (const char*)password));
	login_ok = TRUE;
	return TRUE;
}

bool TSQLCont::tsql_connect()
{
	if ( !login_ok ) {
		emit tsql_fatal(I18N_NOOP("No login information avaible!"));
		return FALSE;
	} else {
		tds = tds_connect(login, context, NULL);
		if (!tds) {
			emit tsql_fatal(I18N_NOOP("Connection error !"));
			return FALSE;
		}
	}
	startTimer(50);
	return TRUE;
}

/*
 * tsql_doQuery()
 * Submit a query to the DB-Server and get the Results back.
 * This functions emit 2 Signals:
 * columnsReceived(int, QStringList &)
 *   Number of columns, QStringList with column-Names
 * rowReceived(int, QStringList &)
 *   Number of columns in row, QStringList with the fields
*/
bool TSQLCont::tsql_doQuery(const QString &query)
{
	break_run = FALSE;
	QTime time;
	time.start();
	QValueList<QStringList> all_rows[2481];
	int rows, num_columns;

	QStringList columns[2481], row;
	QStringList tableInfo;
	QString rec;
	QString tmp_src;
	int i, ctype, tab_cnt, rows_cnt, cancelled, marker;
	TDS_INT srclen;
	unsigned char *src;
	TDSCOLINFO *col;
	CONV_RESULT dres;
	bool is_bin = FALSE;

	rows = num_columns = tab_cnt = rows_cnt = 0;

	rows = 0;
	if ( tds_submit_query(tds,const_cast <char *>( (const char*)query)) != TDS_SUCCEED ) {
		emit tsql_fatal(I18N_NOOP("tds_submit_query() failed\n"));
		emit tsql_queryReady(0,0);
		return FALSE;
	}

	while ((tds_process_result_tokens(tds))==TDS_SUCCEED) {
		qApp->processEvents();
		if ( break_run ) {
			continue;
/*
			// tds_send_canel()
			tds->out_flag=0x06;
			tds_flush_packet(tds);

			// tds_process_canel()
			do {
				marker=tds_get_byte(tds);
				if (marker==TDS_DONE_TOKEN) {
					tds_process_end(tds, marker, NULL, &cancelled);
				} else if (marker==0) {
					cancelled = 1;
				} else {
					tds_process_default_tokens(tds,marker);
				}
			} while (!cancelled);
			tds->state = 0;
			tds->res_info = 0;
			tds->rows_affected = 0;
			break_run = FALSE;
			continue;
*/
		}
		if ( tab_cnt > 2480 ) {
			continue;
		}

		if (tds->res_info) {
			num_columns = tds->res_info->num_cols;
			for (i=0; i<tds->res_info->num_cols; i++) {
				rec = QString("%1").arg(tds->res_info->columns[i]->column_name);
					columns[tab_cnt].append(rec);
			}
		}

		while ((tds_process_row_tokens(tds))==TDS_SUCCEED) {
			if ( break_run ) {
				continue;
/*
				// tds_send_canel()
		        tds->out_flag=0x06;
		        tds_flush_packet(tds);

				// tds_process_canel()
				while ( !cancelled )  {
					marker=tds_get_byte(tds);
					if (marker==TDS_DONE_TOKEN) {
						tds_process_end(tds, marker, NULL, &cancelled);
					} else if (marker==0) {
						cancelled = 1;
					} else {
						tds_process_default_tokens(tds,marker);
					}
				}

tds->res_info->rows_exist = 0;
//tds->res_info->row_count = 0;
//tds->res_info->row_size = 0;
//tds->res_info->num_cols = 0;
//tds->res_info->more_results = 0;
//tds->res_info->null_info_size = 0;
//tds->res_info->current_row = '0';
				break_run = FALSE;
continue;
*/
			}

			rows_cnt++;
			if (!tds->res_info)
				continue;

			for (i=0; i<tds->res_info->num_cols; i++) {
				is_bin = FALSE;
				if (tds_get_null(tds->res_info->current_row, i)) {
					rec.sprintf(" NULL ");
					row.append(rec);
					continue;
				}
				col = tds->res_info->columns[i];
				ctype = tds_get_conversion_type(col->column_type, col->column_size);

				if (is_blob_type(col->column_type)) {
					src = (unsigned char *)col->column_textvalue;
					srclen = col->column_textsize;
/*debug("%d", srclen);
QWidget *tst = new QWidget(0L);
tst->resize(200,300);
QLabel *tstLab = new QLabel(tst);
tstLab->setGeometry(0,0,200,300);
QPixmap px;
bool lfd = px.loadFromData(src, srclen);
debug("%d", lfd);
tstLab->setPixmap(px);
tst->show();
*/
					is_bin = TRUE;
				} else {
					src = &(tds->res_info->current_row[col->column_offset]);
					srclen = col->cur_row_size;
				}

				if(tds_convert(tds->tds_ctx,
												ctype,
												reinterpret_cast <TDS_CHAR *>(src),
												srclen,
												SYBVARCHAR,
												255,
												&dres) == TDS_FAIL)
					continue;
				rec = QString("%1").arg(dres.c);

				row.append(rec.latin1());
				qApp->processEvents();
			}
			// An der Stelle habe ich die Records fr 1 Reihe in 'records' und die Namen der Coloumns
			// in Columns.
			rows++;
			all_rows[tab_cnt].append(row);
			row.clear();
		}
		tmp_src.sprintf("%d_%d", rows, num_columns);
		tableInfo.append(tmp_src);
		rows = num_columns = 0;
		tab_cnt++;
	}

	if ( !tab_cnt ) {
		if (tds->res_info) {
			num_columns = tds->res_info->num_cols;
			for (i=0; i<tds->res_info->num_cols; i++) {
				rec = QString("%1").arg(tds->res_info->columns[i]->column_name);
				columns[tab_cnt].append(rec);
			}
			tmp_src.sprintf("%d_%d", 0, tds->res_info->num_cols);
			tableInfo.append(tmp_src);
			emit tsql_tableinfo_received(tableInfo);

			emit tsql_querycolumns_received(0, columns[0]);
			columns[0].clear();

			// Datenreihen
			emit tsql_queryrow_received(all_rows[0]);
			all_rows[0].clear();

			emit tsql_queryReady(rows_cnt, time.elapsed());
			break_run = FALSE;
			return TRUE;
		}
	}

	// Tabellen-Metainformationen bertragen
	emit tsql_tableinfo_received(tableInfo);

	// Tabellen-Daten bertragen
	for ( int i=0 ; i < tab_cnt ; i++ ) {
		// Namen der Columns
		emit tsql_querycolumns_received(i+1, columns[i]);

		columns[i].clear();

		// Datenreihen
		emit tsql_queryrow_received(all_rows[i]);
		all_rows[i].clear();
	}

	// Query-Abschluss besttigen
	emit tsql_queryReady(rows_cnt, time.elapsed());
	break_run = FALSE;

	return TRUE;
}

void TSQLCont::timerEvent(QTimerEvent *te)
{
	msg = tds->msg_info;
	sql_err = "";
	sql_msg = "";

	if( msg->priv_msg_type==1 && msg->msg_number > 0  && msg->msg_number != 5701) {
		// Error Messages
		sql_err.sprintf("FATAL: Msg %d, Level %d, State %d, Server %s, Line %d\n%s\n",
		msg->msg_number,
		msg->msg_level,
		msg->msg_state,
		msg->server,
		msg->line_number,
		msg->message);
		emit tsql_error(sql_err);
		tds->msg_info->msg_number = 0;
		tds->msg_info->msg_state = 0;
		sql_err = "";
	} else 	if( msg->msg_state==1 && msg->msg_number != 5701 ) {
		// Normal messages
		sql_msg = QString("%1").arg(msg->message);
		emit tsql_message(sql_msg);
		tds->msg_info->msg_number = 0;
		tds->msg_info->msg_state = 0;
		sql_msg = "";
	} else 	if( msg->priv_msg_type!=1 && msg->msg_number > 0   && msg->msg_number == 5701) {
		// Databases changes
		sql_msg = QString("%1").arg(msg->message);
		int start, end;
		start = sql_msg.find('\'', 0);
		end = sql_msg.find('\'', start+1);
		sql_msg = sql_msg.mid(start+1, (end-start)-1);
		emit tsql_dbchange(sql_msg);
		tds->msg_info->msg_number = 0;
		tds->msg_info->msg_state = 0;
	}
}

int TSQLCont::getData(const QString &txt, QPixmap *px)
{
	// Try to load data as Image
	CONV_RESULT dres;
	unsigned char * src;
	int srclen = ((txt.length()/2)-1);
	int type;
	bool loaded;

	type = tds_convert(tds->tds_ctx, SYBVARCHAR, const_cast <char *>( (const char*)txt), txt.length(), SYBIMAGE, 255,&dres);
	src = (unsigned char*)dres.c;

	loaded = px->loadFromData(src, srclen);

	if ( loaded )
		type = 2;
	else if ( !type )
		type = 1;
	else
		type = 3;

	return type;
}
